//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: connection.cc,v 1.4 1996/03/03 03:57:43 smp Exp smp $";


//
// connection.c
//

#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

#include "util/misc/misc.hh"
#include "connection.hh"


///////////////////////////////////////////////////////////////////////////////
// static:
//

/// FIX ME need to use dArray class here
// an array of open conncetions
Connection::Udgs Connection::_connections[ MAX_CONNECTION ];


///////////////////////////////////////////////////////////////////////////////
// public:
//

//
// 
//
Connection::Connection( const char* myPath, const char* itsPath )
{
    // get handle;
    if ( (_handle = getSocket( myPath, itsPath )) == ERROR )
    {
	cerr << "Connection(): couldn't create socket" << endl;
    }
}




//
// 
//
Connection::~Connection( void )
{
    // look for existing connection
    if ( _connections[ _handle ]._refCount > 0 )
    {
	// close socket on last de-reference
	if ( --_connections[ _handle ]._refCount == 0 )
	{
	    // reclaim socket
	    delete _connections[ _handle ]._socket;

	    // reclaim names
	    delete _connections[ _handle ]._myName;
	    delete _connections[ _handle ]._yourName;
	}
    }
}


//
// 
//
int
Connection::send( const char* msg )
{
    int status;
    int length = strlen( msg ) + 1;

    // send the message to the remote end
    while ( (status = _connections[ _handle ]._socket->send(
		       (char*)msg, length )) != length )
    {
	if ( status == -1 )
	{
	    if ( errno == ENOBUFS )		// temporary, retry
	    {
		continue;
	    }
	    else				// real problem
	    {
		perror( "Connection, send()" );
		return ERROR;
	    }
	}

	// bad count returned
	cerr << "Connection: bad send() count: " << status << endl;
	return ERROR;
    }

    return OK;
}


//
// 
//
int
Connection::recv( char* msg )
{
    int status;
    int length = strlen( msg ) + 1;

    // receive a message from the remote end
    while ( (status = _connections[ _handle ]._socket->recv((char*)msg)) < 0 )
    {
	if ( errno == ENOBUFS )		// temporary, retry
	{
	    continue;
	}
	else				// real problem
	{
	    perror( "Connection, recv()" );
	    return ERROR;
	}
    }

    return status;
}


///////////////////////////////////////////////////////////////////////////////
// private:
//

//
// 
//
int
Connection::getSocket( const char* me, const char* you )
{
    // lookup name for existing connection
    for ( int index = 0; index < MAX_CONNECTION; ++index )
    {
	// find existing sockets
	if ( _connections[ index ]._refCount > 0 )
	{
	    if ( (strcmp( _connections[ index ]._myName, me ) == 0) &&
	         (strcmp( _connections[ index ]._yourName, you ) == 0) )
	    {
		_connections[ index ]._refCount++;
		return index;
	    }
	}
    }

    // establish a new connection
    for ( int index = 0; index < MAX_CONNECTION; ++index )
    {
	// create a new socket if necessary
	if ( _connections[ index ]._refCount == 0 )
	{
	    // create socket
	    UnixDGSocket* socket = new UnixDGSocket( (char*)me, (char*)you );

	    // verify construction
	    if ( socket->getFD() != ERROR )
	    {
		_connections[ index ]._refCount++;
		_connections[ index ]._socket = socket;
		_connections[ index ]._myName = new char[ strlen( me )+1 ];
		strcpy( _connections[ index ]._myName, me );
		_connections[ index ]._yourName = new char[ strlen( you )+1 ];
		strcpy( _connections[ index ]._yourName, you );
		cerr << "Connection::getSocket(): "<<me<<", "<<you<<endl;
		return index;
	    }
	    else
	    {
		cerr << "Connection::getSocket(): couldn't create" << endl;
		return ERROR;
	    }
	}
    }

    // table full
    cerr << "Connection::getSocket(): table full" << endl;
    return ERROR;
}



///////////////////////////////////////////////////////////////////////////////
#if 0

//
// write message to addr thru specific UDG socket
//
int
Connection::writeMessage( UnixDGSocket* udgs,
			  UNIXSOCKADDR* addr,
			  const u_char* msg,
			  int count )
{
    // send 'msg' of 'count' bytes to 'addr'
    if ( udgs->send( (char *)msg, count, &addr ) != count )
    {
	perror( "writeMessage(): UnixDGSocket->send()" );
	return ERROR;
    }

    return OK;
}

#endif  // 0
