//
// Copyright 1996 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

// $Id: rcclient.hh,v 1.3 1996/03/07 02:00:41 smp Exp smp $


//
// rcdaemon.hh, header for rcdaemon.cc, etc.
//

#ifndef _RCDAEMON_H
#define _RCDAEMON_H


#include "util/misc/misc.hh"
#include "util/misc/tokens.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/namedstr.hh"

#include "connection.hh"
#include "rcserver.hh"

#if defined( HAVE_VOICE )
#include "voicedaemon.hh"
#endif  // HAVE_VOICE


#define DAEMON_DATA		1
#define CLIENT_DATA		2

class RcClient
{

 public:

    // constructor
    RcClient( const char* name );

    // destructor
    virtual 		~RcClient( void ) {}

    // initialize daemon
    virtual int		initialize( const char* path = NULL,
				    const char* logFile = NULL );

    // make a User DataGram Connection
    int			makeUDGConnection( const char* destination,
					   Connection*& connection,
					   int& connectionFD,
					   char* tmpAddr = NULL );

    // wait for data from stdin & default daemon via select()
    int			waitForData( void );

    // wait for data from stdin & registered sources via select()
    int			waitOnData( void );

    // add & remove an fd from the default fd_set
    void		addSelect( int fd );
    void		rmvSelect( int fd );

    // tests for data available from various sources
    int			dataReady( int fd ) { return FD_ISSET( fd, &_fdSet ); }
    int			clientDataReady( void )
    				{ return FD_ISSET( _clientFD, &_fdSet ); }
    int			daemonDataReady( void )
    				{ return FD_ISSET( _daemonFD, &_fdSet ); }

#if 0
    //
    // send a message to daemon:
    //

    /// FIXME: make more efficient
    // path & message in one string
    int			sendMessage( const char* path );

    /// FIXME: make more efficient
    // path & message in seperate strings
    int			sendMessage( const char* path, const char* msg );

    /// FIXME: make more efficient
    // path & message in seperate strings, wait for reply
    int			sendMessage( const char* path,
				     const char* msg, char* reply );
#endif  // 0

    // 
    int			incVerbose( void ) { return ++_verbose; }
    int			decVerbose( void ) { return --_verbose; }
    int			getVerbose( void ) { return _verbose; }
    int			setVerbose( int v ) { return (_verbose = v); }

#if defined( HAVE_VOICE )
    virtual int		say( char* s ) { _voice.say( s ); }
#else
    virtual int		say( char* s ) { return -1; }
#endif  // HAVE_VOICE

    // read message from daemon
    int			readDaemonMsg( char* msg );

    // return answer to daemon
    int			writeDaemonMsg( const char* msg, int count );


 protected:

    // base method for termination
    virtual void	terminate( void );

    void		markLog( const char* m );

    // table of strings
    NamedStr 		_defaults;


 private:

    // die cleanly on external SIGKILL
    static void		signalTerminate( void );

    // my (client process) name
    const char*		_myName;

    // my address
    const char*		_myPath;

    // data used by client socket mechanism
    UnixDGSocket*	_daemonSocketPtr;
    char		_daemonMsg[ MAX_PIPE_MSG ];
    UNIXSOCKADDR	_retAddr;

    // data used by waitForData() and select()
    fd_set		_fdSet;
    fd_set		_selectSet;
    int			_maxFD;
    int			_clientFD;
    int			_daemonFD;

    int			_verbose;

#if defined( USES_LOGFILE )
    // add time-stamped entry in daemons log
    void		markLog( const char* );
#endif  // USES_LOGFILE

#if defined( HAVE_VOICE )
    int			_verbal;
    Voice		_voice;
#endif  // HAVE_VOICE

#if defined( USE_KEYMAPS )
    KeyMap		_keyMap;
#endif  // USE_KEYMAPS

};


#endif // _RCCLIENT_H
