//
// Copyright 1996 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

// $Id: rcdaemon.hh,v 1.6 1996/03/05 03:54:41 smp Exp smp $


//
// rcdaemon.hh, header for rcdaemon.cc, etc.
//

#ifndef _RCDAEMON_H
#define _RCDAEMON_H


#include "util/misc/misc.hh"
#include "util/misc/tokens.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/namedstr.hh"

#include "connection.hh"
#include "rcserver.hh"

#if defined( HAVE_VOICE )
#include "voicedaemon.hh"
#endif  // HAVE_VOICE

#define DEVICE_DATA		0x01
#define CLIENT_DATA		0x02

// returned ASCII messages
#define OK_MSG			"<OK>"
#define FAIL_MSG		"<FAIL>"


#define USE_CONNECTION

class RcDaemon
{

 public:

    // constructor
    RcDaemon( void );

    // destructor
    virtual 		~RcDaemon( void ) {}

    // initialize daemon
    virtual int		initialize( const char* name,
				    const char* path,
				    const char* logFile,
				    const int port );

    // initialize device controlled by daemon
    virtual int		deviceInitialize( void ) { return -1; }

    //
    // send a message to client:
    //

    /// FIXME: make more efficient
    // path & message in one string
    int			sendMessage( const char* path );

    /// FIXME: make more efficient
    // path & message in seperate strings
    int			sendMessage( const char* path, const char* msg );

    /// FIXME: make more efficient
    // path & message in seperate strings, wait for reply
    int			sendMessage( const char* path,
				     const char* msg, char* reply );

    // get a message from client
    char*		getClientMsg() { return _clientMsg; }

    // return answer to client
    int			writeClientMsg( const char* msg, int count );
    int			writeClientMsg( const char* msg,
				        int count, UNIXSOCKADDR& addr );

    // wait for data via select()
    int			waitForData( void );

    // 
    int			incVerbose( void ) { return ++_verbose; }
    int			decVerbose( void ) { return --_verbose; }
    int			getVerbose( void ) { return _verbose; }
    int			setVerbose( int v ) { return (_verbose = v); }

#if defined( HAVE_VOICE )
    virtual int		say( char* s ) { _voice.say( s ); }
#else
    virtual int		say( char* s ) { return -1; }
#endif  // HAVE_VOICE


 protected:

    //
    inline int		returnMessage( const char* msg )
    {
	if ( writeClientMsg( msg, strlen( msg ) + 1 ) == ERROR )
	{
	    perror( "returnMessage(): writeClientMsg()" );
	    return ERROR;
	}

	return OK;
    }

    inline int		returnMessage( const char* msg, UNIXSOCKADDR& addr )
    {
	if ( writeClientMsg( msg, strlen( msg ) + 1, addr ) == ERROR )
	{
	    perror( "returnMessage(): writeClientMsg()" );
	    return ERROR;
	}

	return OK;
    }

    inline int		returnOkMsg( void )
    {
	return returnMessage( OK_MSG );
    }

    inline int		returnFailMsg( void )
    {
	return returnMessage( FAIL_MSG );
    }

    // base method for getting data
    virtual int		getDeviceData( void ) { return -1; }

    // base method for termination
    virtual void	terminate( void );

    // base method for opening control channel
    virtual int		openDevice( void ) { return -1; }

    // table of strings
    NamedStr 		_defaults;

    UNIXSOCKADDR	_retAddr;

    int			_verbose;


 private:

    // read message from client
    int			readClientMsg( void );

    // add time-stamped entry in daemons log
    void		logTime( const char* m = NULL );

    // die cleanly on external SIGKILL
    static void		signalTerminate( void );

#if defined( NO_ZOMBIES_ALLOWED )
    // avoid leaving zombie processes around
    void		childDeath( void ) { wait( (int *)0 ); }
#endif  // NO_ZOMBIES_ALLOWED

    // my (daemon processes) name
    const char*		_myName;

    // my address (well known address)
    const char*		_myPath;

    // data used by client socket mechanism
    UnixDGSocket*	_clientSocketPtr;
    char		_clientMsg[ MAX_PIPE_MSG ];

    // data used by waitForData() and select()
    fd_set		_fdSet;
    int			_maxFD;
    int			_clientFD;
    int			_deviceFD;

#if defined( HAVE_VOICE )
    int			_verbal;
    Voice		_voice;
#endif  // HAVE_VOICE

#if defined( USE_KEYMAPS )
    KeyMap		_keyMap;
#endif  // USE_KEYMAPS

};


#endif // _RCDAEMON_H
