//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: voicedaemon.cc,v 1.2 1996/03/02 08:12:45 smp Exp smp $";


//
// voicedaemon.cc, make a true daemon someday
//

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <string.h>
#include <sys/file.h>

#include "util/misc/misc.hh"
#include "voicedaemon.hh"

char* const _numword[] = {
    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
    "ten", "eleven", "twelve", "thirteen", "fourteen",
    "fifteen", "sixteen", "seventeen", "eighteen", "nineteen",
    "twenty", "thirty", "forty", "fifty",
    "sixty", "seventy", "eighty", "ninety"
};


Voice::Voice( int flag ) : _fp( NULL )
{

#if HAVE_VOICE == SERIAL_TTS

    int			fd;
    struct termios	newTerm;

    // open NONBLOCKing to allow setup of clocal if necessary
    if ( (fd = open( VOICE, O_RDWR|O_NONBLOCK )) == -1 ) return;

    if ( flag )
    {
	if ( tcgetattr( fd, &newTerm ) == ERROR ) return;

	newTerm.c_cc[VMIN] = 1;
	newTerm.c_cc[VTIME] = 0;
	newTerm.c_iflag &= ~( BRKINT|
			     IGNPAR|PARMRK|INPCK|ISTRIP|
			     INLCR|IGNCR|ICRNL|IXON );

	newTerm.c_oflag &= ~(OPOST);

	newTerm.c_cflag &= ~( CSIZE|CSTOPB|HUPCL|PARENB );
	newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

	newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			     ISIG|
			     ICANON|IEXTEN|
			     NOFLSH|
			     TOSTOP);

	if ( cfsetospeed( &newTerm, B9600 ) == ERROR ) return;
	if ( cfsetispeed( &newTerm, B9600 ) == ERROR ) return;
	if ( tcsetattr(fd, TCSANOW, &newTerm) == ERROR ) return;
    }

    // get a stream
    if ( (_fp = fdopen( fd, "r+" )) == NULL ) return;

    // no buffer of output
    setbuf( _fp, NULL );

#elif HAVE_VOICE == RSYNTH_TTS

// we assume rsynth is available

#else

///# error comment out this warning to build without voice support

#endif  // HAVE_VOICE ==

}



//
// 
//
int
Voice::say( char* s )
{

#if HAVE_VOICE == SERIAL_TTS

    fputs( s, _fp );
    putc( '\n', _fp );

#elif HAVE_VOICE == RSYNTH_TTS

    const char* command = "/usr/local/bin/rsay";
    char* string = new char[ strlen( command ) + strlen( s ) + 3 ];
    sprintf( string, "%s %s\n", command, s );
    system( string );
    delete string;

#else

// noop

#endif  // HAVE_VOICE ==

    return OK;
}


//
// 
//
int
Voice::say( int n )
{
    char s[ 256 ];

    if ( n < 20 )
    {
	strcpy( s, _numword[ n ] );
	say( s );
	return OK;
    }
    else if ( n < 100 )
    {
	strcpy( s, _numword[ (n/10) + 18 ] );
	if ( n %= 10 ) strcat( s, _numword[ n ] );
	say( s );
	return OK;
    }
    else
    {
	return ERROR;
    }
}
