//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id: hcs2control.cc,v 1.7 1996/03/31 02:08:44 smp Exp smp $";


//
// hcs2control.cc
//


#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/udgsocket.hh"
#include "rc/daemon/connection.hh"
#include "rc/daemon/rcserver.hh"

#include "hcs2control.hh"

static int		doArgs( int argc, char** argv, Hcs2Control& hcs2c );


main( int argc, char *argv[] )
{
    // create the (ONE and ONLY ONE) instance of the daemon object
    Hcs2Control hcs2c( argv[ 0 ] );

    // get args
    if ( doArgs( argc, argv, hcs2c ) == ERROR )
    {
	exit( 1 );
    }

    // initialize client, default daemon, NO log file
    if ( hcs2c.initialize( "HCS2D_DG_PATH", (char*)NULL ) == ERROR )
    {
	cerr << "hcs2control failure!" << endl;
	exit( 1 );
    }

    //
    // do the work here
    //
    return hcs2c.hcs2Main();
}


//
// 
//
int
Hcs2Control::hcs2Main( void )
{
    // get the socket paths from the RC defaults file
    char	myPath[ 15 ];
    if ( char* mp = (char*)_defaults.getString( "RCD_DG_TMP" ) )
    {
	strcpy( myPath, mp );
	mktemp( myPath );
    }
    else
    {
	cerr << "couldn't get myPath\n";
	return ERROR;
    }

    const char*	hcs2dPath;
    if ( !(hcs2dPath = _defaults.getString( "HCS2D_DG_PATH" )) )
    {
	cerr << "couldn't get hcs2d path\n";
	return ERROR;
    }

    // build a connection for watching inputs
    Connection*	monitor;
    int		monitorFD;
    monitor = new Connection( (const char*)myPath, (const char*)hcs2dPath );
    monitorFD = monitor->getFD();
    addSelect( monitorFD );

    // main loop
    while ( TRUE )
    {
	// wait for data
	int dataAvailable = waitOnData();

	// get message from the HCSII daemon
	if ( daemonDataReady() )
	{
	    char message[ MAX_DG_MSG ];
	    if ( readDaemonMsg( message ) < 0 )
	    {
		perror( "readDaemonMsg()" );
		return ERROR;
	    }

	    // process 'status' message
	    if ( message[ 0 ] == STATUS_MSG_START )
	    {
		cout << form( "status: %s\n", message );
	    }

	    else
	    {
		cerr << form( "unexpected daemon data: '%s'", message) << endl;
	    }
	}

	// data from client/user interface
	if ( clientDataReady() )
	{
	    // get input from stdin
	    char str[ MAX_INPUT ];
	    if ( fgets( str, MAX_INPUT, stdin ) == NULL ) break;

	    // remove WS, look for comments, get length
	    char* p = str;
	    while ( isspace( *p ) ) ++p;
	    if ( (*p == COMMENT_CHAR) || (*p == '\0') )
	    {
		continue;
	    }
	    int length = strlen( p ) + 1;	// count terminating '\0'

	    if ( strncmp( p, "wi", 2 ) == 0 )
	    {
		monitor->send( p );
		monitor->recv( str );
		cout << "wi: " << str << endl;
	    }
	    else if ( strncmp( p, "ii", 2 ) == 0 )
	    {
		monitor->send( p );
		monitor->recv( str );
		cout << "ii: " << str << endl;
	    }
	    else if ( strncmp( p, "monitor", 7 ) == 0 )
	    {
		monitor->send( p );
		monitor->recv( str );
		cout << "monitor: " << str << endl;
	    }

	    else
	    {
		// send the message to the daemon
		int count;
		while ( (count = writeDaemonMsg( p, length )) != length )
		{
		    if ( count == -1 )
		    {
			if ( errno == ENOBUFS ) continue;
			perror( "writeDaemonMsg()" );
		    }
		    else
		    {
			cerr << "writeDaemonMsg() count: " << count << endl;
		    }
		    return ERROR;
		}
	    }
	}

	// data from other sources
	if ( dataReady( monitorFD ) )
	{
	    char message[ MAX_DG_MSG ];
	    if ( monitor->recv( message ) < 0 )
	    {
		perror( "readDaemonMsg()" );
		return ERROR;
	    }

	    // process 'status' message
	    if ( message[ 0 ] == STATUS_MSG_START )
	    {
		cout << form( "status: %s\n", message );
	    }

	    // async data  FIXME: this ASSUMES tokens[ 0 ] == "Input:"
	    else
	    {
		Tokens& tokens = *new Tokens( message );

		char s[ 256 ];
		sprintf( s, "The %s is now %s.\n",
			 sensorData[ atoi( tokens[1] ) ].name, tokens[ 2 ] );
		cout << s;

		if ( getVerbose() ) say( s );

		delete &tokens;
	    }
	}

    }  // endOfWhile( TRUE )

    // reached EOF on client input
    return OK;
}


//
// process command line arguments
//
static int
doArgs( int argc, char** argv, Hcs2Control& hcs2c )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		hcs2c.incVerbose();
	        break;

	    default:
		cerr << "usage: %s [-v]" << argv[ 0 ] << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
