//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id: hcs2d.cc,v 1.9 1996/03/23 18:36:28 smp Exp smp $";


//
// HCS2 Real Time Controller daemon
//

#include <iostream.h>
#include <stdlib.h>
#include <string.h>

#include "hcs2d.hh"
#include "hcs2daemon.hh"


//
// local data & functions
//

static int	doArgs( int, char**, Hcs2Daemon& );


main( int argc, char *argv[] )
{
    // create the (ONE and ONLY ONE) instance of the daemon object
    Hcs2Daemon hcs2d;

    // process commandline args
    if ( doArgs( argc, argv, hcs2d ) == ERROR )
    {
	exit( 1 );
    }

    // initialize the HCSII
    int port = hcs2d.deviceInitialize();
    if ( port < 0 )
    {
	cerr << "HCS2D device initialization failure!" << endl;
	exit( 1 );
    }

    // initialize daemon
    if ( hcs2d.initialize( argv[0], "HCS2D_DG_PATH", "HCS2D_LOGFILE", port )
	  == ERROR )
    {
	cerr << "HCS2D daemon failure!" << endl;
	exit( 1 );
    }

    //
    // do the work here
    //
    hcs2d.hcs2Main();

    // should never get here!
    exit( 2 );
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[], Hcs2Daemon& hcs2d )
{
    const char* loadCmd = "hcs2Load ";

    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		hcs2d.incVerbose();
		break;

	    case 'k':
		hcs2d.sendMessage( "HCS2D_DG_PATH", "kill" );
		exit( 0 );

	    case 'l':
		if ( ++index >= argc ) return ERROR;

		char s[ 1024 /**FIXME: MAX_PATH*/ ];
		strcpy( s, loadCmd );
		strcat( s, argv[ index ] );
		hcs2d.sendMessage( "HCS2D_DG_PATH", s, s );
		cerr << "load result: '" << s << "'" << endl;
		exit( 0 );

	    default:
		cerr << form( "usage: %s [-klv]", argv[ 0 ] ) << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
