//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id: hcs2daemon.cc,v 1.14 1996/03/31 02:09:02 smp Exp smp $";


//
// hcs2daemon.cc
//

#include <iostream.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/file.h>

#include "hcs2daemon.hh"


//
// table of command/reply sizes used by HCSII
//  see hcs2daemon.h for details
//
struct Hcs2Daemon::Hcs2CmdInfo::ENTRY Hcs2Daemon::Hcs2CmdInfo::_entry[ 50 ] =
{
      {  0,  0 },	// reset
      {  0,  0 },	// 
      {  0,  0 },	// 
      {  1,  0 },	// 
      {  0,  8 },	// 
      {  8,  0 },	// 
      {  0,  0 },	// load program
      {  0,  0 },	// 
      {  0,  2 },	// 

#if defined( GET_LOGGED_DATA )
      {  0, -1 },	// get logged data
#else
      { -2, -2 },	// get logged data
#endif  // GET_LOGGED_DATA

      {  3,  0 },	// 

// 0x0b
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>

// 0x10
      {  2,  0 },	// 
      {  1,  2 },	// 
      {  3,  0 },	// 
      {  4,  0 },	// 
      {  1,  2 },	// 
      {  2,  0 },	// 
      {  4,  0 },	// 
      {  1,  2 },	// 
      {  2,  0 },	// 
      {  3,  0 },	// 
      {  1,  3 },	// 
      {  3,  0 },	// 
      {  1,  0 },	// 
      {  1,  2 },	// 
      {  2,  0 },	// 
      {  1,  0 },	// 
      {  1,  2 },	// 
      {  5,  0 },	// 
      {  2,  3 },	// 
      {  3,  0 },	// 
      {  1,  3 },	// get variable once
      {  3,  0 },	// set variable

// 0x26
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>
      { -2, -2 },	// <not defined>

// 0x30
      { -1,  0 },	// send string to network
      { -1,  0 }	// send string to voice
};



//
// initializer function
//
int
Hcs2Daemon::initialize( const char* name,	// process name
		        const char* pathName,	// symbolic path name
		        const char* logName,	// symbolic log name
		        const int port )	// device fd
{
    // call the base class initializer
    int status = RcDaemon::initialize( name, pathName, logName, port );
    if ( status != OK )
    {
	return status;
    }

    //
    memset( _currentInputs, 0,  HCS2_IO_BLOCKS );
    memset( _currentOutputs, 0,  HCS2_IO_BLOCKS );
    memset( _watchedInputs, 0,  HCS2_IO_BLOCKS );
    memset( _watchedOutputs, 0,  HCS2_IO_BLOCKS );

    /// do I really want to change the boards state???
    hcs2Quiet();

    // announce ourself
    say( "H C S 2 deemon started,,,\n" );

    return OK;
}


//
// initialize the HCSII hardware/software
//
int
Hcs2Daemon::deviceInitialize( void )
{
    // get a device channel
    if ( openDevice() == ERROR )
    {
	perror( "deviceInitialize(): openDevice()" );
	return ERROR;
    }

    return _port;
}


//
// turn off all EXPRESS default reports
//
void
Hcs2Daemon::hcs2Quiet( void )
{
    // time display off
    hcs2Command( HCS2_CMD_TIME_DISPLAY, 1, 0x00 );

    // ADC display off
    hcs2Command( HCS2_CMD_ADC_DISPLAY, 3, 0x00, 0x00, 0x00 );

    // digital in display off
    hcs2Command( HCS2_CMD_DI_DISPLAY, 4, 0x00, 0x00, 0x00, 0x00 );

    // digital out display off
    hcs2Command( HCS2_CMD_DO_DISPLAY, 4, 0x00, 0x00, 0x00, 0x00 );
}


//
// cleanup
//
void
Hcs2Daemon::terminate( void )
{
    say( "H C S 2 deemon stopped,,,\n" );

    RcDaemon::terminate();

    exit( 0 );
}


//
// main loop
//
int
Hcs2Daemon::hcs2Main( void )
{
    while ( TRUE )
    {
	// wait for data
	int status = waitForData();

	if ( status & DEVICE_DATA )
	{
	    // device ready
	    switch ( getDeviceData() )
	    {
	    case OK:				// no data
	    case TRUE:				// good data
		break;
		
	    case ERROR:				// bad data
	    default:				// bad ???
		terminate();
		exit( 2 );			// should never get here!
	    }
	}

	if ( status & CLIENT_DATA )
	{
	    if ( doClient() == ERROR )
	    {
		return ERROR;
	    }
	}
    }
}


//
// 
//
int
Hcs2Daemon::doClient( void )
{
    // client ready,  process packet
    Tokens& tokens = *new Tokens( getClientMsg() );

    // check for 'token' problems
    int count;
    if ( (count = tokens.count()) <= 0 )
    {
	if ( _verbose )
	  cerr << "doClient(): no args" << endl;
	returnFailMsg();
    }

    // send native HCSII command to board
    else if ( (strcmp( tokens[ 0 ], "hcs2Cmd" ) == 0) && (count == 2) )
    {
	int size;
	char buffer[ 128 ];
	if ( (size = hcs2Command( tokens[ 1 ] )) > 0 )
	{
	    formatData( size, _data, buffer );
	    returnMessage( buffer );
	}
	else if ( size == 0 )	returnOkMsg();
	else			returnFailMsg();
    }
    
    // load the program referenced in arg1
    else if ( (strcmp( tokens[ 0 ], "hcs2Load" ) == 0) && (count == 2) )
    {
	hcs2LoadProgram( tokens[ 1 ] ) < 0 ? returnFailMsg() : returnOkMsg();
    }
    
    // better 'notification request' parser, replace 'wi, ii, wo, io'
    else if ( (strcmp( tokens[ 0 ], "monitor" ) == 0) )
    {
	// expected command line:
	//	<monitor> <on|off> <i|o><#> [notification address]
	//	t[0]      t[1]     t[2]     t[3]

	// check arg count
	if ( (count == 3) || (count == 4) )
	{
	    // determine return address for notification
	    UNIXSOCKADDR addr;
	    if ( count == 4 ) UnixDGSocket::makeAddress( addr, tokens[ 3 ] );
	    else addr = _retAddr;

	    // decide if monitor will be ON or OFF
	    int action;
	    if ( strcmp( tokens[ 1 ], "on" ) == 0 )	  action = TRUE;
	    else if ( strcmp( tokens[ 1 ], "off" ) == 0 ) action = FALSE;
	    else					  goto failure;

	    // decide whether to monitor input or output
	    int type;
	    switch ( tokens[ 2 ][ 0 ] )
	    {
	    case 'i':	type = TYPE_INPUT;	break;
	    case 'o':	type = TYPE_OUTPUT;	break;
	    default:				goto failure;
	    }

	    // 
	    if ( modifyWatchedIO( type, action, &tokens[ 2 ][ 1 ], addr )
		== ERROR )
	      returnMessage( FAIL_MSG );
	    else
	      returnMessage( OK_MSG );
	}
	else
	{
	    returnMessage( FAIL_MSG );
	}
    }
    
#if 1
    /// FIXME: remove old style requests
    else if ( (strcmp( tokens[ 0 ], "wi" ) == 0) )
    {
	if ( (count == 2) || (count == 3) )
	{
	    UNIXSOCKADDR addr;
	    if ( count == 3 ) UnixDGSocket::makeAddress( addr, tokens[ 2 ] );
	    else addr = _retAddr;

	    if ( modifyWatchedIO( TYPE_INPUT, TRUE, tokens[ 1 ], addr )
		== ERROR )
	      returnMessage( FAIL_MSG );
	    else
	      returnMessage( OK_MSG );
	}
	else
	{
	    returnMessage( FAIL_MSG );
	}
    }
    
    else if ( (strcmp( tokens[ 0 ], "wo" ) == 0) && (count == 2) )
    {
	if ( modifyWatchedIO( TYPE_OUTPUT, TRUE, tokens[ 1 ], _retAddr )
	      == ERROR )
	{
	    returnMessage( FAIL_MSG );
	}
	else
	{
	    returnMessage( OK_MSG );
	}
    }
    
    else if ( strcmp( tokens[ 0 ], "ii" ) == 0 )
    {
	if ( (count == 2) || (count == 3) )
	{
	    UNIXSOCKADDR addr;
	    if ( count == 3 ) UnixDGSocket::makeAddress( addr, tokens[ 2 ] );
	    else addr = _retAddr;

	    if ( modifyWatchedIO( TYPE_INPUT, FALSE, tokens[ 1 ], addr )
		== ERROR )
	      returnMessage( FAIL_MSG );
	    else
	      returnMessage( OK_MSG );
	}
	else
	{
	    returnMessage( FAIL_MSG );
	}
    }
    
    else if ( (strcmp( tokens[ 0 ], "io" ) == 0) && (count == 2) )
    {
	if ( modifyWatchedIO( TYPE_OUTPUT, FALSE, tokens[ 1 ], _retAddr )
	      == ERROR )
	{
	    returnMessage( FAIL_MSG );
	}
	else
	{
	    returnMessage( OK_MSG );
	}
    }
#endif  /// FIXME
    
    // increment the 'verbose' member data
    else if ( strcmp( tokens[ 0 ], "iv" ) == 0 )
    {
	++_verbose;
	returnOkMsg();
    }
    
    // decrement the 'verbose' member data
    else if ( strcmp( tokens[ 0 ], "dv" ) == 0 )
    {
	--_verbose;
	returnOkMsg();
    }
    
    // kill thyself
    else if ( strcmp( tokens[ 0 ], "kill" ) == 0 )
    {
	terminate();
	exit( 2 );			// should never get here!
    }
    
    // no valid command found
    else
    {
	cerr << "Illegal command: '" << tokens[ 0 ];
	if ( count == 2 ) cerr << " " << tokens[ 1 ];
	cerr << "'" << endl;
	returnFailMsg();
    }

    delete &tokens;
    return OK;

 failure:
    returnMessage( FAIL_MSG );
    delete &tokens;
    return OK;
    
}


//
// 
//  type == TYPE_INPUT || TYPE_OUTPUT
//  what == TRUE || FALSE
//
int
Hcs2Daemon::modifyWatchedIO( int type,
			     int what,
			     const char* bit,
			     UNIXSOCKADDR& addr )
{
    char s[ 128 ];

    // determine bit#
    int watch = strtol( bit, (char**)NULL, 0 );
    if ( (watch < 0) || (watch >= HCS2_IO_BITS) )	// 0 based
    {
	return ERROR;
    }

    // get lists, etc.
    int cmd;
    SLList<UNIXSOCKADDR>* list;
    unsigned char *watched, *mapByte, *map;
    if ( type == TYPE_INPUT )
    {
	cmd = HCS2_CMD_DI_DISPLAY;
	list = &_clientsWatchingInputs[ watch ];
	watched = &_watchedInputs[ IO_BLOCK( watch ) ];
	mapByte = &_currentInputMap[ BITMAP_BYTE( watch ) ];
	map = _currentInputMap;
    }
    else  // type == TYPE_OUTPUT
    {
	cmd = HCS2_CMD_DO_DISPLAY;
	list = &_clientsWatchingOutputs[ watch ];
	watched = &_watchedOutputs[ IO_BLOCK( watch ) ];
	mapByte = &_currentOutputMap[ BITMAP_BYTE( watch ) ];
	map = _currentOutputMap;
    }

    if ( what == TRUE )
    {
	// get current state
	if ( (*list).empty() )
	{
	    updateCurrentIO( type, watch );
	}

	// append return address to list
	(*list).append( addr );

	// mark bit as being watched
	*watched |= IO_BLOCKMASK( watch );

	// mark this block as active
	*mapByte |= BITMAP_BIT( watch );
    }

    else  // what == FALSE
    {
	// search list for instance of addr, delete if found
	if ( deleteFromList( *list, addr ) == 0 )
	{
	    return ERROR;
	}

	// check for other active watchers
	if ( (*list).empty() )
	{
	    // clear the watched bit from watched byte
	    *watched &= ~IO_BLOCKMASK( watch );

	    // turn off the HCSII report if nothing watched in this block
	    if ( *watched == 0 )
	    {
		*mapByte &= ~BITMAP_BIT( watch );
	    }
	}
    }

    // build HCSII command to send periodic reports
    sprintf( s, "0x%02x 0x%x 0x%x 0x%x 0x%x",
	     cmd, map[ 0 ], map[ 1 ], map[ 2 ], map[ 3 ] );

    // send the command to HCSII
    if ( hcs2Command( s ) < 0 )
    {
	cerr << "modifyWatchedIO(): hcs2Command()" << endl;
	return ERROR;
    }

    // done
    return OK;
}


//
// get the current status of 'bit', update cache byte
//
int
Hcs2Daemon::updateCurrentIO( int type, int bit )
{
    int cmd;
    unsigned char* current;

    if ( type == TYPE_INPUT )
    {
	cmd = HCS2_CMD_GET_DI_ONCE;
	current = &_currentInputs[ IO_BLOCK( bit ) ];
    }
    else  // type == TYPE_OUTPUT
    {
	cmd = HCS2_CMD_GET_DO_ONCE;
	current = &_currentOutputs[ IO_BLOCK( bit ) ];
    }

    if ( (hcs2Command( cmd, 1, bit ) == 3)
	  && (_data[ 0 ] == cmd ) && (_data[ 1 ] == bit) )
    {
	if ( _data[ 2 ] )
	{
	    // bit is currently high, set it in byte
	    *current |= IO_BLOCKMASK( bit );
	}
	else
	{
	    // bit is currently low, clear it in byte
	    *current &= ~IO_BLOCKMASK( bit );
	}
	return OK;
    }
    else
    {
	cerr << "updateCurrentIO(), hcs2Command() failure!" << endl;
	return ERROR;
    }
}


//
// 
//
int
Hcs2Daemon::reportChangedIO( int type, int group, u_char state )
{
    // get lists, etc.
    unsigned char watched, current;
    if ( type == TYPE_INPUT )
    {
	watched = _watchedInputs[ group ];
	current = _currentInputs[ group ];
	_currentInputs[ group ] = state;
    }
    else  // type == TYPE_OUTPUT
    {
	watched = _watchedOutputs[ group ];
	current = _currentOutputs[ group ];
	_currentOutputs[ group ] = state;
    }

    // compare to old state, mask off unwatched bits
    unsigned char changed = (current ^ state) & watched;

    // report remaining changed bits
    if ( changed )
    {
	char s[ 256 ];

	// check each bit in block
	int bitNumber = 0;
	unsigned char mask = 0x01;
	for (  ; bitNumber < 8; ++bitNumber, mask <<= 1 )
	{
	    if ( changed & mask )
	    {
		int bit = GROUPBIT_TO_BIT( group, bitNumber );

		// build a message for the client(s)
		sprintf( s, "%s %d %s",
			 (type == TYPE_INPUT) ? "input" : "output",
			 bit,
			 (state & mask) ? "open" : "closed" );
	        if ( _verbose ) { cerr << s << endl; }

		// get the appropriate list
		SLList<UNIXSOCKADDR>& list = ((type == TYPE_INPUT) ?
					      _clientsWatchingInputs[ bit ] :
					      _clientsWatchingOutputs[ bit ]);

		// walk list sending message to each client
		for ( Pix p = list.first(), lp = 0; p != 0;  )
		{
///		    cerr << "name: " << (UNIXSOCKADDR&)list(p).name << endl;
		    if ( returnMessage( s, (UNIXSOCKADDR&) list( p ) ) < 0 )
		    {
			// client dead?
			char ss[ 128 ];
			sprintf( ss, "%d", bit );
			modifyWatchedIO( type, FALSE, ss,
					 (UNIXSOCKADDR&) list( p ) );

			// backup pointer to 'last pointer'
			if ( lp )
			{
			    p = lp;		// use 'last pointer'
			    list.next( p );	//  to get 'new pointer'
			}
			else
			{
			    p = list.first();	// p was 1st, get new 1st
			}
		    }
		    else
		    {
			lp = p;		   // advance 'last pointer' to pointer
			list.next( p );    // advance pointer to 'next pointer'
		    }
	        }
	    }
	}
    }

    return OK;
}


//
// 
//
int
Hcs2Daemon::deleteFromList( SLList<UNIXSOCKADDR>& list, UNIXSOCKADDR& addr )
{
    int count = 0;

    if ( _verbose )
    {
	cerr << form( "dt: %s, list:\n", addr.addr.sun_path ); printList(list);
    }

    // walk list looking for EACH INSTANCE OF addr
    for ( Pix p = list.first(), lp = 0; p != 0;  )
    {
	if ( !strcmp( list( p ).addr.sun_path, addr.addr.sun_path ) )
	{
	    // delete the entry
	    list.del_after( lp );

	    // backup pointer to 'last pointer'
	    if ( lp )
	    {
		p = lp;			// use 'last pointer'
		list.next( p );		//  to get 'new pointer'
	    }
	    else
	    {
		p = list.first();	// p was 1st, get new 1st
	    }

	    // inc count of deleted nodes
	    ++count;
	}

	else
	{

	    lp = p;			// advance 'last pointer' to pointer
	    list.next( p );		// advance pointer to 'next pointer'
	}
    }

    if ( _verbose ) { cerr << "list out:\n"; printList( list ); }

    // return count of deleted nodes
    return count;
}


//
// 
//
void
Hcs2Daemon::printList( SLList<UNIXSOCKADDR>& list )
{
    for ( Pix p = list.first(); p != 0; list.next( p ) )
    {
	cerr << form( "client: %s", list( p ).addr.sun_path ) << endl;
    }
    cerr << endl;
}


//
// front end:
//  expect an ASCII string of 'byte values'
//
int
Hcs2Daemon::hcs2Command( char* asciiBytes )
{
    // parse the command and data
    Tokens& tokens = *new Tokens( asciiBytes );
    int count = tokens.count();
    unsigned char* data = new unsigned char[ count ];

    // place into an array
    for ( int x = 0; x < count; ++x )
    {
	data[ x ] = (unsigned char)strtol( tokens[ x ], (char**)NULL, 0 );
    }

    // send to the hardware
    int status = hcs2Command( count, data );

    // return storage
    delete data;
    delete &tokens;

    return status;
}


//
// front end:
//  expect an integer command, an integer count, and 'count' integer data
//
int
Hcs2Daemon::hcs2Command( int cmd, int count ... )
{
    va_list ap;

    va_start( ap, count );

    // storage for the array: command byte + 'count' data bytes
    unsigned char* data = new unsigned char[ ++count ];

    // the specific command
    data[ 0 ] = (unsigned char)cmd;

    // the command data
    for ( int x = 1; x < count; ++x )
    {
	data[ x ] = va_arg( ap, int );
    }

    int status = hcs2Command( count, data );
    delete data;

    va_end( ap );

    return status;
}


//
// low level HCSII command routine
//  expect an integer count, and an array of 'count' bytes data
//
int
Hcs2Daemon::hcs2Command( int count, unsigned char* data )
{
    const unsigned char byte = HCS2_COMMAND;
    unsigned char& command = data[ 0 ];

    // verify count
    int length = _cmdInfo.cmdLength( command );
    if ( length == -2 )
    {
	if ( _verbose )
	  cerr << form( "hcs2Command(), bad cmd: %d", command ) << endl;
	return ERROR;
    }

    if ( length == -1 )
    {
	if ( _verbose )
	  cerr << "hcs2Command(), variable length NOT supported" << endl;
	return ERROR;
    }

    if ( length != (count-1) )
    {
	if ( _verbose )
	  cerr << form( "hcs2Command(), bad count: %d", (count-1) ) << endl;
	return ERROR;
    }

    // the header: '!'
    if ( write( _port, &byte, 1 ) != 1 )
    {
	if ( _verbose )
	  perror( "header write" );
	return ERROR;
    }

    // the command and data
    if ( write( _port, data, count ) != count )
    {
	if ( _verbose )
	  perror( "data write" );
	return ERROR;
    }

    // program loads are special case
    if ( command == HCS2_CMD_LOAD_PROG )
    {
	return 0;
    }

    // consumed acknowledge phase
    return hcs2Packet( (int)command );
}


//
// get HCSII packets
//  expected >=  0 means we expect a specific packet
//  expected == -1 means we expect async data
//
int
Hcs2Daemon::hcs2Packet( int expected )
{
    unsigned char	inputBuffer[ 1024 ];
    int			inputIndex = 0;

 top:

    // get <$> header
    if ( (hcs2Byte() != 1) || (_byte != HCS2_REPLY) )
    {
	cerr << form( "hcs2Packet(), bad <$>: 0x%02x", _byte ) << endl;
	return ERROR;
    }

    // get <cmd>
    if ( hcs2Byte() != 1 )
    {
	perror( "hcs2Packet(), <cmd> read" );
	return ERROR;
    }

    // compare to expected command
    if ( (expected == -1) || (_byte != expected) )
    {
	// check for async data
	if ( (_byte >= HCS2_FIRST_ASYNC) && (_byte <= HCS2_LAST_ASYNC) )
	{
	    if ( hcs2AsyncData( _byte ) == ERROR )
	    {
		cerr << "hcs2Packet(), bad hcs2AsyncData()" << endl;
		return ERROR;
	    }
	    if ( expected == -1 )
	    {
		return OK;
	    }
	    goto top;  // forgive me...
	}

	// something wrong
	else
	{
	    cerr << form( "hcs2Packet(), bad <cmd>: 0x%02x", _byte ) << endl;
	    return ERROR;
	}
    }

    // no expected data
    int inputExpected = _cmdInfo.dataLength( _byte );
    if ( inputExpected == 0 )
    {
	return 0;
    }

    // unknown command
    if ( inputExpected == -2  )
    {
	cerr << form( "hcs2Packet(), unknown <cmd>: 0x%02x", _byte ) << endl;
	return ERROR;
    }

    // numeric data
    inputBuffer[ inputIndex++ ] = _byte;
    for ( int x = 0; x < inputExpected; ++x )
    {
	if ( hcs2Byte() != 1 )
	{
	    perror( "hcs2Packet(), read" );
	    return ERROR;
	}
	inputBuffer[ inputIndex++ ] = _byte;
    }

    // process packet
    return processData( inputExpected + 1, inputBuffer );
}


//
// async data, periodically/programmatically generated by HCSII
//
int
Hcs2Daemon::hcs2AsyncData( int asyncType )
{
    unsigned char	inputBuffer[ 256 ];
    int			inputExpected = 0;
    int			inputIndex = 0;

    // string data
    if ( asyncType == HCS2_STAT_CONSOLE )
    {
	inputExpected = -1;
	while ( inputExpected == -1 )
	{
	    if ( hcs2Byte() != 1 )
	    {
		perror( "hcs2AsyncData(), read" );
		return ERROR;
	    }

	    switch ( _byte )
	    {
	    case '\r':
	    case '\n':						break;
	    case 0:	inputExpected = 0;			break;
	    default:	inputBuffer[ inputIndex++ ] = _byte;	break;
	    }
	}

	// terminate string & process
	inputBuffer[ inputIndex ] = '\0';
	return processString( (char*)inputBuffer );
    }

    switch ( asyncType )
    {

    case 0x80:			inputExpected =  8;	break;
    case 0x81:			inputExpected =  3;	break;
    case HCS2_STAT_DI:		inputExpected =  2;	break;
    case HCS2_STAT_DO:		inputExpected =  2;	break;
    case HCS2_STAT_AI:		inputExpected = 17;	break;
    case HCS2_STAT_AO:		inputExpected =  5;	break;
    case 0x86:			inputExpected =  2;	break;
    case 0x87:			inputExpected =  2;	break;

    default:						return ERROR;

    }

    // numeric data
    inputBuffer[ inputIndex++ ] = _byte;
    for ( int x = 0; x < inputExpected; ++x )
    {
	if ( hcs2Byte() != 1 )
	{
	    perror( "hcs2AsyncData(), read" );
	    return ERROR;
	}
	inputBuffer[ inputIndex++ ] = _byte;
    }

    // process packet
    return processData( inputExpected + 1, inputBuffer );
}


//
// download a program to the HCSII
//
#define LOAD_BUFFER_SIZE	64
int
Hcs2Daemon::hcs2LoadProgram( const char* name )
{
    int file, status, inCount, outCount;
    unsigned char buffer[ LOAD_BUFFER_SIZE ];

    if ( openDevice() == ERROR )
    {
	perror( "hcs2LoadProgram(), openDevice()" );
	return ERROR;
    }

    if ( (file = open( name, O_RDONLY)) < 0 )
    {
	perror( "hcs2LoadProgram(), open()" );
	return ERROR;
    }

    // send 'load program' command
    hcs2Command( HCS2_CMD_LOAD_PROG, 0 );

    // wait for ACK
    while ( TRUE )
    {
	switch ( hcs2Byte() )
	{
	case 0:
	    continue;
	case 1:
	    if ( _byte != HCS2_ACK ) continue;
	    break;
	default:
	    cerr << "ERROR program load ACK#1" << (int)_byte << endl;
	    return ERROR;
	}

	break;
    }

    // send program to HCSII
    while ( TRUE )
    {
	// read a block of data from file
	if ( (inCount = read( file, buffer, LOAD_BUFFER_SIZE )) < 0 )
	{
	    perror( "hcs2LoadProgram(), read()" );
	    close( file );
	    return ERROR;
	}

	// EOF, all done
	else if ( inCount == 0 )
	{
	    close( file );
	    break;
	}

	// send a block of data to HCSII
	if ( (outCount = write( _port, buffer, inCount )) != outCount )
	{
	    perror( "hcs2LoadProgram(), write()" );
	    close( file );
	    return ERROR;
	}
    }

    // wait for ACK
    while ( TRUE )
    {
	switch ( hcs2Byte() )
	{
	case 0:
	    continue;

	case 1:
	    switch ( _byte )
	    {
	    case HCS2_ACK:				break;

	    case HCS2_BAD_FIRMWARE:
		cerr << "firmware ERROR" << endl;	return ERROR;

	    case HCS2_TOO_BIG:
		cerr << "prog TOO LARGE" << endl;	return ERROR;
	    }
	    break;

	default:
	    cerr << "ERROR program load ACK#2" << endl;
	    return ERROR;
	}

	break;
    }

    // board was reset, shut off EXPRESS default output
    hcs2Quiet();

    // all done
    return OK;
}


//
// process HCSII data
//
int
Hcs2Daemon::processData( int count, unsigned char* data )
{
    if ( count > MAX_DATA ) return ERROR;

    // store in permanent buffer
    memcpy( _data, data, count );

    switch ( data[ 0 ] )
    {

    // async digital input report
    case HCS2_STAT_DI:
	// data[ 1 ] == group#, data[ 2 ] == new state
	reportChangedIO( TYPE_INPUT, data[ 1 ], data[ 2 ] );
	break;

    // async digital output report
    case HCS2_STAT_DO:
	// data[ 1 ] == group#, data[ 2 ] == new state
	reportChangedIO( TYPE_OUTPUT, data[ 1 ], data[ 2 ] );
	break;

    // acceptable
    case HCS2_CMD_GET_DI_ONCE:
    case HCS2_CMD_GET_DO_ONCE:
	break;

    // all others
    default:
	cerr << "Got unexpected data: ";
	logData( count, data );
	break;
    }

    // result is number of bytes
    return count;
}


//
// log HCSII data
//
int
Hcs2Daemon::logData( int count, const unsigned char* data )
{
    char buffer[ 128 ];

    formatData( count, data, buffer );
    cerr << buffer << endl;

    return OK;
}


//
// format HCSII data
//
int
Hcs2Daemon::formatData( int count, const unsigned char* data, char* buffer )
{
    int x = 0;
    sprintf( (char*)&buffer[ 0 ], "%02x ", (int)data[ x++ ] );

    int	index = 3;
    for ( ; x < count; ++x )
    {
	sprintf( (char*)&buffer[ index ], "%02x", (int)data[ x ] );
	index += 2;
    }
    buffer[ index ] = '\0';

    return OK;
}


//
// process HCSII strings
//
//  expected format: 0123 ... n
//                   *: <string>0
int
Hcs2Daemon::processString( const char* string )
{
    switch( string[ 0 ] )
    {
    case 'L':
	cerr << (char*)&string[ 2 ] << endl;
	break;
    case 'M':
	sendMessage( &string[ 2 ] );
	break;
    case 'S':
	system( (char*)&string[ 2 ] );
	break;
    case 'V':
	say( (char*)&string[ 2 ] );
	break;
    default:
	cerr << " Event: " << string << endl;
	break;
    }

    return OK;
}


//
// open the serial port connected to the HCSII
//
int
Hcs2Daemon::openDevice( void )
{
    struct termios	newTerm;

    if ( _port >= 0 )
    {
	return OK;
    }

    // get the default strings
    const char*	hcs2File;
    if ( !(hcs2File = _defaults.getString( "HCS2PORT" )) )
    {
	cerr << "couldn't find HCS2PORT" << endl;
	return ERROR;
    }

    // open port
    if ( (_port = open( hcs2File, O_RDWR )) < 0 )
    {
	perror( "open( HCS2PORT )" );
	return ERROR;
    }

    if ( tcgetattr( _port, &newTerm ) == ERROR ) {
	close( _port );
	_port = -1;
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	hcs2Baud;
    if ( !(hcs2Baud = _defaults.getString( "HCS2BAUD" )) )
    {
	cerr << "couldn't get HCS2BAUD" << endl;
	close( _port );
	_port = -1;
	return ERROR;
    }
    if ( cfsetspeed( &newTerm, atoi( hcs2Baud ) ) == ERROR )
    {
	close( _port );
	_port = -1;
	return ERROR;
    }

    if ( tcsetattr( _port, TCSANOW, &newTerm ) == ERROR )
    {
	close( _port );
	_port = -1;
	return ERROR;
    }

    return OK;
}

template class SLNode<UNIXSOCKADDR>;
template class SLList<UNIXSOCKADDR>;

///////////////////////////////////////////////////////////////////////////////
#if 0  // moved to rcdaemon base class
inline int
Hcs2Daemon::returnMessage( const char* msg )
{
    if ( writeClientMsg( msg, strlen( msg ) + 1 ) == ERROR )
    {
	perror( "returnMessage(): writeClientMsg()" );
	return ERROR;
    }

    return OK;
}
inline int
Hcs2Daemon::returnMessage( const char* msg, UNIXSOCKADDR& addr )
{
    if ( writeClientMsg( msg, strlen( msg ) + 1, addr ) == ERROR )
    {
	perror( "returnMessage(): writeClientMsg()" );
	return ERROR;
    }

    return OK;
}
#endif  // moved
