//
// Copyright 1995 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHOR (Steve Passe)
// is not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHOR makes no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHOR
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// AUTHOR:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Header: /usr/local/src/control/rc/hcs2evnt/RCS/zap.c,v 1.1 1996/02/26 02:56:37 smp Exp smp $";

#include <stdio.h>
#include <unistd.h>
#include <sgtty.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <signal.h>

char* progname;

// all procs, pid and command
char* PS_CMD = "ps -a -opid,command";

int	unconditional = 0;
int	verbose = 0;

int	processMatch( char* buf, char* arg, int& pid );
char	ttyin( void );
int	kbhit( FILE* tty );
void	doargs( int& ac, char**& av );

int
main( int argc, char* argv[] )
{
    int pid;
    int myPid;
    int match;
    char buf[ BUFSIZ ];
///    ifstream fin;
    FILE* fin;

    progname = argv[ 0 ];
    myPid = getpid();

    doargs( argc, argv );

    // run "ps", read output
    if ( (fin = popen( PS_CMD, "r" )) == NULL )
    {
	cerr << progname << ": can't run " << PS_CMD << endl;
	exit( 1 );
    }

    // header
    fgets( buf, sizeof( buf ), fin );
    if ( verbose ) cerr << buf << endl;

    while ( fgets( buf, sizeof( buf ), fin ) != NULL )
    {
	// strip newline
	buf[ strlen( buf ) - 1 ] = '\0';

	// check for match
	match = processMatch( buf, argv[ 0 ], pid );

	// dont kill ourselves
	if ( pid == myPid )
	{
	    continue;
	}

	// want all or conditional match
	if ( (argc == 0) || (match && !unconditional) )
	{
	    cerr << buf << "? ";
	    switch ( ttyin() )
	    {
	    case 'y':
		if ( verbose ) cerr << "kill: " << pid << endl;
		kill( pid, SIGKILL );
		break;

	    case 'n':
		break;

	    case 'q':
		exit( 0 );

	    default:
		break;

	    }
	}

	// match and NO ask
	else if ( match && unconditional )
	{
	    if ( verbose ) cerr << "kill: " << pid << endl;
	    kill( pid, SIGKILL );
	}
    }
}


int
processMatch( char* buf, char* arg, int& pid )
{
    sscanf( buf, "%d", &pid );

    // return process id on match to name
    if ( strstr( buf, arg ) )
    {
	return 1;
    }

    return 0;
}


char
ttyin( void )
{
    char buf[ BUFSIZ ];
    static FILE* tty = NULL;

    if ( tty == NULL )
    {
	tty = fopen( "/dev/tty", "r" );
    }
#if 0
    while ( !kbhit( tty ) ) continue;
    return fgetc( tty );
#else
    if ( fgets( buf, BUFSIZ, tty ) == NULL )
    {
	return 0;
    }
    return buf[0];
#endif
}


int kbhit( FILE* tty )
{
    int n;

    if (ioctl(fileno(tty), FIONREAD, &n) == -1)
    {
        n = 0;
    }

    return n != 0;
}


void
doargs( int& ac, char**& av )
{
    int		c = ac--;
    char*	p;

    while ( c-- > 1 ) {
	if ( *(p = *(++av)) == '-') {
	    --ac;
	    if ( *(++p) != '\0' ) {
		switch( *p & 0x7f ) {

		case 'u':
		    unconditional = 1;
		    break;

		case 'v':
		    verbose = 1;
		    break;

		default:
		    break;
		}
	    }
	}
    }
}
