//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: ircontrol.c,v 1.1 1995/08/31 00:49:30 smp Exp $";


//
// ircontrol.cc
//

#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/ird/ird.hh"
#include "rc/daemon/rcserver.hh"


#define MAX_INPUT	1024
#define COMMENT_CHAR	'#'

static int		verbose = 0;
static int		doArgs( int, char ** );
static int		control( FILE * );


main( int argc, char *argv[] )
{
    // get args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // body of client
    return control( stdin );
}


static int
control( FILE *fp )
{
    // get the RC defaults file
    NamedStr defaults;
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << "\n";
	return BAD;
    }

    // get the socket paths from the RC defaults file
    const char*	myPath;
    const char*	itsPath;
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	  || !(itsPath = defaults.getString( "IRD_DG_PATH" )) )
    {
	cerr << "couldn't get socket paths\n";
	return BAD;
    }

    // create a socket
    UnixDGSocket udgSocket( (char*)myPath, (char*)itsPath );
    if ( udgSocket.getFD() == ERROR ) return BAD;

    char str[ MAX_INPUT ];
    while ( fgets( str, MAX_INPUT, fp ) != NULL )
    {
	// remove WS, look for comments
	char* p = str;
	while ( isspace( *p ) ) ++p;
	if ( *p == COMMENT_CHAR )
	{
	    continue;
	}

	// send the message to the ir daemon
	int status;
	int commandLength = strlen( p ) + 1;

	while ( (status = udgSocket.send( p, commandLength ))
	         != commandLength )
	{
	    if ( status == -1 )
	    {
		if ( errno == ENOBUFS ) continue;
		perror( "ircontrol, sendto()" );
		return BAD;
	    }
	    else
	    {
		cerr << "ircontrol: bad send() count: " << status << endl;
		return BAD;
	    }
	}
    }

    return OK;
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << "usage: %s [-v]" << argv[ 0 ] << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
