//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: ird.cc,v 1.2 1996/03/31 02:12:02 smp Exp smp $";


//
// IR equipment daemon
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/keymap.hh"
#include "util/misc/tokens.hh"
#define IRD
#include "ird.hh"
#include "irdaemon.hh"
#include "../daemon/rcserver.hh"

/// FIX ME when irDaemon becomes an object!!!
IR*			irp;

#if defined( HAVE_VOICE )

#include "../daemon/voicedaemon.hh"

/// FIX ME when irDaemon becomes an object!!!
Voice*			vp;

#endif  // HAVE_VOICE


static int	verbose = 0;
static int	verbal = FALSE;		/// THIS REALLY BELONGS IN ???

#if defined( HAVE_READER )

static int	readerPid;
static int	readerPipe;

#endif  // HAVE_READER

//
// local functions
//
static void	daemon( const char* p );
static void	daemonTerminate( void );
static int	killIRd( void );
static int	readUnixDG( UnixDGSocket* );

#if defined( HAVE_READER )

static int	readIR( int );

#endif  // HAVE_READER

static void	markLog( char* );
static int	doArgs( int, char** );


main( int argc, char *argv[] )
{
    struct sigaction	action;
    char		s[ 255 ];
    const char*		p;
    NamedStr		defaults;

    // get args
    if ( doArgs( argc, argv ) == ERROR )
    {
	exit( 1 );
    }

    // load defaults file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << form( "couldn't load file: %s", RCD_STRINGS ) << endl;
	exit( 1 );
    }

    // send stderr to logfile
    if ( p = defaults.getString( "IRD_LOGFILE" ) )
    {
	if ( freopen( p, "a+", stderr ) == NULL )
	{
	    perror( "logfile open()" );
	    exit( 1 );
	}
    }

    // no buffer of output
    setbuf( stderr, NULL );

    // 
    markLog( "\n\nIR Daemon started: " );

    // prepare for signals
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;

    // setup the INTerrupt handler
#if 0 /**(defined(BSD) && (BSD >= 199306))*/
    action.sa_handler = daemonTerminate;
#else
    action.sa_handler = (PFIV)daemonTerminate;
#endif  // BSD

    sigaction( SIGINT, &action, (struct sigaction *)NULL );
    sigaction( SIGQUIT, &action, (struct sigaction *)NULL );

#if defined( NO_ZOMBIES_ALLOWED )
    action.sa_handler = childDeath;
    sigaction( SIGCHLD, &action, (struct sigaction *)NULL );
#endif  // NO_ZOMBIES_ALLOWED

    // close unneeded files
    fclose( stdin );
    fclose( stdout );

    // run the daemon proper
    if ( p = defaults.getString( "IRD_DG_PATH" ) )
    {
    	daemon( p );
    }
    else
    {
	cerr << "IRD_DG_PATH not found!" << endl;
	daemonTerminate();
    }
}


//
// the daemon process:
//  wait for IO from either the reader process or the client socket(s):
//   reader process:
//    process IR data as appropriate
//   socket:
//    process commands as appropriate
//
static void
daemon( const char* p )
{
    int			maxFd, count, unixDG;
    fd_set		fdSet;
    UnixDGSocket	udgSocket( (char *)p );

#if defined( HAVE_VOICE )

    /// FIX ME when irDaemon becomes an object!!!
    vp = new Voice;
    vp->say( "in fra red deemon started,,,\n" );

#endif  // HAVE_VOICE

    /// FIX ME when irDaemon becomes an object!!!
    irp = new IR;

    // setup data for select()
    unixDG = udgSocket.getFD();

    FD_ZERO( &fdSet );

#if defined( HAVE_READER )
    maxFd = MAX( readerPipe, unixDG );
#else
    maxFd = unixDG;
#endif  // HAVE_READER

    // fds are zero based indices, turn into a count
    ++maxFd;

    // main processing loop
    while ( TRUE )
    {
	// wait for traffic
	FD_SET( unixDG, &fdSet );

#if defined( HAVE_READER )
	FD_SET( readerPipe, &fdSet );
#endif  // HAVE_READER

	count = select( maxFd, &fdSet, (fd_set*)0, (fd_set*)0,
		        (struct timeval*)0 );

#if defined( HAVE_READER )
	if ( FD_ISSET( readerPipe, &fdSet ) )
	{
	    if ( readIR( readerPipe ) == FALSE )
	    {
		daemonTerminate();
	    }
	}
#endif  // HAVE_READER

	if ( FD_ISSET( unixDG, &fdSet ) )
	{
	    if ( readUnixDG( &udgSocket ) == FALSE )
	    {
		daemonTerminate();
	    }
	}
    }
}


//
// 
//
static int
readUnixDG( UnixDGSocket* udgs )
{
    int			cnt;
    char		msg[ MAX_PIPE_MSG ];
    UNIXSOCKADDR	retAddr;

    retAddr.len = sizeof( retAddr.addr );
    if ( (cnt = udgs->recv( (char *)msg, &retAddr )) < 0 )
    {
	perror( "recvfrom()" );
	return FALSE;
    }

    // effective EOF, time to quit
    if ( cnt == 0 ) return FALSE;

    // process packet
    Tokens& tokens = *new Tokens( msg, 2 );

    if ( strcmp( tokens[ 0 ], "emit" ) == 0 )
    {
	irp->send( (const char*)tokens[ 1 ] );
    }

    else if ( strcmp( tokens[ 0 ], "kill" ) == 0 )
    {
	daemonTerminate();
	exit( 2 );				// should never get here!
    }

    else
    {
	cerr << form( "Illegal command: 0x%02x\n", msg[ 0 ] ) << endl;
    }

    delete &tokens;
    return TRUE;
}


#if defined( HAVE_READER )

//
// 
//
static int
readIR( int rp )
{
}

#endif  // HAVE_READER


//
// terminate
//
static void
daemonTerminate( void )
{
    pid_t	pid;

#if defined( HAVE_READER )

    if ( readerPid ) kill( readerPid, SIGTERM );

    while ( TRUE )
    {
	if ( wait( (int *)0 ) > 0 ) continue;
	if ( errno == ECHILD ) break;
    }
#endif  // HAVE_READER

/// in destructor???
///    unlink( RCD_DG_PATH );

    markLog( "\nIR Daemon stopped: " );

#if defined( HAVE_VOICE )

    /// FIX ME when irDaemon becomes an object!!!
    vp->say( "in fra red deemon stopped,,,\n" );

#endif  // HAVE_VOICE

    exit( 0 );
}


//
// 
//
static int
killIRd( void )
{
    char		s[] = "kill";
    const char*		myPath;
    const char*		itsPath;
    NamedStr		defaults;

    // get the socket paths from the RC defaults file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load file: %s" << RCD_STRINGS << endl;
	return ERROR;
    }
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	  || !(itsPath = defaults.getString( "IRD_DG_PATH" )) )
    {
	cerr << "couldn't get socket paths" << endl;
	return ERROR;
    }

    // get a socket to the ird server
    UnixDGSocket udgSocket( (char*)myPath, (char*)itsPath );
    if ( udgSocket.getFD() == ERROR )
    {
	cerr << "couldn't get socket fd" << endl;
	return ERROR;
    }

    // send the message to the ird daemon
    int l = strlen( s ) + 1;
    if ( udgSocket.send( s, l ) != l )
    {
	perror( "killIRd sendto()" );
	return ERROR;
    }

    // close shop & go home
    return OK;
}


#if defined( NO_ZOMBIES_ALLOWED )
static void
childDeath( void )
{
    wait( (int *)0 );
}
#endif  // NO_ZOMBIES_ALLOWED


//
// 
//
static void
markLog( char *m )
{
    char	s[ 255 ];
    time_t	clock;

    time( &clock );
    strcpy( s, m );
    strcat( s, ctime( &clock ) );
    cerr << s << endl;
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    int		index;

    // process optional arguments
    for ( index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		++verbose;
		break;

	    case 'k':
		if ( killIRd() ) exit( 1 );
		else exit( 0 );
		break;

	    default:
		cerr << form( "usage: %s [-kv]", argv[ 0 ] ) << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
