//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: irdaemon.cc,v 1.2 1996/03/31 02:12:02 smp Exp smp $";


//
// irdaemon.cc
//

#if defined( HAVE_IR )

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/keymap.hh"
#include "util/misc/tokens.hh"
#include "../daemon/rcserver.hh"
#include "irdaemon.hh"


IR::IR( void )
{
    // 
    if ( _defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << form( "couldn't load necessary file: %s",
		      RCD_STRINGS ) << endl;
	terminate();
    }

    if ( openSerial() == ERROR )
    {
	perror( "openSerial()" );
	terminate();
    }

    // find and read the button database
    const char*	s = _defaults.getString( "BINIRKEYMAP" );
    if ( s == NULL )
    {
	cerr << "couldn't find IRKEYMAP" << endl;
	terminate();
    }

    /// if ...
    if ( _irMap.readMap( (char*)s ) == ERROR )
    {
	cerr << "couldn't find read " << s << endl;
	terminate();
    }

    return;
}


IR::~IR( void )
{
    cerr << "\nirDaemon stopped" << endl;
}


//
// cleanup
//
void
IR::terminate( void )
{
    cerr << "irDaemon terminated" << endl;

    exit( 0 );
}


///////////////////////////////////////////////////////////////////////////////
# if HAVE_IR == SIR

//
// Serial IR code
//

// bitfields within output bytes
#define IR_STSP		0x12			// start/stop bits: x10x10x
#define IR_DMASK	0x49			// data bits: dxxdxxd
#define IR_1		0x01
#define IR_2		0x08
#define IR_3		0x40
#define IR_LOW		(IR_STSP)		// 3 low bits
#define IR_HIGH		(IR_STSP|IR_DMASK)	// 3 high bits
#define IR_LLH		(IR_STSP|IR_3)		// low, low, high
#define IR_LHH		(IR_STSP|IR_2|IR_3)	// low, high, high
#define IR_HLL		(IR_STSP|IR_1)		// high, low, low
#define IR_HHL		(IR_STSP|IR_1|IR_2)	// high, high, low

#define IR_BITS_PER_BYTE	3

#define BY_THE_BYTE_NOT
#define HAPPY_TV

# if defined( HAPPY_TV )
#define LO_ADJ	5
#define HI_ADJ	-5
# else
#define LO_ADJ	0
#define HI_ADJ	0
#endif


//
// break string up into individual commands and send each
//
int
IR::send( const char* commands )
{
    // get a local copy of commands, Tokens corrupts it!
    char* s = new char[ strlen( commands ) + 1 ];
    strcpy( s, commands );

    // break string up into individual words
    Tokens& tokens = *new Tokens( s );
    int argc = tokens.count();

    for ( int x = 0; x < argc;  )
    {
	// filter comments
	if ( tokens[x][0] == '#' )
	{
	    x = argc;
	    break;
	}

	command( tokens[ x ] );
	if ( ++x < argc ) sleep( 1 );
    }

    // return storage to system
    delete &tokens;
    delete s;

    return OK;
}


//
// 
//
int
IR::command( char* command )
{
    const u_char ir001 = IR_LLH;
    const u_char ir011 = IR_LHH;
    const u_char ir100 = IR_HLL;
    const u_char ir110 = IR_HHL;
    int x, y, bytes, rem;

    // get the IR data sample
    short sample[ 1024 ];
    _irMap.getKey( command, sample );
    int count = (int)sample[ 0 ];

    cerr << form( "-> %-20s", command );
    time_t	clock;
    time( &clock );
    cerr << ctime( &clock );

#if 0
    cerr << "count: " << count << endl;
    for ( x = 1, y = count; x <= y; ++x )
    { cerr << form( " %3d", sample[ x ] ); if ( (x % 20) == 0 ) cerr << endl; }
    cerr << endl;
#endif

    if ( count & 1 )
    {
	// odd number of samples, add a final IR_LOW count
	sample[ ++count ] = 500;
    }

#if defined( BY_THE_BYTE )

    for ( x = 1, y = count; x <= y; x += 2 )
    {
	irHiBits( sample[  x  ] - 1 );
	irLoBits( sample[ x+1 ] + 1 );
    }

#else  // BY_THE_BYTE

    for ( x = 1, y = count; x <= y; ++x )
    {
	bytes = (sample[ x ] + HI_ADJ) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + HI_ADJ) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
#if defined( HAPPY_TV )
	if ( rem >= 2 ) ++bytes;
#endif
	irHiBits( bytes );

#if !defined( HAPPY_TV )
	switch( rem )
	{
	case 1:
	    write( _irPort, &ir100, 1 );		// 1 more Hi bit
	    sample[ x+1 ] -= 2;			//  plus 2 Lo bits
	    break;
	case 2:
	    write( _irPort, &ir110, 1 );		// 2 more Hi bits
	    sample[ x+1 ] -= 1;			//  plus 1 Lo bit
	    break;
	default:
	    // no rem means no more Hi bits to send
	    break;
	}
#endif

	++x;
	bytes = (sample[ x ] + LO_ADJ) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + LO_ADJ) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
#if defined( HAPPY_TV )
	if ( rem >= 2 ) ++bytes;
#endif
	irLoBits( bytes );

	if ( x == count ) break;

#if !defined( HAPPY_TV )
	switch( rem )
	{
	case 1:
	    write( _irPort, &ir011, 1 );	// 1 more Lo bit
	    sample[ x+1 ] -= 2;			//  plus 2 Hi bits
	    break;
	case 2:
	    write( _irPort, &ir001, 1 );	// 2 more Lo bits
	    sample[ x+1 ] -= 1;			//  plus 1 Hi bit
	    break;
	default:
	    // no rem means no more Hi bits to send
	    break;
	}
#endif

    }

#endif  // BY_THE_BYTE

    return OK;
}


int
IR::irLoBits( int count )
{
    const u_char irLo[] = {
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW
    };

    for ( int x = count / 128; x; --x )
    {
	write( _irPort, irLo, 128 );
    }
    write( _irPort, irLo, count % 128 );

    return OK;
}


int
IR::irHiBits( int count )
{
    const u_char irHi[] = {
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH
    };

    for ( int x = count / 128; x; --x )
    {
	write( _irPort, irHi, 128 );
    }
    write( _irPort, irHi, count % 128 );

    return OK;
}


int
IR::openSerial( void )
{
    struct termios	newTerm;

    // get the default strings
    const char*		irFile;
    if ( !(irFile = _defaults.getString( "IRTPORT" )) )
    {
	cerr << "couldn't find IRTPORT" << endl;
	return ERROR;
    }

    // open port
#if 0
    if ( (_irPort = open( irFile, O_RDWR | O_NONBLOCK )) < 0 )
#else
    if ( (_irPort = open( irFile, O_WRONLY )) < 0 )
#endif
    {
	perror( "open( IR WRITERPORT )" );
	return ERROR;
    }

    if ( tcgetattr( _irPort, &newTerm ) == ERROR ) {
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

#if 0
    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS7 );		// 7 bit words
#else
    newTerm.c_cflag &= ~( CREAD|CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CS7 );			// 7 bit words
#endif

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	irBaud;
    if ( !(irBaud = _defaults.getString( "IRTBAUD" )) )
    {
	cerr << "couldn't get IRTBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( irBaud ) ) == ERROR )
    {
	return ERROR;
    }

    if ( tcsetattr( _irPort, TCSANOW, &newTerm ) == ERROR )
    {
	return ERROR;
    }

    return OK;
}


# endif  // SIR


///////////////////////////////////////////////////////////////////////////////
# if HAVE_IR == LIR

//
// Learning IR code
//

// misc stuff
#define IR_KEY_NUM		108
#define IR_ACK			0x5a

//
// fields within the irx controller command byte
//
#define IR_DEVICE_FIELD		0xc0
#define IR_DEVICE_SHIFT		6
#define IR_COLUMN_FIELD		0x30
#define IR_COLUMN_SHIFT		4
#define IR_ROW_FIELD		0x0f
#define IR_ROW_SHIFT		0


//
// the serial port initialization function
//
int
IR::openSerial( void )
{
    struct termios	newTerm;

    // get the default strings
    const char*		irFile;
    if ( !(irFile = _defaults.getString( "IRPORT" )) )
    {
	cerr << "couldn't find IRPORT" << endl;
	return ERROR;
    }

    // open port
    if ( (_irPort = open( irFile, O_RDWR | O_NONBLOCK )) < 0 )
    {
	perror( "open( IR WRITERPORT )" );
	return ERROR;
    }

    if ( tcgetattr( _irPort, &newTerm ) == ERROR ) return ERROR;

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSIZE|CSTOPB|HUPCL|PARENB );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	irBaud;
    if ( !(irBaud = _defaults.getString( "IRBAUD" )) )
    {
	cerr << "couldn't get IRBAUD" << endl;
	return BAD;
    }
    if ( cfsetispeed( &newTerm, atoi( irBaud ) ) == ERROR ) return ERROR;
    if ( cfsetospeed( &newTerm, atoi( irBaud ) ) == ERROR ) return ERROR;

    if ( tcsetattr( _irPort, TCSANOW, &newTerm ) == ERROR ) return ERROR;

    return OK;
}


int
IR::send( u_char* buf )
{
    int		status, device;
    u_char	command, ack;

    // buf[ 0 ] has 'unit' number, for future use...

    device = buf[1];
    command = ((device / 27) << IR_DEVICE_SHIFT) & IR_DEVICE_FIELD;;
    device %= 27;
    command |= ((device / 9) << IR_COLUMN_SHIFT) & IR_COLUMN_FIELD;
    command |= ((device % 9) << IR_ROW_SHIFT) & IR_ROW_FIELD;

    write( _irPort, &command, 1 );
    status = read( _irPort, &ack, 1 );			// wait for ACK signal
    if ( (status != 1) || (ack != IR_ACK ) )
    {
	perror( "read ERROR" );
	return ERROR;
    }

    return OK;
}

# endif  // LIR

#endif  // HAVE_IR
