//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//
// portions derived from:
//    ird.c: 94/05/16 john h. dubois iii (john@armory.com)
//

static char rcsid[] = "$Id: irdump.c,v 1.1 1995/11/07 07:18:30 smp Exp smp $";


//
// irdump.cc
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/rcserver.hh"

#include "ir.hh"


#define DUMP_BUF_SIZE		1024
#define LONG_PAUSE		255
#define DEFAULT_DUMP		255

static int	verbose = 0;
static int	irTPort;
static NamedStr defaults;

static int	hiAdjust = 0;
static int	loAdjust = 0;

static int	doArgs( int, char** );
static int	openTransmitter( void );
static int	dumpSample( char*, int );

static int	irCommand( short* sample, int port );
static int	irHiBits( int count, int port );
static int	irLoBits( int count, int port );


#if defined( COMMENT )

KEY: chUp
SAMPLE: 20 12 36 12 21 12 21 12 20 13 20 13 20 28 35 13 20 13 20 13 20 13 20

#endif  // COMMENT


main( int argc, char *argv[] )
{
    // process command line args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // get the name of the RC 'defaults' file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << endl;
	return BAD;
    }

    // open the serial port
    if ( openTransmitter() == ERROR )
    {
	cerr << "openTransmitter() failure!" << endl;
	return BAD;
    }

    // dump samples
    int count;
    char sample[ MAX_SAMPLE ];
    while ( fgets( sample, MAX_SAMPLE-1, stdin ) )
    {
	if ( dumpSample( sample, irTPort ) == ERROR )
	{
	    cerr << "problem dumping sample!" << endl;
	    return BAD;
	}
    }

    // close shop & go home
    return OK;
}


//
// 
//
static int
dumpSample( char* sample, int port )
{
    // break string up into individual words
    Tokens& tokens = *new Tokens( sample, MAX_SAMPLE );
    int argc = tokens.count();

    short samples[ MAX_SAMPLE + 1 ];
    samples[ 0 ] = argc;
    for ( int x = 0; x < argc; ++x )
    {
	samples[ x+1 ] = atoi( tokens[ x ] );
    }

    // return storage to system
    delete &tokens;

    // send data
    char s[ 80 ];
    short scratch[ MAX_SAMPLE + 1 ];
    for (  ;;  )
    {
#if 1
	memcpy( scratch, samples, (argc+1) * sizeof( short ) );
	irCommand( scratch, port );
#else
	irHiBits( 118/3, port );
	irLoBits( 33/3, port );
	irHiBits( 240/3, port );
	irLoBits( 33/3, port );
	irHiBits( 45/3, port );
	irLoBits( 33/3, port );
	irHiBits( 90/3, port );
	irLoBits( 33/3, port );
	irHiBits( 240/3, port );
	irLoBits( 240/3, port );
#endif
	gets( s );
    }

    // all  done
    return OK;
}


///////////////////////////////////////////////////////////////////////////////
// FIX ME
// this code from ird/irdaemon.c, irdump.c, merge back someday

//
// Serial IR code
//

// bitfields within output bytes
#define IR_STSP		0x12			// start/stop bits: x10x10x
#define IR_DMASK	0x49			// data bits: dxxdxxd
#define IR_1		0x01
#define IR_2		0x08
#define IR_3		0x40
#define IR_LOW		(IR_STSP)		// 3 low bits
#define IR_HIGH		(IR_STSP|IR_DMASK)	// 3 high bits
#define IR_LLH		(IR_STSP|IR_3)		// low, low, high
#define IR_LHH		(IR_STSP|IR_2|IR_3)	// low, high, high
#define IR_HLL		(IR_STSP|IR_1)		// high, low, low
#define IR_HHL		(IR_STSP|IR_1|IR_2)	// high, high, low

#define IR_BITS_PER_BYTE	3

#define HAPPY_TV_NOT

//
// 
//
static int
irCommand( short* sample, int port )
{
    const u_char ir001 = IR_LLH;
    const u_char ir011 = IR_LHH;
    const u_char ir100 = IR_HLL;
    const u_char ir110 = IR_HHL;
    int x, y, bytes, rem;

#if 0
    cerr << "count: " << count << endl;
    for ( x = 1; x <= count; ++x ) { cerr << form( " %3d", sample[ x ] ); if ( (x % 20) == 0 ) cerr << endl; }
    cerr << endl;
#endif

#if defined( HAPPY_TV )

    // 
    int count = (int)sample[ 0 ];

    for ( x = 1, y = count; x <= y; ++x )
    {
	bytes = (sample[ x ] + hiAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + hiAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	irHiBits( bytes, port );

	++x;
	bytes = (sample[ x ] + loAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + loAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	irLoBits( bytes, port );

	if ( x == count ) break;
    }
    if ( count & 1 )
    {
	// odd number of samples, add a final IR_LOW count
	irLoBits( 200, port );
    }

#else  // HAPPY_TV

    // 
    int count = (int)sample[ 0 ];
    int alreadySent = 0;
    int bits;
    for ( x = 1; x <= count; ++x )
    {
	bits = (sample[ x ] - alreadySent) + hiAdjust;
	bytes = bits / IR_BITS_PER_BYTE;
	rem = bits % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	/// cerr << "s: " << sample[x] << "b: " << bits << ", B: " << bytes << ", r: " << rem << endl;
	irHiBits( bytes, port );

	switch( rem )
	{
	case 1:
	    write( port, &ir100, 1 );		// 1 more Hi bit
	    alreadySent = 2;			//  plus 2 Lo bits
	    break;
	case 2:
	    write( port, &ir110, 1 );		// 2 more Hi bits
	    alreadySent = 1;			//  plus 1 Lo bit
	    break;
	default:
	    // no rem means no more Hi bits to send
	    alreadySent = 0;
	    break;
	}

	++x;
	bits = (sample[ x ] - alreadySent) + loAdjust;
	bytes = bits / IR_BITS_PER_BYTE;
	rem = bits % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}

	irLoBits( bytes, port );

	switch( rem )
	{
	case 1:
	    write( port, &ir011, 1 );	// 1 more Lo bit
	    alreadySent = 2;		//  plus 2 Hi bits
	    break;
	case 2:
	    write( port, &ir001, 1 );	// 2 more Lo bits
	    alreadySent = 1;		//  plus 1 Hi bit
	    break;
	default:
	    // no rem means no more Lo bits to send
	    alreadySent = 0;
	    break;
	}
    }

    if ( count & 1 )
    {
	// odd number of samples, add a final IR_LOW count
	irLoBits( 100, port );
    }

#endif  // !HAPPY_TV

    return OK;
}


//
// 
//
static int
irLoBits( int count, int port )
{
    const u_char irLo[] = {
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW
    };

    int blocks = count / 128;
    int rem = count % 128;
    for ( int x = blocks; x; --x )
    {
	write( port, irLo, 128 );
    }
    if ( rem ) write( port, irLo, rem );

    return OK;
}


//
// 
//
static int
irHiBits( int count, int port )
{
    const u_char irHi[] = {
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH
    };

    int blocks = count / 128;
    int rem = count % 128;
    /// cerr << "b: " << blocks << ", r: " << rem << endl;

    for ( int x = blocks; x; --x )
    {
	write( port, irHi, 128 );
    }
    if ( rem ) write( port, irHi, rem );

    return OK;
}
///////////////////////////////////////////////////////////////////////////////


//
// 
//
static int
openTransmitter( void )
{
    struct termios	newTerm;

    // get the default strings
    const char*		irFile;
    if ( !(irFile = defaults.getString( "IRTPORT" )) )
    {
	cerr << "couldn't find IRTPORT" << endl;
	return ERROR;
    }

    // open port
#if 0
    if ( (irTPort = open( irFile, O_RDWR | O_NONBLOCK )) < 0 )
#else
    if ( (irTPort = open( irFile, O_WRONLY )) < 0 )
#endif
    {
	perror( "openTransmitter(): open()" );
	return ERROR;
    }

    if ( tcgetattr( irTPort, &newTerm ) == ERROR ) {
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS7 );		// 7 bit words

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	irBaud;
    if ( !(irBaud = defaults.getString( "IRTBAUD" )) )
    {
	cerr << "couldn't get IRTBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( irBaud ) ) == ERROR )
    {
	return ERROR;
    }

    if ( tcsetattr( irTPort, TCSANOW, &newTerm ) == ERROR )
    {
	return ERROR;
    }

    return OK;
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'h':
		hiAdjust = atoi( argv[ ++index ] );
		break;

	    case 'l':
		loAdjust = atoi( argv[ ++index ] );
		break;

	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << "usage: " << argv[ 0 ] << " [-mv]" << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
