//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: irnamer.c,v 1.1 1995/11/07 07:18:30 smp Exp smp $";


//
// irnamer.cc
//

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/rcserver.hh"

#include "ir.hh"


static int	verbose = 0;

static int	doArgs( int, char** );
static int	nameKeys( const char* );

#if defined( CAN_SAMPLE )
static int	sampleKeys( const char* );
#endif  // CAN_SAMPLE


main( int argc, char *argv[] )
{
    // get args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // get the file paths from the RC defaults file
    NamedStr defaults;
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << endl;
	return BAD;
    }
    const char*	irKeyMapFile;
    if ( !(irKeyMapFile = defaults.getString( "IRKEYMAP" )) )
    {
	cerr << "couldn't get IRKEYMAP" << endl;
	return BAD;
    }

    // name appropriate keys
    if ( nameKeys( irKeyMapFile ) == ERROR )
    {
	return BAD;
    }

#if defined( CAN_SAMPLE )
    // name appropriate keys
    if ( sampleKeys( irKeyMapFile ) == ERROR )
    {
	cerr << "problem sampling buttons!" << endl;
	return BAD;
    }
#endif  // CAN_SAMPLE

    // close shop & go home
    return OK;
}


static int
nameKeys( const char* file )
{
    int		x, n, key;
    char	s[ MAX_INPUT ];
    char	s2[ MAX_INPUT ];
    char*	p;
    char*	p2;
    char	areaName[ MAX_INPUT ];
    char	deviceName[ MAX_INPUT ];
    char	keyName[ MAX_INPUT ];
    char	keySample[ MAX_SAMPLE ];
    FILE*	fp;
    KeyMap	keyMap;

    // open keytrain file
    if ( (fp = fopen( file, "r" )) == NULL )
    {
	cerr << form( "fopen( %s ): %s", file, strerror( errno ) );
	return ERROR;
    }

    // read keymap, building key names
    key = 0;
    while ( fgets( s, MAX_INPUT, fp ) )
    {
	// skip whitespace
	for ( p = s; isspace( *p ); ++p );

	// ignore comments
	if ( (*p == '\0') || (*p == '#') ) continue;

	// look for 'area' keyword
	if ( strncmp( p, AREA_TOKEN, strlen(AREA_TOKEN) ) == EQUAL )
	{
	    for ( p += strlen(AREA_TOKEN)+1; isspace( *p ); ++p );
	    for ( x = 0; isprint( *p ); ++x ) areaName[ x ] = *p++;
	    areaName[ x ] = '\0';
	}

	// look for 'device' keyword
	else if ( strncmp( p, DEVICE_TOKEN, strlen(DEVICE_TOKEN) ) == EQUAL )
	{
	    for ( p += strlen(DEVICE_TOKEN)+1; isspace( *p ); ++p );
	    for ( x = 0; isprint( *p ); ++x ) deviceName[ x ] = *p++;
	    deviceName[ x ] = '\0';
	}

	// look for 'key' keyword
	else if ( strncmp( p, KEY_TOKEN, strlen(KEY_TOKEN) ) == EQUAL )
	{
	    for ( p += strlen(KEY_TOKEN)+1; isspace( *p ); ++p );
	    for ( x = 0; isprint( *p ); ++x ) keyName[ x ] = *p++;
	    keyName[ x ] = '\0';
	    ++key;
	}

	// look for 'sample' keyword
	else if ( strncmp( p, SAMPLE_TOKEN, strlen(SAMPLE_TOKEN) ) == EQUAL )
	{
	    for ( p += strlen(SAMPLE_TOKEN)+1; isspace( *p ); ++p );
	    for ( x = 0; isprint( *p ); ++x ) keySample[ x ] = *p++;
	    keySample[ x ] = '\0';

	    // build name string from components
	    sprintf( s2, "%s.%s.%s", areaName, deviceName, keyName );
	    p2 = new char[ strlen( s2 ) + 1 ];
	    strcpy( p2, s2 );

	    // break string up into individual words
	    Tokens& tokens = *new Tokens( (char*)keySample, MAX_SAMPLE );
	    int argc = tokens.count();

	    short samples[ MAX_SAMPLE + 1 ];
	    samples[ 0 ] = argc;
	    for ( x = 0; x < argc; ++x )
	    {
		samples[ x+1 ] = atoi( tokens[ x ] );
	    }

	    // store the key
	    keyMap.putKey( p2, samples );

	    // return storage to system
	    delete &tokens;
	    delete p2;
	}
    }

    // close the input file
    fclose( fp );

    // create the output file
    strcat( strcpy( s, file ), ".bin" );
    keyMap.writeMap( s );

    return OK;
}


#if defined( CAN_SAMPLE )
//
// 
//
static int
sampleKeys( const char* file )
{
    return OK;
}
#endif  // CAN_SAMPLE


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << "usage: " << argv[ 0 ] << "[-v]\n";
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
