//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//
// portions derived from:
//    ird.c: 94/05/16 john h. dubois iii (john@armory.com)
//

static char rcsid[] = "$Id: irsample.c,v 1.3 1995/11/07 07:18:30 smp Exp smp $";


//
// irsample.cc
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#if 0
/// FIX ME kludge in util/misc/misc.h should be fixed!
#include <ctype.h>
#endif
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/rcserver.hh"

#include "ir.hh"

// max wait for read() is: READ_TIMEOUT * 0.1 seconds
#define READ_TIMEOUT		50		// 5 seconds

// keep samples as integers, with a multiplication factor of "FACTOR"
#define FACTOR			1000
#if defined( FACTOR )

# define SHARP_HI_OFFSET	800		// 0.800
# define SHARP_LO_OFFSET	1200		// 1.200

static int	sharpHiOffset = SHARP_HI_OFFSET;
static int	sharpLoOffset = SHARP_LO_OFFSET;


#else

# define SHARP_OFFSET		0.800

#endif  // FACTOR

#define SAMPLE_BUF_SIZE		4096

#if 0
#define LONG_PAUSE		1000
#else
#define LONG_PAUSE		255
#endif

static int	longPause = LONG_PAUSE;

#define DEFAULT_MIN		255
#define END_OF_SAMPLE		(-1)

enum commandType
{
    CMD_HELP = 1,
    CMD_AVERAGE,
    CMD_DISPLAY,
    CMD_OFFSET,
    CMD_LOOFFSET,
    CMD_HIOFFSET,
    CMD_PAUSE,
    CMD_SAMPLE,
    CMD_XSAMPLE,
    CMD_TEST,
    CMD_INPUT,
    CMD_QUIT,
    CMD_ERROR
};

static int		verbose = 0;
static int		minSample = DEFAULT_MIN;
static int		irTPort;
static int		irRPort;

static NamedStr		defaults;

static int		doArgs( int, char** );
static void		menu( void );
static commandType	getCommand( void );

static int		openTransmitter( void );
static int		openReceiver( int );

static int		doit( int, int );

static int		doSample( int file, int* sampleBuffer,
				  int* averageBuffer );
static int		sampleKey( int, int* );
static int		countEdges( int, u_char*, int* );
static int		displaySample( int* buffer );

static int		averageSamples( const int* samples, int* average );
static int		offsetSamples( int* average );
static int		inputSample( int* average );


static int		irCommand( int* sample, int port );
static int		ir3HiBits( int count, int port );
static int		ir3LoBits( int count, int port );


#if defined( COMMENT )

KEY: chUp
SAMPLE: 20 12 36 12 21 12 21 12 20 13 20 13 20 28 35 13 20 13 20 13 20 13 20

#endif  // COMMENT


//
// 
//
main( int argc, char *argv[] )
{
    // process command line args
    if ( doArgs( argc, argv ) == ERROR )
    {
	cerr << "doArgs() failure!" << endl;
	exit( 1 );
    }

    // get the name of the RC 'defaults' file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << endl;
	return BAD;
    }

    // initialize the serial ports
    if ( openTransmitter() == ERROR )
    {
	return BAD;
    }
    if ( openReceiver( minSample ) == ERROR )
    {
	return BAD;
    }

    // collect samples
    if ( doit( irRPort, irTPort ) == ERROR )
    {
	cerr << "problem sampling buttons!" << endl;
	return BAD;
    }

    // close shop & go home
    return OK;
}


//
// 
//
static int
doit( int iFd, int oFd )
{
    int status;
    commandType command;
    int sampleBuffer[ SAMPLE_BUF_SIZE ];
    int averageBuffer[ SAMPLE_BUF_SIZE ];

    for ( ;; )
    {
    top:
	// display commands
	menu();
	switch ( command = getCommand() )
	{
	case CMD_AVERAGE:
	    averageSamples( sampleBuffer, averageBuffer );
	    break;

	case CMD_OFFSET:
	    offsetSamples( averageBuffer );
	    break;

	case CMD_LOOFFSET:
	    break;

	case CMD_HIOFFSET:
	    break;

	case CMD_DISPLAY:
	    displaySample( averageBuffer );
	    break;

	case CMD_SAMPLE:
	    if ( doSample( iFd, sampleBuffer, averageBuffer ) == ERROR )
	    {
		return ERROR;
	    }
	    break;

	case CMD_XSAMPLE:
	XSample:
	    switch ( status = sampleKey( iFd, sampleBuffer ) )
	    {
	    case 0:				// end of sample
	        cerr << "sampleKey() timeout" << endl;
		break;

	    case ERROR:
		cerr << "sampleKey() returned ERROR" << endl;
		return ERROR;

	    default:
		displaySample( sampleBuffer );
		break;
	    }
	    goto XSample;

	case CMD_INPUT:
	    inputSample( averageBuffer );
	    break;

	case CMD_TEST:
	    irCommand( averageBuffer, oFd );
	    break;

	case CMD_QUIT:
	    return OK;

	default:
	    cerr << "huh?" << endl;
	    goto top;
	}

    }

    return status;
}


//
// 
//
static int
doSample( int file, int* sampleBuffer, int* averageBuffer )
{
    int status;

again:
    switch ( status = sampleKey( file, sampleBuffer ) )
    {
    case 0:				// end of sample
        cerr << "sample timeout" << endl;
	break;
	
    case ERROR:
	cerr << "sampleKey() returned ERROR" << endl;
	return ERROR;

    default:

	// grab a group of samples
	if ( displaySample( sampleBuffer ) != OK )
	{
	    cerr << "BAD sample(s), try again" << endl;
	    goto again;
	}

	// average the group
	if ( averageSamples( sampleBuffer, averageBuffer ) != OK )
	{
	    cerr << "BAD sample(s), try again" << endl;
	    goto again;
	}

	// display the averaged sample
	if ( displaySample( averageBuffer ) != OK )
	{
	    cerr << "BAD sample(s), try again" << endl;
	    goto again;
	}

	break;
    }

    return OK;
}


//
// 
//
static int
sampleKey( int fd, int* buffer )
{
    u_char readBuffer[ SAMPLE_BUF_SIZE ];
    int count = 0;

#if 1
    while( read( fd, readBuffer, SAMPLE_BUF_SIZE ) > 0 )
    {
	// flush stream
    }
#endif

    // 
    cout << " <ready to sample> " << endl;

    // 
    int cnt;
    for ( int x = 0; x < SAMPLE_BUF_SIZE; x += cnt, count += cnt )
    {
	cnt = read( fd, &readBuffer[x], SAMPLE_BUF_SIZE - x );
	if ( cnt == -1 )
	{
	    perror( "read" );
	    break;
	}
	if ( cnt == 0 ) break;
    }
    if ( verbose >= 1 )
    {
	cerr << form( "\ncount: %ld", count ) << endl;
    }

    // 
    switch ( count )
    {
    case 0:				// end of sample
	return 0;

    case -1:
	perror( "sampleKey()" );
	return ERROR;

    default:
#if 0
	cerr << "count: " << count << endl;
	for ( x = 0; x < count; ++x )
	{
	    if ( (x % 20) == 0 ) cerr << endl;
	    cerr << form( " %3x", readBuffer[ x ] );
	}
	cerr << endl;
#endif
	countEdges( count, readBuffer, buffer );
	return count;
    }
}


#if 0

volUp from VSX-95:



160 76 12  7 11  7 12 27 11  7 12  8 11 27 11  8 11 27 12 27 11 27 11  7 12 27
159 76 12 27 12 7 12 27 12 8 12 8 11 27 12 8 12 28 11 8 11 28 11 8 12 28 11 27
159 74 13 26 13 7 12 27 12 7 13 7 12 27 12 7 13 27 12 7 13 26 12 8 12 27 12 27

 11 27 11  8 11 27 11  8 11  8 11 27 11 27 11  8 11 27 11  8 11 27 11  8 11 27
 12 8 11 28 11 8 12 8 12 27 12 8 11 27 12 8 12 8 11 8 12 8 11 28 11 8 12 27 12
 12 7 12 27 12 8 12 7 12 27 12 8 12 27 12 7 12 8 12 7 12 8 12 27 12 7 12 27 12

11  8 11  7 12 27 11  7 12 28 11  8 11 28 11
 8 11 28 11 28 11 28 11 28 12
 8 12 27 12 27 12 27 12 27 12
#endif

//
// 
//
static int
countEdges( int number, u_char* inBuffer, int* outBuffer )
{
    static int bitCount = 0;	// # of bits passed since last falling edge
    static int lastBit = 1;	// last bit seen, expecting first bits are hi
    const u_char allHi = 0xff;	// byte with all bits Hi
    const u_char allLo = 0x00;	// byte with all bits Lo
    int bx = 0;

    if ( verbose >= 2 )
    {
	cerr << form( "initial number: %d", number ) << endl;
    }

    while ( number )
    {
	// search for first byte with bit different from last bit seen
	u_char same = lastBit ? allHi : allLo;
	while ( number && (*inBuffer == same) )
	{
	    bitCount += 8;
	    --number;
	    ++inBuffer;
	}
	if ( number == 0 ) break;

	// byte has at least one edge in it
	u_char byte = *(inBuffer++);
	for ( int bitsLeft = 8; bitsLeft--; byte >>= 1 )
	{
	    if ( (byte & 1) != lastBit )		// edge found
	    {
		if ( !lastBit && bitCount >= longPause )
		{
		    // ignore if LONG_PAUSE is first thing encountered
		    if ( bx > 0 ) outBuffer[ ++bx ] = END_OF_SAMPLE;
		}
		else
		{
		    outBuffer[ ++bx ] = bitCount;
		}

		lastBit = !lastBit;
		bitCount = 0;
	    }

	    ++bitCount;
	}
	--number;
    }

    // terminate list and record count
    if ( bx > 0 ) outBuffer[ ++bx ] = END_OF_SAMPLE;
    outBuffer[ 0 ] = bx;

#if defined( FACTOR )
    /// multiply by FACTOR
    for ( int x = 1; x <= bx; ++x )
    {
	if ( outBuffer[ x ] != END_OF_SAMPLE )
	{
	    outBuffer[ x ] *= FACTOR;
	}
    }
#endif  // FACTOR

    // return edge count
    return bx;
}


//
// 
//
static int
displaySample( int* buffer )
{
    int sampleCount = 0;
    int count = buffer[ 0 ];
    int x;

    // count the samples in buffer
    for ( x = 1; x <= (count+1); ++x )
    {
	if ( buffer[ x ] == END_OF_SAMPLE )
	{
	    ++sampleCount;
	}
    }

    // look for display of average
    if ( (sampleCount == 1) && (x == count+2) )
    {
	cout << form( " average:\n\n" );
    }
    else
    {
	cout << form( " %d sample(s) found:\n\n", sampleCount );
    }

    if ( sampleCount == 0 )
    {
	return ERROR;
    }

    // print each sample
    for ( x = 1; x <= count; ++x )
    {
	if ( buffer[ x ] == END_OF_SAMPLE )
	{
	    cout << "\n" << endl;
	    /// cerr << "END_OF_SAMPLE" << endl;
	}
	else if ( buffer[ x ] == 0 )
	{
	    cout << endl;
	    cerr << "unexpected ZERO count in buffer!" << endl;
	    return ERROR;
	}
	else
	{
#if defined( FACTOR )
	    cout << form("%2d ", buffer[ x ] / FACTOR );
#else  // FACTOR
	    cout << form("%2d ", buffer[ x ] );
#endif  // FACTOR
	}
    }

    if ( buffer[ x-1 ] != END_OF_SAMPLE )
    {
	cout << "\n" << endl;
    }

    return OK;
}


//
// 
//
static int
averageSamples( const int* samples, int* average )
{
    int	sampleCount = 0;
    int	count = samples[ 0 ];
    int x, y;

    // calculate number of samples in buffer
    for ( x = 1; x <= count; ++x )
    {
	if ( samples[ x ] == END_OF_SAMPLE )
	{
	    ++sampleCount;
	}
    }
    cout << form( " %d sample(s) averaged\n", sampleCount );
    if ( sampleCount == 0 ) return ERROR;

    // clear the average buffer
    for ( x = 0; x <= count; ++x )
    {
	average[ x ] = 0;
    }

    // add the samples
    int length = -1;
    for ( x = 1, y = 1; x <= count; ++x )
    {
	if ( samples[ x ] == END_OF_SAMPLE )
	{
	    if ( length == -1 )		// the 1st sample
	    {
		length = y;
#if defined( ONE_SAMPLE )
		x = count+1;
#endif  // ONE_SAMPLE
	    }
	    else
	    {
		if ( y != length )	// different length than 1st sample
		{
		    cerr << "unequal samples!" << endl;
		    return ERROR;
		}
	    }
	    y = 1;				// recock average index
	}
	else
	{
	    average[ y++ ] += samples[ x ];
	}
    }
    average[ 0 ] = length - 1;
    average[ length ] = END_OF_SAMPLE;

    // average the samples
#if !defined( ONE_SAMPLE )
    for ( x = 1; x <= length; ++x )
    {
	if ( average[ x ] == END_OF_SAMPLE )
	{
	    break;
	}
	else
	{
	    average[ x ] /= sampleCount;
	}
    }
#endif  // ONE_SAMPLE

#if 0
    for ( x = 0; x <= average[ 0 ]; ++x )
    {
	cerr << form( " %d", average[ x ] );
    }
    cerr << endl;
#endif

    return OK;
}


//
// 
//
static int
offsetSamples( int* average )
{
    int	count = average[ 0 ];
    int x;

    // 
    for ( x = 1; x <= count; ++x )
    {
	if ( average[ x ] == END_OF_SAMPLE )
	{
	    break;
	}
	else
	{
	    // Sharp IR device has differnet rise and fall times,
	    //  samples need adjustment
	    //  we dont adjust the first Hi since it behaves differently
	    //  than values that are "midstream"

#if defined( FACTOR )
	    /// cerr << "a[" << x << "]: " << average[x];
	    int temp;
	    if ( x & 1 )		// Hi value
	    {
		temp = average[ x ] * sharpHiOffset / FACTOR;
	    }
	    else			// Lo value
	    {
		temp = average[ x ] * sharpLoOffset / FACTOR;
	    }
	    average[ x ] = ((temp + (FACTOR/2)) / FACTOR) * FACTOR;
	    /// cerr << ", a[" << x << "]: " << average[x] << endl;
#else  // FACTOR

	    if ( (x > 1) && (x & 1) )	// NOT first and Hi value
	    {
		double temp = ((double)average[ x ] / count) * SHARP_OFFSET;
		average[ x ] = (int)floor( temp + 0.5 );
	    }
	    else			// Lo value
	    {
		average[ x ] = (int)floor( ((double)average[x]/count) + 0.5 );
	    }

#endif  // FACTOR

	}
    }

    cout << "\n offset";
    displaySample( average );

    return OK;
}


//
// 
//
static int
inputSample( int* average )
{
    int	count = 0;
    int x;

    // 
    for ( x = 1;  ; ++x )
    {
	cin >> average[ x ];
	if ( average[ x ] == -1 ) break;
#if defined( FACTOR )
	average[ x ] *= FACTOR;
#endif
    }

    average[ x ] = END_OF_SAMPLE;
    average[ 0 ] = x;

    cout << "\n input";
    displaySample( average );

    return OK;
}


///////////////////////////////////////////////////////////////////////////////
// FIX ME
// this code from ird/irdaemon.c, irdump.c, merge back someday

//
// Serial IR code
//

// bitfields within output bytes
#define IR_STSP			0x12		// start/stop bits: x10x10x
#define IR_DMASK		0x49		// data bits: dxxdxxd
#define IR_1			0x01		// first bit
#define IR_2			0x08		// second bit
#define IR_3			0x40		// third bit
#define IR_LOW			(IR_STSP)		// 3 low bits
#define IR_HIGH			(IR_STSP|IR_DMASK)	// 3 high bits
#define IR_LLH			(IR_STSP|IR_3)		// low, low, high
#define IR_LHH			(IR_STSP|IR_2|IR_3)	// low, high, high
#define IR_HLL			(IR_STSP|IR_1)		// high, low, low
#define IR_HHL			(IR_STSP|IR_1|IR_2)	// high, high, low

#define IR_BITS_PER_BYTE	3
#define FINAL_LOW_BITS		100

#define HAPPY_TV_NOT

//
// 
//
static int
irCommand( int* sample, int port )
{
    const u_char ir001 = IR_LLH;
    const u_char ir011 = IR_LHH;
    const u_char ir100 = IR_HLL;
    const u_char ir110 = IR_HHL;
    int x, y;

#if 0
    cerr << "count: " << sample[ 0 ] << endl;
    for ( x = 1; x <= sample[ 0 ]; ++x )
    {
	cerr << form( " %3d", sample[ x ] / FACTOR );
	if ( (x % 20) == 0 ) cerr << endl;
    }
    cerr << endl;
#endif

    // 
    int count = sample[ 0 ];
    int alreadySent = 0;
    int bits, bits3, finalBits;

    // send each sample
    for ( x = 0; x <= count;  )		// x == 0 -> count
    {
	// send Hi sample
	bits = (sample[ ++x ] / FACTOR) - alreadySent;
	if ( (bits3 = bits / IR_BITS_PER_BYTE) == 0 )
	{
	    bits3 = 1;
	    finalBits = 0;
	}
	else
	{
	    finalBits = bits % IR_BITS_PER_BYTE;
	}

	// send all Hi bytes
	ir3HiBits( bits3, port );

	// finish Hi sample
	switch( finalBits )
	{
	case 1:
	    write( port, &ir100, 1 );		// 1 more Hi bit
	    alreadySent = 2;			//  plus 2 Lo bits
	    break;
	case 2:
	    write( port, &ir110, 1 );		// 2 more Hi bits
	    alreadySent = 1;			//  plus 1 Lo bit
	    break;
	default:
	    // no finalBits means no more Hi bits to send
	    alreadySent = 0;
	    break;
	}

	// send the Lo sample
	bits = (sample[ ++x ] / FACTOR) - alreadySent;
	bits3 = bits / IR_BITS_PER_BYTE;
	finalBits = bits % IR_BITS_PER_BYTE;
	if ( bits3 == 0 )
	{
	    bits3 = 1;
	    finalBits = 0;
	}

	// send all Lo bytes
	ir3LoBits( bits3, port );

	// finish Lo sample
	switch( finalBits )
	{
	case 1:
	    write( port, &ir011, 1 );	// 1 more Lo bit
	    alreadySent = 2;		//  plus 2 Hi bits
	    break;
	case 2:
	    write( port, &ir001, 1 );	// 2 more Lo bits
	    alreadySent = 1;		//  plus 1 Hi bit
	    break;
	default:
	    // no finalBits means no more Lo bits to send
	    alreadySent = 0;
	    break;
	}
    }

    if ( count & 1 )
    {
	// odd number of samples (expected), add a final IR_LOW count
	ir3LoBits( FINAL_LOW_BITS / IR_BITS_PER_BYTE, port );
    }

    return OK;
}

/// OLD code:
#if defined( HAPPY_TV )
    // 
    int count = (int)sample[ 0 ];

    for ( x = 1, y = count; x <= y; ++x )
    {
	bytes = (sample[ x ] + hiAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + hiAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	ir3HiBits( bytes, port );

	++x;
	bytes = (sample[ x ] + loAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + loAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	ir3LoBits( bytes, port );

	if ( x == count ) break;
    }
    if ( count & 1 )
    {
	// odd number of samples, add a final IR_LOW count
	ir3LoBits( 200, port );
    }
#endif  // HAPPY_TV



//
// 
//
static int
ir3HiBits( int count, int port )
{
    const u_char irHi[] = {
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH
    };

    int blocks = count / 128;
    int rem = count % 128;

    for ( int x = blocks; x; --x )
    {
	write( port, irHi, 128 );
    }
    if ( rem ) write( port, irHi, rem );

    return OK;
}


//
// 
//
static int
ir3LoBits( int count, int port )
{
    const u_char irLo[] = {
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW
    };

    int blocks = count / 128;
    int rem = count % 128;

    for ( int x = blocks; x; --x )
    {
	write( port, irLo, 128 );
    }
    if ( rem ) write( port, irLo, rem );

    return OK;
}


///////////////////////////////////////////////////////////////////////////////

static void
menu( void )
{
    cout << "\n <d>isplay <sS>ample <o>ffset <t>est <i>nput <p>ause <q>uit <?>: ";
}


static commandType
getCommand( void )
{
    static commandType lastCommand = CMD_ERROR;
    char	s[ 80 ];
    float val;

    fgets( s, 79, stdin );

    if ( isascii( s[0] ) && isupper( s[0] ) )
    {
	s[0] = toupper( s[0] );
    }

    switch( s[ 0 ] )
    {
    case '\n':
	return lastCommand;

///    case 'a':
///	return (lastCommand = CMD_AVERAGE);

    case 'd':
	return (lastCommand = CMD_DISPLAY);

    case 'o':
	return (lastCommand = CMD_OFFSET);

    case 'l':
	sscanf( s+1, "%f", &val );
	sharpLoOffset = (int)(val * FACTOR);
	cerr << "new Lo Offset: " << sharpLoOffset << endl;
	return (lastCommand = CMD_LOOFFSET);

    case 'h':
	sscanf( s+1, "%f", &val );
	sharpHiOffset = (int)(val * FACTOR);
	cerr << "new Hi Offset: " << sharpHiOffset << endl;
	return (lastCommand = CMD_HIOFFSET);

    case 's':
	return (lastCommand = CMD_SAMPLE);

    case 'S':
	return (lastCommand = CMD_XSAMPLE);

    case 't':
	return (lastCommand = CMD_TEST);

    case 'i':
	return (lastCommand = CMD_INPUT);

    case 'p':
	sscanf( s+1, "%d", &longPause );
	cerr << "new longPause: " << longPause << endl;
	return (lastCommand = CMD_PAUSE);

    case 'q':
	return (lastCommand = CMD_QUIT);

    default:
	return (lastCommand = CMD_ERROR);
    }
}

//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'l':
		sharpLoOffset =  atoi( argv[ ++index ] );
		break;

	    case 'h':
		sharpHiOffset =  atoi( argv[ ++index ] );
		break;

	    case 'm':
		minSample = atoi( argv[ ++index ] );
		if ( minSample > 255 ) minSample = 255;
	        break;

	    case 'p':
		longPause = atoi( argv[ ++index ] );
		break;

	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << "usage: " << argv[ 0 ]
		     << " [-v][-m <size>][-l <offset>][-h <offset>]" << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}


//
// 
//
static int
openTransmitter( void )
{
    struct termios	newTerm;

    // get the default strings
    const char*		irFile;
    if ( !(irFile = defaults.getString( "IRTPORT" )) )
    {
	cerr << "couldn't find IRTPORT" << endl;
	return ERROR;
    }

    // open port
#if 0
    if ( (irTPort = open( irFile, O_RDWR | O_NONBLOCK )) < 0 )
#else
    if ( (irTPort = open( irFile, O_WRONLY )) < 0 )
#endif
    {
	perror( "openTransmitter(): open()" );
	return ERROR;
    }

    if ( tcgetattr( irTPort, &newTerm ) == ERROR ) {
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS7 );		// 7 bit words

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	irBaud;
    if ( !(irBaud = defaults.getString( "IRTBAUD" )) )
    {
	cerr << "couldn't get IRTBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( irBaud ) ) == ERROR )
    {
	return ERROR;
    }

    if ( tcsetattr( irTPort, TCSANOW, &newTerm ) == ERROR )
    {
	return ERROR;
    }

    return OK;
}


//
// 
//
static int
openReceiver( int minimum )
{
    struct termios newTerm;

    // get the serial irRPort name
    const char* irFile;
    if ( !(irFile = defaults.getString( "IRRPORT" )) )
    {
	cerr << "couldn't find IRRPORT" << endl;
	return ERROR;
    }

    // open the serial port
    if ( (irRPort = open( irFile, O_RDONLY )) < 0 )
    {
	cerr << form( "open( %s )", irFile ) << endl;
	return ERROR;
    }

    if ( tcgetattr( irRPort, &newTerm ) == -1 )
    {
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 0;		// VTIME is a read timer
    newTerm.c_cc[VTIME] = READ_TIMEOUT;

    newTerm.c_iflag &= ~( BRKINT|
                          IGNPAR|PARMRK|INPCK|ISTRIP|
                          INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
                          ISIG|
                          ICANON|IEXTEN|
                          NOFLSH|
                          TOSTOP);

    // get the desired baudrate
    const char*	irRBaud;
    if ( !(irRBaud = defaults.getString( "IRRBAUD" )) )
    {
	cerr << "couldn't get IRRBAUD" << endl;
	return ERROR;
    }

    if ( cfsetispeed( &newTerm, atoi( irRBaud ) ) == -1 )
    {
	return ERROR;
    }

    if ( tcsetattr( irRPort, TCSANOW, &newTerm ) == -1 )
    {
	return ERROR;
    }

    return OK;
}


