//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//
// portions derived from:
//    ird.c: 94/05/16 john h. dubois iii (john@armory.com)
//

static char rcsid[] = "$Id: irsample.c,v 1.3 1995/11/07 07:18:30 smp Exp smp $";


//
// irsend.cc
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#if 0
/// FIX ME kludge in util/misc/misc.h should be fixed!
#include <ctype.h>
#endif
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/rcserver.hh"

#include "ir.hh"

// keep samples as integers, with a multiplication factor of "FACTOR"
#define FACTOR			1000
#if defined( FACTOR )

# define SHARP_HI_OFFSET	800		// 0.800
# define SHARP_LO_OFFSET	1000		// 1.000

static int	sharpHiOffset = SHARP_HI_OFFSET;
static int	sharpLoOffset = SHARP_LO_OFFSET;

#else

# define SHARP_OFFSET		0.800

#endif  // FACTOR

#define SAMPLE_BUF_SIZE		4096

#if 0
#define LONG_PAUSE		1000
#else
#define LONG_PAUSE		255
#endif

#define DEFAULT_MIN		255
#define END_OF_SAMPLE		(-1)

enum commandType
{
    CMD_HELP = 1,
    CMD_AVERAGE,
    CMD_DISPLAY,
    CMD_OFFSET,
    CMD_LOOFFSET,
    CMD_HIOFFSET,
    CMD_SAMPLE,
    CMD_XSAMPLE,
    CMD_TEST,
    CMD_INPUT,
    CMD_DUMP,
    CMD_QUIT,
    CMD_ERROR
};

static int		verbose = 0;
static int		minSample = DEFAULT_MIN;
static int		irTPort;
static int		irRPort;

static NamedStr		defaults;

static int		doArgs( int, char** );
static void		menu( void );
static commandType	getCommand( void );

static int		openTransmitter( void );

static int		doit( int );

static int		displaySample( int* buffer );

static int		offsetSamples( int* average );
static int		inputSample( int* average );

static int		irCommand( int* sample, int port );
static int		ir3HiBits( int count, int port );
static int		ir3LoBits( int count, int port );

static int		irStream( int port );

#if defined( COMMENT )

KEY: chUp
SAMPLE: 20 12 36 12 21 12 21 12 20 13 20 13 20 28 35 13 20 13 20 13 20 13 20

#endif  // COMMENT


//
// 
//
main( int argc, char *argv[] )
{
    // process command line args
    if ( doArgs( argc, argv ) == ERROR )
    {
	cerr << "doArgs() failure!" << endl;
	exit( 1 );
    }

    // get the name of the RC 'defaults' file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << endl;
	return BAD;
    }

    // initialize the serial ports
    if ( openTransmitter() == ERROR )
    {
	return BAD;
    }

    // collect samples
    if ( doit( irTPort ) == ERROR )
    {
	cerr << "problem sampling buttons!" << endl;
	return BAD;
    }

    // close shop & go home
    return OK;
}


//
// 
//
static int
doit( int oFd )
{
    int status;
    commandType command;
    int sampleBuffer[ SAMPLE_BUF_SIZE ];
    int averageBuffer[ SAMPLE_BUF_SIZE ];

    for ( ;; )
    {
    top:
	// display commands
	menu();
	switch ( command = getCommand() )
	{
	case CMD_OFFSET:
	    offsetSamples( averageBuffer );
	    break;

	case CMD_LOOFFSET:
	    /// BOGUS: real work done in getCommand()
	    break;

	case CMD_HIOFFSET:
	    /// BOGUS: real work done in getCommand()
	    break;

	case CMD_DISPLAY:
	    displaySample( averageBuffer );
	    break;

	case CMD_INPUT:
	    inputSample( averageBuffer );
	    break;

	case CMD_TEST:
	    irCommand( averageBuffer, oFd );
	    break;

	case CMD_DUMP:
	    irStream( oFd );
	    break;

	case CMD_QUIT:
	    return OK;

	default:
	    cerr << "huh?" << endl;
	    goto top;
	}

    }

    return status;
}


//
// 
//
static int
displaySample( int* buffer )
{
    int sampleCount = 0;
    int count = buffer[ 0 ];
    int x;

    // count the samples in buffer
    for ( x = 1; x <= (count+1); ++x )
    {
	if ( buffer[ x ] == END_OF_SAMPLE )
	{
	    ++sampleCount;
	}
    }

    // look for display of average
    if ( (sampleCount == 1) && (x == count+2) )
    {
	cout << form( " average:\n\n" );
    }
    else
    {
	cout << form( " %d sample(s) found:\n\n", sampleCount );
    }

    if ( sampleCount == 0 )
    {
	return ERROR;
    }

    // print each sample
    for ( x = 1; x <= count; ++x )
    {
	if ( buffer[ x ] == END_OF_SAMPLE )
	{
	    cout << "\n" << endl;
	    /// cerr << "END_OF_SAMPLE" << endl;
	}
	else if ( buffer[ x ] == 0 )
	{
	    cout << endl;
	    cerr << "unexpected ZERO count in buffer!" << endl;
	    return ERROR;
	}
	else
	{
#if defined( FACTOR )
	    cout << form("%2d ", buffer[ x ] / FACTOR );
#else  // FACTOR
	    cout << form("%2d ", buffer[ x ] );
#endif  // FACTOR
	}
    }

    if ( buffer[ x-1 ] != END_OF_SAMPLE )
    {
	cout << "\n" << endl;
    }

    return OK;
}


//
// 
//
static int
offsetSamples( int* average )
{
    int	count = average[ 0 ];
    int x;

    // 
    for ( x = 1; x <= count; ++x )
    {
	if ( average[ x ] == END_OF_SAMPLE )
	{
	    break;
	}
	else
	{
	    // Sharp IR device has differnet rise and fall times,
	    //  samples need adjustment
	    //  we dont adjust the first Hi since it behaves differently
	    //  than values that are "midstream"

#if defined( FACTOR )
	    /// cerr << "a[" << x << "]: " << average[x];
	    int temp;
	    if ( x & 1 )		// Hi value
	    {
		temp = average[ x ] * sharpHiOffset / FACTOR;
	    }
	    else			// Lo value
	    {
		temp = average[ x ] * sharpLoOffset / FACTOR;
	    }
	    average[ x ] = ((temp + (FACTOR/2)) / FACTOR) * FACTOR;
	    /// cerr << ", a[" << x << "]: " << average[x] << endl;
#else  // FACTOR

	    if ( (x > 1) && (x & 1) )	// NOT first and Hi value
	    {
		double temp = ((double)average[ x ] / count) * SHARP_OFFSET;
		average[ x ] = (int)floor( temp + 0.5 );
	    }
	    else			// Lo value
	    {
		average[ x ] = (int)floor( ((double)average[x]/count) + 0.5 );
	    }

#endif  // FACTOR

	}
    }

    cout << "\n offset";
    displaySample( average );

    return OK;
}


//
// 
//
static int
inputSample( int* average )
{
    int	count = 0;
    int x;

    // 
    for ( x = 1;  ; ++x )
    {
	cin >> average[ x ];
	if ( average[ x ] == -1 ) break;
#if defined( FACTOR )
	average[ x ] *= FACTOR;
#endif
    }

    average[ x ] = END_OF_SAMPLE;
    average[ 0 ] = x;

    cout << "\n input";
    displaySample( average );

    return OK;
}


///////////////////////////////////////////////////////////////////////////////
// FIX ME
// this code from ird/irdaemon.c, irdump.c, merge back someday

//
// Serial IR code
//

// bitfields within output bytes
#define IR_STSP			0x12		// start/stop bits: x10x10x
#define IR_DMASK		0x49		// data bits: dxxdxxd
#define IR_1			0x01		// first bit
#define IR_2			0x08		// second bit
#define IR_3			0x40		// third bit
#define IR_LOW			(IR_STSP)		// 3 low bits
#define IR_HIGH			(IR_STSP|IR_DMASK)	// 3 high bits
#define IR_LLH			(IR_STSP|IR_3)		// low, low, high
#define IR_LHH			(IR_STSP|IR_2|IR_3)	// low, high, high
#define IR_HLL			(IR_STSP|IR_1)		// high, low, low
#define IR_HHL			(IR_STSP|IR_1|IR_2)	// high, high, low

#define IR_HLH			(IR_STSP|IR_1|IR_3)	// high, low, high
#define IR_LHL			(IR_STSP|IR_2)		// low, high, low
#define IR_HHH			(IR_STSP|IR_1|IR_2|IR_3) // high, high, high

#define IR_BITS_PER_BYTE	3
#define FINAL_LOW_BITS		100

#define HAPPY_TV_NOT

//
// 
//
static int
irStream( int port )
{
    const u_char irp[] = { IR_HHH, IR_HHH, IR_HHH, IR_HHH,
    			   IR_HHH, IR_HHH, IR_HHH, IR_HHH,
    			   IR_HHH, IR_HHH, IR_HHH, IR_HHH,
    			   IR_HHH, IR_HHH, IR_HHH, IR_HHH };
    int x, y;

    for ( ;; )
    {
	write( port, &irp, 16 );
    }
}



//
// 
//
static int
irCommand( int* sample, int port )
{
    const u_char ir001 = IR_LLH;
    const u_char ir011 = IR_LHH;
    const u_char ir100 = IR_HLL;
    const u_char ir110 = IR_HHL;
    int x, y;


#if 0
    cerr << "count: " << sample[ 0 ] << endl;
    for ( x = 1; x <= sample[ 0 ]; ++x )
    {
	cerr << form( " %3d", sample[ x ] / FACTOR );
	if ( (x % 20) == 0 ) cerr << endl;
    }
    cerr << endl;
#endif

    // 
    int count = sample[ 0 ];
    int alreadySent = 0;
    int bits, bits3, finalBits;

    // send each sample
    for ( x = 0; x <= count;  )		// x == 0 -> count
    {
	// send Hi sample
	bits = (sample[ ++x ] / FACTOR) - alreadySent;
	if ( (bits3 = bits / IR_BITS_PER_BYTE) == 0 )
	{
	    bits3 = 1;
	    finalBits = 0;
	}
	else
	{
	    finalBits = bits % IR_BITS_PER_BYTE;
	}

	// send all Hi bytes
	ir3HiBits( bits3, port );

	// finish Hi sample
	switch( finalBits )
	{
	case 1:
	    write( port, &ir100, 1 );		// 1 more Hi bit
	    alreadySent = 2;			//  plus 2 Lo bits
	    break;
	case 2:
	    write( port, &ir110, 1 );		// 2 more Hi bits
	    alreadySent = 1;			//  plus 1 Lo bit
	    break;
	default:
	    // no finalBits means no more Hi bits to send
	    alreadySent = 0;
	    break;
	}

	// send the Lo sample
	bits = (sample[ ++x ] / FACTOR) - alreadySent;
	bits3 = bits / IR_BITS_PER_BYTE;
	finalBits = bits % IR_BITS_PER_BYTE;
	if ( bits3 == 0 )
	{
	    bits3 = 1;
	    finalBits = 0;
	}

	// send all Lo bytes
	ir3LoBits( bits3, port );

	// finish Lo sample
	switch( finalBits )
	{
	case 1:
	    write( port, &ir011, 1 );	// 1 more Lo bit
	    alreadySent = 2;		//  plus 2 Hi bits
	    break;
	case 2:
	    write( port, &ir001, 1 );	// 2 more Lo bits
	    alreadySent = 1;		//  plus 1 Hi bit
	    break;
	default:
	    // no finalBits means no more Lo bits to send
	    alreadySent = 0;
	    break;
	}
    }

    if ( count & 1 )
    {
	// odd number of samples (expected), add a final IR_LOW count
	ir3LoBits( FINAL_LOW_BITS / IR_BITS_PER_BYTE, port );
    }

    return OK;
}

/// OLD code:
#if defined( HAPPY_TV )
    // 
    int count = (int)sample[ 0 ];

    for ( x = 1, y = count; x <= y; ++x )
    {
	bytes = (sample[ x ] + hiAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + hiAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	ir3HiBits( bytes, port );

	++x;
	bytes = (sample[ x ] + loAdjust) / IR_BITS_PER_BYTE;
	rem = (sample[ x ] + loAdjust) % IR_BITS_PER_BYTE;
	if ( bytes == 0 )
	{
	    bytes = 1;
	    rem = 0;
	}
	if ( rem >= 2 ) ++bytes;
	ir3LoBits( bytes, port );

	if ( x == count ) break;
    }
    if ( count & 1 )
    {
	// odd number of samples, add a final IR_LOW count
	ir3LoBits( 200, port );
    }
#endif  // HAPPY_TV



//
// 
//
static int
ir3HiBits( int count, int port )
{
    const u_char irHi[] = {
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH,
	IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH, IR_HIGH
    };

    int blocks = count / 128;
    int rem = count % 128;

    for ( int x = blocks; x; --x )
    {
	write( port, irHi, 128 );
    }
    if ( rem ) write( port, irHi, rem );

    return OK;
}


//
// 
//
static int
ir3LoBits( int count, int port )
{
    const u_char irLo[] = {
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW,
	IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW, IR_LOW
    };

    int blocks = count / 128;
    int rem = count % 128;

    for ( int x = blocks; x; --x )
    {
	write( port, irLo, 128 );
    }
    if ( rem ) write( port, irLo, rem );

    return OK;
}


///////////////////////////////////////////////////////////////////////////////

static void
menu( void )
{
    cout << "\n <d>isplay <o>ffset <h>ios <l>oos <t>est <i>nput <p>ump <q>uit <?>: ";
}


static commandType
getCommand( void )
{
    static commandType lastCommand = CMD_ERROR;
    char	s[ 80 ];
    float val;

    fgets( s, 79, stdin );

    if ( isascii( s[0] ) && isupper( s[0] ) )
    {
	s[0] = toupper( s[0] );
    }

    switch( s[ 0 ] )
    {
    case '\n':
	return lastCommand;

    case 'd':
	return (lastCommand = CMD_DISPLAY);

    case 'o':
	return (lastCommand = CMD_OFFSET);

    case 'l':
	sscanf( s+1, "%f", &val );
cerr << "val: " << val << endl;
	sharpLoOffset = (int)(val * FACTOR);
	cerr << "new Lo Offset: " << sharpLoOffset << endl;
	return (lastCommand = CMD_LOOFFSET);

    case 'h':
	sscanf( s+1, "%f", &val );
	sharpHiOffset = (int)(val * FACTOR);
	cerr << "new Hi Offset: " << sharpHiOffset << endl;
	return (lastCommand = CMD_HIOFFSET);

    case 't':
	return (lastCommand = CMD_TEST);

    case 'i':
	return (lastCommand = CMD_INPUT);

    case 'p':
	return (lastCommand = CMD_DUMP);

    case 'q':
	return (lastCommand = CMD_QUIT);

    default:
	return (lastCommand = CMD_ERROR);
    }
}

//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'l':
		sharpLoOffset =  atoi( argv[ ++index ] );
		break;

	    case 'h':
		sharpHiOffset =  atoi( argv[ ++index ] );
		break;

	    case 'm':
		minSample = atoi( argv[ ++index ] );
		if ( minSample > 255 ) minSample = 255;
	        break;

	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << "usage: " << argv[ 0 ]
		     << " [-v][-m <size>][-l <offset>][-h <offset>]" << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}


//
// 
//
static int
openTransmitter( void )
{
    struct termios	newTerm;

    // get the default strings
    const char*		irFile;
    if ( !(irFile = defaults.getString( "IRTPORT" )) )
    {
	cerr << "couldn't find IRTPORT" << endl;
	return ERROR;
    }

    // open port
#if 0
    if ( (irTPort = open( irFile, O_RDWR | O_NONBLOCK )) < 0 )
#else
    if ( (irTPort = open( irFile, O_WRONLY )) < 0 )
#endif
    {
	perror( "openTransmitter(): open()" );
	return ERROR;
    }

    if ( tcgetattr( irTPort, &newTerm ) == ERROR ) {
	return ERROR;
    }

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSTOPB|HUPCL|PARENB|CSIZE );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS7 );		// 7 bit words

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	irBaud;
    if ( !(irBaud = defaults.getString( "IRTBAUD" )) )
    {
	cerr << "couldn't get IRTBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( irBaud ) ) == ERROR )
    {
	return ERROR;
    }

    if ( tcsetattr( irTPort, TCSANOW, &newTerm ) == ERROR )
    {
	return ERROR;
    }

    return OK;
}


