//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

// $Id: mlbusmaster.hh,v 1.4 1996/03/13 22:46:39 smp Exp smp $

//
// MicroLAN is a trademark Dallas Semiconductor
//

//
// mlbusmaster.hh
//


#ifndef _MLBUSMASTER_H
#define _MLBUSMASTER_H

#include <machine/cpufunc.h>

#include "util/misc/misc.hh"


//
// 
//
class MLBusMaster
{

 public:
    
    // constructor
    MLBusMaster( int port );

    // destructor
    ~MLBusMaster( void ) {}


    // read a ROM section of device in a SINGLE DEVICE system
    int			readROM( char* asciiBuffer );

    // address device by family, serial#, crc
    int			matchROM( const BYTE* id );

    // skip the ROM addressing cycle
    int			skipROM( void );

    // search for all existing devices
    int			searchROM( BYTE* ids, const int maxID );

    // send a ROM command
    int			mlROMCommand( const int command );

    // send a memory/control function byte, read data
    int			mlReadFunction( int function,
				        int dataLength, BYTE* data );

    // send a memory/control function byte, write data
    int			mlWriteFunction( int function,
					 int dataLength = 0,
					 const BYTE* data = (const BYTE*)0 );

    // MicroLAN(tm) read data phase, NO RESET
    int			mlReadData( int dataLength, BYTE* data );

    // MicroLAN(tm) write data phase, NO RESET
    int			mlWriteData( int dataLength, const BYTE* data );

    // send MicroLAN(tm) command sequence
    int			mlCommand( int commandLength, const BYTE* command,
				   int dataLength = 0, BYTE* data = NULL );

    // calculate MicroLAN(tm) crc
    int			mlCRC8( BYTE *buffer );

    // microtimer functions
    int			delay( int millisecs );

    // read a bit from the bus
    int			mlReadBit( void );


 private:

    // 
    const int COMMAND_RETRIES	= 10;

    // bytes to represent various bit values
    const BYTE RESET_PULSE	= 0x00;		// all 0's @ 9600 baud to reset
    const BYTE LO_BIT		= 0x00;		// all 0's sends a 0 bit
    const BYTE HI_BIT		= 0xff;		// all 1's sends a 1 bit

    // each read byte has a bit representing the data
    const BYTE SAMPLE_POSITION	= 0x02;		// the bit to sample,
						//  longer cable needs later,
						//   (ie higher #) bit
    // mask for loading bits into bytes
    const BYTE EIGHTH_BIT	= 0x80;

    // this code assummes standard pc type serial ports
    enum TTY_ADDR
    {
	TTY0			= 0x3f8,
	TTY1			= 0x2f8,
	TTY2			= 0x3e8,
	TTY3			= 0x2e8
    };

    // bit fields/masks for the uart
    enum UART_DATA
    {
	BAUD_9600		= 0x12,
	BAUD_115200		= 0x01,
	BAUD_LOAD		= 0x80,
	DATA_LOAD		= 0x03,
	DATA_READY		= 0x01
    };

    // various times expressed in milliseconds
    enum MILLI_TIMES
    {
	DATA_TIME		=    1,
	RESET_TIME		=   10,
	TENTH_SECOND		=  100,
    };

    enum CRC_TYPE
    {
	CRC_TYPE_NONE,
	CRC_TYPE_8BIT,
	CRC_TYPE_16BIT
    };

    // reset the MicroLAN(tm) hardware
    int			mlReset( void );

    // send a 'clock' byte to the bus, return data
    int			mlReadByte( BYTE* data );

    // send a data byte to the bus
    int			mlWriteByte( const BYTE data );

    // 
    int			mlWriteBit( const int data );

    // 
    int			dataDelay( void );

    // 
    static void		delayDone( void );

    // port manipulation functions
    inline BYTE LSR( void )	{ return inb( _ttyAddr + 5 ); }
    inline BYTE DATA( void )	{ return inb( _ttyAddr     ); }

    inline void DATA( int b )	{ outb( _ttyAddr,     b ); }
    inline void BAUD( int b )	{ outw( _ttyAddr,     b ); }
    inline void IER( int b )	{ outb( _ttyAddr + 1, b ); }
    inline void LCR( int b )	{ outb( _ttyAddr + 3, b ); }
    inline void MCR( int b )	{ outb( _ttyAddr + 4, b ); }

    int			_ttyAddr;

///    MLDevice		_device;
};

#endif // _MLBUSMASTER_H
