//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id$";


//
// mlcontrol.cc
//

#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/udgsocket.hh"
#include "rc/daemon/rcserver.hh"

#include "mlcontrol.hh"

struct DEVNAMES {
    const char*		name;
} devNames[] =
{
  { "<noname>"			},
  { "greenhouse.loshade"	},
  { "greenhouse.hisun"		},
  { "compcloset.hi"		},
  { "compcloset.mid"		},
  { "outside.northshade"	},
  { "outside.southsun"		},
  { "compcloset.lo"		},
  { "attic.center"		}
};

static int		doArgs( int argc, char** argv, MLControl& mlc );


main( int argc, char *argv[] )
{
    // create the (ONE and ONLY ONE) instance of the client object
    MLControl mlc( argv[ 0 ] );

    // get args
    if ( doArgs( argc, argv, mlc ) == ERROR )
    {
	exit( 1 );
    }

    // initialize client, default daemon, NO log file
    if ( mlc.initialize( "MLD_DG_PATH", (char*)NULL ) == ERROR )
    {
	cerr << "mlcontrol failure!" << endl;
	exit( 1 );
    }

    //
    // do the work here
    //
    return mlc.mlMain();
}


//
// 
//
int
MLControl::mlMain( void )
{
    // main loop
    while ( TRUE )
    {
	// wait for data
	int dataAvailable = waitForData();

	// get message from the MicroLAN(tm) daemon
	if ( dataAvailable & DAEMON_DATA )
	{
	    char message[ MAX_DG_MSG ];
	    if ( readDaemonMsg( message ) < 0 )
	    {
		perror( "readDaemonMsg()" );
		return ERROR;
	    }

	    // process 'status' message
	    if ( message[ 0 ] == STATUS_MSG_START )
	    {
		cout << form( "status: %s\n", message );
	    }

	    // async data
	    else
	    {
#if 1
// FIXME: kludge
		char* p;
		if ( p = strchr( message, '#' ) )
		{
		    char s[ 256 ];
		    sprintf( s, "The %s temp is now %s degrees.\n",
			    devNames[ atoi( p+1 ) ].name,
			    p+4 );
		    cout << s;
		}
		else
		{
		    cout << form( "data: %s", message ) << endl;
		}
#endif
	    }
	}

	// data from client/user interface
	if ( dataAvailable & CLIENT_DATA )
	{
	    // get input from stdin
	    char str[ MAX_INPUT ];
	    if ( fgets( str, MAX_INPUT, stdin ) == NULL ) break;

	    // remove WS, look for comments, get length
	    char* p = str;
	    while ( isspace( *p ) ) ++p;
	    if ( (*p == COMMENT_CHAR) || (*p == '\0') )
	    {
cout << str << endl;
		continue;
	    }
	    int length = strlen( p ) + 1;	// count terminating '\0'

	    // send the message to the daemon
	    int count;
	    while ( (count = writeDaemonMsg( p, length )) != length )
	    {
		if ( count == -1 )
		{
		    if ( errno == ENOBUFS )
		    {
			cerr << "ENOBUFS" << endl;
			sleep( 3 );
			continue;
		    }
		    perror( "writeDaemonMsg() ERROR" );
		    cerr << "errno: " << errno << endl;
		}
		else
		{
		    cerr << "writeDaemonMsg() count: " << count << endl;
		}
		return ERROR;
	    }
	}

    }  // endOfWhile( TRUE )

    // reached EOF on client input
    return OK;
}


//
// process command line arguments
//
static int
doArgs( int argc, char** argv, MLControl& mlc )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		mlc.incVerbose();
	        break;

	    default:
		cerr << "usage: %s [-v]" << argv[ 0 ] << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
