//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id: mld.cc,v 1.2 1996/03/09 05:29:03 smp Exp smp $";


//
// MicloLAN(tm) Controller daemon
//

#include <iostream.h>
#include <stdlib.h>
#include <string.h>

#include "mld.hh"
#include "mldaemon.hh"


//
// local data & functions
//

static int	doArgs( int, char**, MLDaemon& );


main( int argc, char *argv[] )
{
    // create the (ONE and ONLY ONE) instance of the daemon object
    MLDaemon mld;

    // process commandline args
    if ( doArgs( argc, argv, mld ) == ERROR )
    {
	exit( 1 );
    }

    // initialize the MicroLAN(tm)
    int port = mld.deviceInitialize();
    if ( port < 0 )
    {
	cerr << "MLD device initialization failure!" << endl;
	exit( 1 );
    }

    // initialize daemon
    if ( mld.initialize( argv[0], "MLD_DG_PATH", "MLD_LOGFILE", port )
	  == ERROR )
    {
	cerr << "MLD daemon failure!" << endl;
	exit( 1 );
    }

    //
    // do the work here
    //
    mld.mlMain();

    // should never get here!
    exit( 2 );
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[], MLDaemon& mld )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		mld.incVerbose();
		break;

	    case 'k':
		mld.sendMessage( "MLD_DG_PATH", "kill" );
		exit( 0 );

	    default:
		cerr << form( "usage: %s [-kv]", argv[ 0 ] ) << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
