//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

// $Id: mldaemon.hh,v 1.8 1996/03/31 03:09:06 smp Exp smp $

//
// MicroLAN is a trademark Dallas Semiconductor
//

//
// mldaemon.hh, header for mldaemon.cc
//


#ifndef _MLDAEMON_H
#define _MLDAEMON_H

#include <unistd.h>

#include "util/misc/keymap.hh"
#include "../daemon/rcdaemon.hh"

#include "mlclass.hh"
#include "mldevice.hh"
#include "mlbusmaster.hh"


#define PORTNUM_INDEX		9
#define PORT_STRING		"/dev/ttyd"

struct romTag_t
{
    BYTE	rom[ 8 ];
    MLDevice*	device;
};


class MLDaemon : public RcDaemon
{

 public:

    // constructor
    MLDaemon( void );

    // destructor
    virtual		~MLDaemon( void );

    // initialize daemon
    virtual int		initialize( const char* name,
				    const char* path,
				    const char* logFile,
				    const int port );

    // initialize device controlled by daemon
    virtual int		deviceInitialize( void );

    // main processing loop
    int			mlMain( void );


 protected:

    // MicroLAN(tm) specific method for getting data
    /// FIXME: not implimented yet
    virtual int		getDeviceData( void ) { return -1; }

    // MicroLAN(tm) specific method for termination
    virtual void	terminate( void );

    // MicroLAN(tm) specific method for opening control channel
    virtual int		openDevice( void );


 private:

    //
    // unique commands & data
    //

    // process client messages
    int			doClient( void );

    // direct IO enablement
    int			mlOpenIO( void );
    int			mlCloseIO( void ) { return fclose( _IOPerm ); }

    // qualify device @ _mlDevices[ i ] asto family, etc.
    int			validateDevice( const char* i );

    // find (and possibly test) all devices on the bus
    int			mlDiscover( void );

    // 
    int			nameToDevice( char* id, MLDevice*& device );


    // 
    void		ROMString( MLDevice* device, char* asciiBuffer );

    // 
    MLBusMaster*	_master;

    static const int	_NUM_MLDEVICES  = 256;
    int			_numberDevices;

    // MicroLAN(tm) devices
    MLDevice*		_mlDevices[ _NUM_MLDEVICES + 1 ];

    //
    MLDS1820Thermometer**	_thermometer;
    MLDS2401SerialNumber**	_serialNumber;
    MLDS2405AddressableSwitch**	_switch;
    MLDS2407AddressableSwitch**	_dualSwitch;

    // MicroLAN(tm) device port
    int			_port;
    FILE*		_IOPerm;

    // 
    KeyMap		_mlMap;
    romTag_t*		_romTags;
};

#endif // _MLDAEMON_H
