//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id$";


//
// mlds1820.cc
//

#include <iostream.h>

#include "mlds1820.hh"


//
// 
//
int
MLDS1820Thermometer::powerSupply( void )
{
    // address myself
    if ( addressDevice() == ERROR ) return ERROR;

    // send the command
    if ( controlDevice( _READ_POWERSUPPLY ) == ERROR ) return ERROR;

    // device sends 0 bit if no power, 1 bit if +5volts
    return readBit();
}


//
// get temperature in degrees fahrenheit
//
int
MLDS1820Thermometer::fahrenheit( double& temperature )
{
    double t;
    if ( centigrade( t ) == ERROR ) return ERROR;

    temperature = (t * 1.8) + 32.0;
    return OK;
}


//
// get temperature in degrees centigrade
//
int
MLDS1820Thermometer::centigrade( double& temperature )
{
    BYTE buffer[ _SCRATCHPAD_LENGTH ];

    // address myself
    if ( addressDevice() == ERROR ) return ERROR;

    // send the command
    if ( controlDevice( _CONVERT_COMMAND ) == ERROR ) return ERROR;

    // wait for conversion to take place
#if 0
    delay( _CONVERT_TIME );
#else
    // device outputs 0's while busy doing conversion
    while ( readBit() == 0 ) /* spin */;
#endif

    int goodCRC8;
    for ( int x = 0; x < _CRC8_RETRIES; ++x )
    {
	// address myself
	if ( addressDevice() == ERROR ) return ERROR;

	// read the device buffer
	if ( readScratch( buffer ) == ERROR )
	{
	    cerr << "unit is not responding" << endl;
	    continue;
	}

	// calcualte crc of the data
	if ( goodCRC8 = CRC8( buffer ) ) break;
    }

    if ( !goodCRC8 )
    {
	cerr << " checksum ERROR!" << endl;
	return ERROR;
    }

    // calculate extended resolution tempurature
    double reading = (double)((*(short *)&buffer[ _TEMP_OFFSET ]) >> 1);
    double countRemaining = buffer[ _COUNTSREM_OFFSET ];
    double countsPerDegreeC = buffer[ _COUNTSPD_OFFSET ];

    if ( countsPerDegreeC == 0.0 ) return ERROR;

    // 
    temperature = reading - 0.25 +
      (countsPerDegreeC - countRemaining) / countsPerDegreeC;

    // done
    return OK;
}
