//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1996 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//

// $Id$

//
// MicroLAN is a trademark Dallas Semiconductor
//

//
// mlds1820.hh
//


#ifndef _MLDS1820_H
#define _MLDS1820_H

#include "mldevice.hh"
#include "mlbusmaster.hh"


class MLDS1820Thermometer : public MLDevice
{

 public:

    MLDS1820Thermometer( MLBusMaster& m )
	: MLDevice( _MLDS1820_FAMILY_CODE, m ) {}

    MLDS1820Thermometer( MLBusMaster& m, const BYTE* id )
	: MLDevice( id, m ) {}

    ~MLDS1820Thermometer( void ) {}

    // get a temp in degrees centigrade
    int		centigrade( double& temperature );

    // get a temp in degrees fahrenheit
    int		fahrenheit( double& temperature );

    int		powerSupply( void );

    // 
    int		readScratch( BYTE* buf )
		  { return _busMaster.mlReadFunction( _SCRATCH_READ,
						      _SCRATCHPAD_LENGTH,
						      buf ); }

 private:

    enum MLDS1820_CONSTS
    {
	_CRC8_RETRIES		=    4,
	_SCRATCHPAD_LENGTH	=    9,
	_TEMP_OFFSET		=    0,
	_COUNTSREM_OFFSET	=    6,
	_COUNTSPD_OFFSET	=    7,
	_CONVERT_TIME		= 1000
    };

    enum MLDS1820_COMMAND
    {
	_SCRATCH_READ		= 0xbe,
	_SCRATCH_WRITE		= 0x4e,
	_SCRATCH_COPY		= 0x48,
	_RECALL_COMMAND		= 0xb8,
	_CONVERT_COMMAND	= 0x44,
	_READ_POWERSUPPLY	= 0xb4
    };

};

#endif // _MLDS1820_H
