//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10client.c,v 1.1 1995/08/31 01:14:31 smp Exp smp $";


//
// x10client.cc
//

#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/namedstr.hh"
#include "rc/daemon/rcserver.hh"
#include "rc/x10d/x10.hh"
#include "rc/x10d/x10daemon.hh"
#include "rc/x10d/x10convert.hh"


static UnixDGSocket *sp;

int
sendX10Command( int command, int area, int key )
{
    u_char buf[ CNTL_BSIZE ];

    buf[ RC_CMND ] = command;
    buf[ X10_AREA ] = area;
    buf[ X10_UNIT ] = key;
    buf[ X10_RCNT ] = 2;

    // send the message to the x10 daemon
reSend:
    int status = sp->send( (char *)buf, CNTL_BSIZE );
    switch ( status )
    {
    case CNTL_BSIZE:
	break;
    case -1:
	if ( errno == ENOBUFS ) goto reSend;
	perror( "x10client, sendto()" );
	return BAD;
    default:
	cerr << "x10client: bad send() count: " << status << "\n";
	return BAD;
    }

    return OK;
}


int
attachDaemon( void )
{
    // get the RC defaults file
    NamedStr defaults;
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << "\n";
	exit( 1 );
    }

    // get the socket paths from the RC defaults file
    const char*	myPath;
    const char*	itsPath;
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	  || !(itsPath = defaults.getString( "X10D_DG_PATH" )) )
    {
	cerr << "couldn't get socket paths\n";
	return BAD;
    }

    // create a socket
    sp = new UnixDGSocket( (char*)myPath, (char*)itsPath );
    if ( sp->getFD() == ERROR ) return BAD;

    return OK;
}
