//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10control.c,v 1.1 1995/08/31 01:24:28 smp Exp smp $";


//
// x10control.cc
//

#include <stdio.h>
#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/keymap.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/rcserver.hh"
#include "rc/x10d/x10daemon.hh"


#define MAX_INPUT	255
#define COMMENT_CHAR	'#'

static int	doArgs( int, char ** );
static int	control( FILE * );


main( int argc, char *argv[] )
{
    // get args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // body of client
    return control( stdin );
}


static int
control( FILE *fp )
{
    char str[ MAX_INPUT ];

    // get the RC defaults file
    NamedStr defaults;
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load necessary file: " << RCD_STRINGS << "\n";
	exit( 1 );
    }

    // get the socket paths from the RC defaults file
    const char*	myPath;
    const char*	itsPath;
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	 || !(itsPath = defaults.getString( "X10D_DG_PATH" )) )
    {
	cerr << "couldn't get socket paths\n";
	return BAD;
    }

    // create a socket
    UnixDGSocket udgSocket( (char*)myPath, (char*)itsPath );
    if ( udgSocket.getFD() == ERROR ) return BAD;

    // read the x10 controller key mapping
    KeyMap x10Map;
    const char* s;
    if ( s = defaults.getString( "BINX10KEYMAP" ) )
    {
	if ( x10Map.readMap( (char*)s ) == ERROR )
	{
	    cerr << form( "couldn't read %s\n", s );
	}
    }
    else
    {
	cerr << "couldn't find X10KEYMAP\n";
    }

    u_char buf[ CNTL_BSIZE ];
    buf[ RC_CMND ] = X10EXEC;

top:
    while ( fgets( str, MAX_INPUT, fp ) != NULL )
    {

	// break string up into individual words
	Tokens& tokens = *new Tokens( str );

	// process each word in stream
	for ( int x = 0, cnt = tokens.count(); x < cnt; ++x )
	{
	    // skip comments
	    if ( tokens[ x ][ 0 ] == COMMENT_CHAR ) break;

	    // convert name to controller key number
	    if ( x10Map.getKey( tokens[ x ], buf[ X10_AREA ], buf[ X10_UNIT ] )
		 == FALSE )
	    {
		cerr << "NOT found '" << tokens[ x ] << "'\n";
		break;
	    }

	    // default count
	    buf[ X10_RCNT ] = 2;

	    // send the message to the x10 daemon
reSend:
	    int status = udgSocket.send( (char *)buf, CNTL_BSIZE );
	    switch ( status )
	    {

	    case CNTL_BSIZE:
		break;

	    case -1:
		if ( errno == ENOBUFS ) goto reSend;
		perror( "x10control, sendto()" );
		return BAD;

	    default:
		cerr << "x10control: bad send() count: " << status << "\n";
		return BAD;

	    }
	}

	// return storage to system
	delete &tokens;
    }

    return OK;
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    // process optional arguments
    for ( int index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
#if 0
	    	case 'v':
		++verbose;
		break;
#endif
	    	default:
		cerr << "usage: %s [-v]\n";
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
