//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//


static char rcsid[] = "$Id: x10action.cc,v 1.4 1996/03/31 01:58:53 smp Exp smp $";


//
// x10action.cc
//

#include <stdio.h>
#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "util/misc/misc.hh"
#include "util/misc/keymap.hh"
#include "util/misc/tokens.hh"

#include "x10.hh"
#include "x10d.hh"
#include "x10daemon.hh"
#include "x10action.hh"


//
// constructor
//  decode & store args for action
//
X10Action::X10Action( const char* args )
{
    int x;
    int index;

    // sending to local daemon, decode ascii args to binary

    // break string up into individual words
    Tokens& tokens = *new Tokens( (char*)args );
    int argc = tokens.count() - 1;

    // check command
    if ( strcmp( tokens[ 0 ], "emit" ) == 0 )
    {
	_command = COMMAND_EMIT;
    }
    else if ( strcmp( tokens[ 0 ], "toggleVerbal" ) == 0 )
    {
	_command = COMMAND_VTOGGLE;
	_args = NULL;
	goto done;
    }
    else
    {
	_command = ERROR;
	_args = NULL;
	cerr << "X10Action::X10Action(), bad command:" << tokens[ 0 ] << endl;
	goto done;
    }

    // group contains: [room, key, repeat]
    // 1st BYTE == numberOfGroups + (3 * numberOfGroups)
    // following BYTEs contain group(s)
    index = 0;
    if ( argc < 1 )
    {
	_command = ERROR;
	_args = NULL;
	cerr << "X10Action::X10Action(), missing args" << endl;
	goto done;
    }
    _args = new u_char[ 1 + (argc * 3) ];
    _args[ index++ ] = argc;

    // skip address & command, decode and store group(s)
    for ( x = 1; x <= argc; ++x )
    {
	/// FIX ME
	extern KeyMap x10Map;
	x10Map.getKey( tokens[ x ], _args[ index ], _args[ index+1 ] );
	index += 2;
	_args[ index++ ] = REPEAT_COUNT;
    }

 done:
    // return storage to system
    delete &tokens;

    return;
}


//
// use args to control X-10 hardware
//
int
X10Action::action( void )
{
    int		argc;
    u_char*	args;

    switch ( _command )
    {
    case COMMAND_EMIT:
	// get a temporary argc
	args = _args;

	// sending to local daemon, _args already decoded to binary stream
	argc = *args++;
	for ( int x = 0; x < argc; ++x, args += 3 )
	{
	    if ( writeX10( X10EXEC, args ) == ERROR )
	    {
		perror( "writeX10()" );
		return ERROR;
	    }
	}
	break;

    case COMMAND_VTOGGLE:
	verbal = verbal ? 0 : 1;
	break;

    default:
	return ERROR;
    }

    return OK;
}
