//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
///

static char rcsid[] = "$Id: x10convert.c,v 1.1 1995/10/21 02:19:33 smp Exp smp $";


#include "x10.hh"


//
// x10convert.cc
//

#if defined( COMMENTS )

house code values used by x10 hardware are defined in x10.h:
enum X10HouseCode {
    HOUSE_M =  0,
	...
    HOUSE_J = 15
};

area code values used by the software are defined in x10.h:
enum roomCode {
    AREA_A,
	...
    AREA_P
};


unit & function codes used by x10 hardware are defined in x10.h:
enum X10Function {
    UNIT_13 =                0,
	...
    FUNC_STATUS_REQUEST =   31
};

unit & function codes used by the software are defined in x10.h:
enum keyCode {
    KEY_1,
	...
    KEY_STATUS_REQUEST
};

#endif  // COMMENTS


//
// map an x10 HOUSE code to a (zero based) area number
//
int
x10ToArea( int x10Area )
{
    static int	_X10ToArea[] = {
	AREA_M, AREA_N, AREA_O, AREA_P, AREA_C, AREA_D, AREA_A, AREA_B,
	AREA_E, AREA_F, AREA_G, AREA_H, AREA_K, AREA_L, AREA_I, AREA_J
    };

    return _X10ToArea[ x10Area & 0x0f ];
}


//
// map an x10 FUNCTION code to a (zero based) key number
//
int
x10ToKey( int x10Key )
{
    static int	_X10ToKey[] = {
	KEY_13, KEY_14, KEY_15, KEY_16, KEY_3, KEY_4, KEY_1, KEY_2,
	KEY_5, KEY_6, KEY_7, KEY_8, KEY_11, KEY_12, KEY_9, KEY_10,
	KEY_ALL_UNITS_OFF, KEY_HAIL_REQUEST, KEY_DIM, KEY_EXTENDED_DATA,
	KEY_ON, KEY_PRESET_DIM_LOW, KEY_ALL_LIGHTS_OFF, KEY_STATUS_EQ_OFF,
	KEY_ALL_LIGHTS_ON, KEY_HAIL_ACKNOWLEDGE, KEY_BRIGHT, KEY_STATUS_EQ_ON,
	KEY_OFF, KEY_PRESET_DIM_HIGH, KEY_EXTENDED_CODE, KEY_STATUS_REQUEST
    };

    return _X10ToKey[ x10Key & 0x1f ];
}


//
// map an area number (0-15) to an x10 HOUSE code
//
int
areaToX10( int area )
{
    static int	_areaToX10[] = {
	HOUSE_A, HOUSE_B, HOUSE_C, HOUSE_D, HOUSE_E, HOUSE_F, HOUSE_G, HOUSE_H,
	HOUSE_I, HOUSE_J, HOUSE_K, HOUSE_L, HOUSE_M, HOUSE_N, HOUSE_O, HOUSE_P
    };

    return _areaToX10[ area ];
}


//
// map a key number (0-31) to an x10 FUNCTION code
//
int
keyToX10( int key )
{
    static int	_keyToX10[] = {
	UNIT_1, UNIT_2, UNIT_3, UNIT_4, UNIT_5, UNIT_6, UNIT_7, UNIT_8,
	UNIT_9, UNIT_10, UNIT_11, UNIT_12, UNIT_13, UNIT_14, UNIT_15, UNIT_16,
	FUNC_ALL_UNITS_OFF, FUNC_ALL_LIGHTS_ON,
	FUNC_ON, FUNC_OFF, FUNC_DIM, FUNC_BRIGHT,
	FUNC_ALL_LIGHTS_OFF, FUNC_EXTENDED_CODE,
	FUNC_HAIL_REQUEST, FUNC_HAIL_ACKNOWLEDGE,
	FUNC_PRESET_DIM_LOW, FUNC_PRESET_DIM_HIGH,
	FUNC_EXTENDED_DATA, FUNC_STATUS_EQ_ON,
	FUNC_STATUS_EQ_OFF, FUNC_STATUS_REQUEST
    };

    return _keyToX10[ key ];
}


//
// map a LynX-10 house code to an area code
//
int lynX10ToArea( int house )
{
    // bug: relies on ASCII ordering of letters above numbers
    return house >= 'A'? house - 'A' + 10 : house - '0';
}


//
// map a LynX-10 unit code to a key code
//
int lynX10ToKey( int command, int unit )
{
    // bug: relies on ASCII ordering of letters above numbers
    int code = unit >= 'A'? unit - 'A' + 10 : unit - '0';
    return command? code + 16 : code;
}


//
// map an area code to a LynX-10 house code
//
int areaToLynX10( int area )
{
    return area >= 10? area - 10 + 'A' : area + '0';
}


//
// map a key code to a LynX-10 command and unit code.
//
void keyToLynX10( unsigned char* command, unsigned char* unit )
{
    if ( *unit >= 16 )
        *command = '1', *unit -= 16;
    else
        *command = '0';

    *unit = *unit >= 10? *unit-10+'A' : *unit+'0';
}
