//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10d.cc,v 1.4 1996/03/31 02:03:08 smp Exp smp $";


//
// X-10 equipment daemon
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
///#include <sys/socket.h>
///#include <sys/un.h>

#include "util/misc/misc.hh"
#include "../daemon/rcserver.hh"
#define X10D
#include "x10d.hh"
#include "x10daemon.hh"


//
// local functions
//
static int	doArgs( int, char ** );
static int	initSockets( void );
static void	initKeyMaps( void );



main( int argc, char *argv[] )
{
    struct sigaction	action;
    int			initPid;
    int 		rPipe[ 2 ], wPipe[ 2 ];
    char		s[ 255 ];
    const char*		p;
    extern void		readData( void );

    // 
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << form( "couldn't load necessary file: %s",
		      RCD_STRINGS ) << endl;
	exit( 1 );
    }

    // get args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // send stderr to logfile
    if ( p = defaults.getString( "X10D_LOGFILE" ) )
    {
	if ( freopen( p, "a+", stderr ) == NULL )
	{
	    perror( "logfile open()" );
	    exit( 1 );
	}
    }

    setbuf( stderr, NULL );			// no buffer of output
    markLog( "\n\nX10 Daemon started: " );

    // clear the history buffers
    for ( int x = 0; x < 16; ++x )
    {
	lastKey[ x ] = lastUnit[ x ] = lastFunction[ x ] = -1;
    }

    // set the UNITS mask
    for ( int x = 0; x < 16; ++x )
    {
	unitsGroup[ x ] = functionsGroup[ x ] = 0;
    }

    // read the infrared controller key mapping
    initKeyMaps();

    // open reader/writer communication pipes
    if ( (pipe( rPipe ) < 0) || (pipe( wPipe ) < 0) )
    {
	perror( "pipe()" );
	exit( 1 );
    }

    // fork() the writer process
    if ( (writerPid = fork()) < 0 )
    {
	perror( "can't fork() writer" );
	writerPid = 0;
	daemonTerminate();
    }
    else if ( writerPid == 0 )			// we are the writer
    {
	close( rPipe[ 0 ] );
	close( rPipe[ 1 ] );
	close( wPipe[ 1 ] );
	writer( wPipe[ 0 ] );
    }

    /// wait for writer to open & initialize serplix
#if defined( DONE_RIGHT )
/// we should syncronize via a SIGNAL here!
#else
    sleep( 2 );
#endif

    // fork() the reader process
    if ( (readerPid = fork()) < 0 )
    {
        perror( "can't fork() reader" );
	readerPid = 0;
	daemonTerminate();
    }
    else if ( readerPid == 0 )			// we are the reader
    {
        close( wPipe[ 0 ] );
	close( wPipe[ 1 ] );
	close( rPipe[ 0 ] );

#if defined( SERPLIX )
	reader( rPipe[ 1 ], writerPid );
#else
	reader( rPipe[ 1 ] );
#endif  // SERPLIX

    }

    /// wait for reader to open
#if defined( DONE_RIGHT )
/// we should syncronize via a SIGNAL here!
#else
    sleep( 1 );
#endif

//
// we are the daemon
//
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;

    // setup the INTerrupt handler
#if 0 /**(defined(BSD) && (BSD >= 199306))*/
    action.sa_handler = daemonTerminate;
#else
    action.sa_handler = (PFIV)daemonTerminate;
#endif

    sigaction( SIGINT, &action, (struct sigaction *)NULL );
    sigaction( SIGQUIT, &action, (struct sigaction *)NULL );

#if defined( NO_ZOMBIES_ALLOWED )
    action.sa_handler = childDeath;
    sigaction( SIGCHLD, &action, (struct sigaction *)NULL );
#endif  // NO_ZOMBIES_ALLOWED

    // close unneeded files
    close( rPipe[ 1 ] );
    close( wPipe[ 0 ] );
    readerPipe = rPipe[ 0 ];
    writerPipe = wPipe[ 1 ];

    // initialize the key map
    clearMap();

    // fork() the initializer process
    if ( (initPid = fork()) < 0 )
    {
	perror( "can't fork() initializerer" );
	daemonTerminate();
    }
    else if ( initPid == 0 )			// we are the initializer
    {

	/// wait for daemon to open
#if defined( DONE_RIGHT )
/// we should syncronize via a SIGNAL here!
#else
	sleep( 2 );
#endif
	if ( p = defaults.getString( "X10MAP" ) )
	{
	    cerr << form( "X10MAP: '%s'", p ) << endl;
	    system( p );
	}
	else
	{
	    cerr << "no startup X10MAP" << endl;
	}

	if ( p = defaults.getString( "X10INIT" ) )
	{
	    cerr << form( "X10INIT: '%s'", p ) << endl;
	    system( p );
	}
	else
	{
	    cerr << "no startup X10INIT" << endl;
	}

	exit( 0 );
    }

    // close unneeded files
    fclose( stdin );
    fclose( stdout );

    // run the daemon proper
    if ( p = defaults.getString( "X10D_DG_PATH" ) )
    {
    	daemon( p );
    }
    else
    {
	cerr << "X10D_DG_PATH not found!" << endl;
	daemonTerminate();
    }
}


void
markLog( char *m )
{
    char	s[ 255 ];
    time_t	clock;

    time( &clock );
    strcpy( s, m );
    strcat( s, ctime( &clock ) );
    cerr << s << endl;
}


//
// 
//
static void
initKeyMaps( void )
{
    const char* p;
    if ( p = defaults.getString( "BINX10KEYMAP" ) )
    {
	x10Map.readMap( (char*)p );
    }
    else
    {
	cerr << "couldn't find X10KEYMAP\n";
    }
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    int		index;

    // process optional arguments
    for ( index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {

	    case 'v':
		++verbose;
		break;

	    case 'k':
		if ( killX10d() ) exit( 1 );
		else exit( 0 );
		break;

	    default:
		cerr << form( "usage: %s [-kv]", argv[ 0 ] ) << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
