//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10daemon.cc,v 1.2 1996/03/31 01:54:12 smp Exp smp $";

// return X-10 device changes to clients
#define NOTIFY

#define NEW_LANGUAGE

//
// x10daemon.cc
//

#include <stdio.h>
#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#if defined( NOTIFY )
# include <SLList.h>
#endif  // NOTIFY

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/keymap.hh"
#include "util/misc/udgsocket.hh"

#if defined( NEW_LANGUAGE )
#include "util/misc/tokens.hh"
#endif  // NEW_LANGUAGE

#include "../daemon/rcserver.hh"
#include "../daemon/message.hh"
#include "../daemon/system.hh"

#include "x10.hh"
#include "x10d.hh"
#include "x10daemon.hh"
#include "x10convert.hh"
#include "x10action.hh"



#define ASCII_MESSAGES

#if defined( HAVE_VOICE )

#include "../daemon/voicedaemon.hh"

/// FIX ME when (rc)daemon becomes an object!!!
Voice*			vp;

#endif  // HAVE_VOICE


static int		visual = TRUE;
static X10ACTION	*keyMap[ KEYMAPSIZE ];

static u_char		x10vbuf[ READX10SIZE ];

static int		readX10( int );
static int		readUnixDG( UnixDGSocket * );

static X10ACTION	*mapAction( char * );
static X10ACTION	*addAction( int, int, int, int, u_char*, int, char* );

static int		rmvAction( X10ACTION * );
static void		showActions( X10ACTION * );

int			testKeys( u_char* k );
static int		testControl( u_char* k );

#if defined( ASCII_MESSAGES )
# if defined( NOTIFY )
static int		parseMessage( const char* args, UNIXSOCKADDR retAddr );
# else
static int		parseMessage( const char* args );
# endif  // NOTIFY
#endif  // ASCII_MESSAGES


#if defined( NOTIFY )

//
// these are all MAJOR BANDAIDS!!!
// x10d will be replaced by completely new version soon,
//  not going to bother doing this right!!!
//

// returned ASCII messages
#define OK_MSG			"<OK>"
#define FAIL_MSG		"<FAIL>"

int	deleteFromList( SLList<UNIXSOCKADDR>& list, UNIXSOCKADDR& addr );
void	printList( SLList<UNIXSOCKADDR>& list );
int	reportX10( int room, int unit, const char* s );
int	monitorX10( int what, int room, int unit, UNIXSOCKADDR& addr );
int	returnMessage( const char* msg, UNIXSOCKADDR* retAddr );

/// MAJOR, MAJOR BANDAID:
UnixDGSocket* monitorUDGS = NULL;

#define BRITE_DIM_STEP		5

enum BOOLEAN
{
    False = 0,
    True  = 1
};

class X10Device
{
    public :

#if 0
    X10Device( int area, int unit )
		: m_Area( area ), m_Unit( unit ),
		  m_State( OFF ), m_Level( 0 ), m_Monitors( NULL ) {}
#else
    X10Device( int area, int unit );
#endif
    virtual ~X10Device( ) {}

    virtual BOOLEAN Address( void ) {}
    virtual BOOLEAN On( void ) {}
    virtual BOOLEAN Off( void ) {}
    virtual BOOLEAN Bright( void ) {}
    virtual BOOLEAN Dim( void ) {}
    virtual BOOLEAN AllLightsOff( void ) {}
    virtual BOOLEAN AllLightsOn( void ) {}
    virtual BOOLEAN AllUnitsOff( void ) {}
    
    virtual const char* GetStatus( ) { return NULL; }
    
    SLList<UNIXSOCKADDR>*	GetMonitorsList() { return m_Monitors; }
    void			SetMonitorsList( SLList<UNIXSOCKADDR>* l )
    							{ m_Monitors = l; }

    void			SetName( const char* n ) { m_Name = n; }
    protected :

    enum DEV_STATE
    {
	UNKNOWN = 0,
	OFF	= 1,
	ON	= 2,
	LEVEL	= 3
    };

    int				notify( void );

    DEV_STATE			m_State;
    int				m_Level;

    BOOLEAN			m_bAddressed ;
    int				m_Area ;
    int				m_Unit ;

    const char*			m_Name ;
    char			m_Alias[ 6 ] ;

    SLList<UNIXSOCKADDR>*	m_Monitors;

    private :
    
};

#if 1
X10Device::X10Device( int area, int unit )
		: m_Area( area ), m_Unit( unit ),
		  m_State( OFF ), m_Level( 0 ), m_Monitors( NULL )
{
    sprintf( (char*)m_Alias, "%c %d", area + 'A', unit + 1 );
    m_Name = m_Alias;
}
#endif

int
X10Device::notify( void )
{
    char s1[ 128 ];
    char s2[ 256 ];

    // new state
    switch ( m_State )
    {
    case OFF:	strcpy( s1, "off" );			break;
    case ON:	strcpy( s1, "on" );			break;
    case LEVEL:	sprintf( s1, "level: %d%%", m_Level );	break;
    }

    // device name + new state
    sprintf( s2, "%s %s", m_Name, s1 );

    if ( m_Monitors != NULL )
    {
	reportX10( m_Area, m_Unit, s2 );
    }

    return OK;
}


class X10LampModule : public X10Device 
{
    public :
    
    X10LampModule( int area, int unit ) : X10Device( area, unit ) {}
    virtual ~X10LampModule( ) {}
    
    virtual BOOLEAN Address( void ) { return False; }
    virtual BOOLEAN On( void );
    virtual BOOLEAN Off( void );
    virtual BOOLEAN Bright( void );
    virtual BOOLEAN Dim( void );
    virtual BOOLEAN AllLightsOff( void );
    virtual BOOLEAN AllLightsOn( void );
    virtual BOOLEAN AllUnitsOff( void );

    virtual const char* GetStatus( void ) { return NULL; }
    
    private :

    
};

BOOLEAN
X10LampModule::On( )
{
    switch( m_State )
    {

    case OFF:
	m_State = ON;
	notify();
	break;

    case ON:
    case LEVEL:
    default:
	break;

    }
}

BOOLEAN
X10LampModule::Off( )
{
    switch( m_State )
    {

    case ON:
    case LEVEL:
	m_State = OFF;
	notify();
	break;

    case OFF:
    default:
	break;
    }
}

BOOLEAN
X10LampModule::Bright( )
{
    switch( m_State )
    {

    case LEVEL:
	// already @ max, NO state change
	if ( m_Level >= 100 ) break;

	// incrementing LEVEL
	m_Level += BRITE_DIM_STEP;
	if ( m_Level > 100 ) m_Level = 100;
	notify();
	break;

    case OFF:
    case ON:
	// changing from OFF/ON to LEVEL state
	m_State = LEVEL;
	m_Level = 100;
	notify();
	break;

    default:
	break;
    }
}

BOOLEAN
X10LampModule::Dim( )
{
    switch( m_State )
    {

    case LEVEL:
	// already @ min, NO state change
	if ( m_Level <= 0 ) break;

	// decrementing LEVEL
	m_Level -= BRITE_DIM_STEP;
	if ( m_Level < 0 ) m_Level = 0;
	notify();
	break;

    case OFF:
	m_State = LEVEL;
	m_Level = 100;
	notify();
	break;

    case ON:
	m_State = LEVEL;
	m_Level = 100 - BRITE_DIM_STEP;
	notify();
	break;

    default:
	break;
    }
}

BOOLEAN
X10LampModule::AllLightsOff( )
{
    switch( m_State )
    {

    case ON:
    case LEVEL:
	m_State = OFF;
	notify();
	break;

    case OFF:
    default:
	break;
    }
}

BOOLEAN
X10LampModule::AllLightsOn( )
{
    switch( m_State )
    {

    case OFF:
	m_State = ON;
	notify();
	break;

    case ON:		// nothing to do
    case LEVEL:		// ignored
    default:
	break;

    }
}

BOOLEAN
X10LampModule::AllUnitsOff( )
{
    switch( m_State )
    {

    case ON:
    case LEVEL:
	m_State = OFF;
	notify();
	break;

    case OFF:
    default:
	break;
    }
}


class X10DeviceRow // devices on the same house or area code
{
    public :
    
    X10DeviceRow( int area );
    virtual ~X10DeviceRow( ) {}
    
    X10Device& operator[ ]( int index ) { return *body[ index ]; }
    X10Device& operator[ ]( const char* index ); // row[ "P" ];
    
    void		Address( int unit ) ;
    void		Command( int key ) ;

    void		SetAddressed( int unit ) { m_Addressed = (1<<unit); }
    void		AddAddressed( int unit ) { m_Addressed |= (1<<unit); }
    BOOLEAN		IsAddressed( int unit )
			    { return (m_Addressed & (1<<unit)) ? True:False; }

    private :
    
    X10Device*		body[ 16 ];
    
    int			m_Area ;
    unsigned short	m_Addressed ;
    int			m_LastKey ;
};

X10DeviceRow::X10DeviceRow( int area )
{
    for( int x = 0 ; x < 16 ; x++ )
    {
        body[ x ] = new X10LampModule( area, x ) ;
    }

    m_Area = area;
    m_Addressed = 0;
    m_LastKey = FUNC_ALL_UNITS_OFF;	// arbitrary
}

void
X10DeviceRow::Address( int unit )
{
    // 1st address following a function clears address list
    if ( IS_A_FUNC( m_LastKey ) ) m_Addressed = 0;

    // 
    AddAddressed( unit ) ;

    // 
    m_LastKey = unit;
}

void
X10DeviceRow::Command( int key )
{
    int	unit;

    for ( unit = 0; unit < 16; ++unit )
    {
	switch ( key )
	{

	case KEY_OFF:
	    if ( IsAddressed( unit ) )	body[ unit ]->Off();		break;

	case KEY_ON:
	    if ( IsAddressed( unit ) )	body[ unit ]->On();		break;

	case KEY_DIM:
	    if ( IsAddressed( unit ) )	body[ unit ]->Dim();		break;

	case KEY_BRIGHT:
	    if ( IsAddressed( unit ) )	body[ unit ]->Bright();		break;

	case KEY_ALL_UNITS_OFF:		body[ unit ]->AllUnitsOff();	break;
	case KEY_ALL_LIGHTS_OFF:	body[ unit ]->AllLightsOff();   break;
	case KEY_ALL_LIGHTS_ON:		body[ unit ]->AllLightsOn();	break;

	case KEY_PRESET_DIM_LOW:
	case KEY_PRESET_DIM_HIGH:
	case KEY_HAIL_REQUEST:
	case KEY_HAIL_ACKNOWLEDGE:
	case KEY_EXTENDED_CODE:
	case KEY_EXTENDED_DATA:
	case KEY_STATUS_REQUEST:
	case KEY_STATUS_EQ_OFF:
	case KEY_STATUS_EQ_ON:
	default:							break;

	}
    }

    // 
    m_LastKey = unit;
}

class X10DeviceArray // the 16 X 16 grid
{
    public :
    
    X10DeviceArray( );
    virtual ~X10DeviceArray( ) {}
    
    X10DeviceRow& operator[ ]( int index ) { return *body[ index ]; }
    
    private :
    
    X10DeviceRow* body[ 16 ];
};

X10DeviceArray::X10DeviceArray( )
{
    for( int x = 0 ; x < 16 ; x++ )
    {
        body[ x ] = new X10DeviceRow( x ) ;
    }
}

X10DeviceArray	grid;

static void
stateEngine( int room, int key )
{
///cerr << form( "state engine, room: %c, key: 0x%02x", room + 'A', key ) << endl;

    // this is an ADDRESS
    if ( IS_A_UNIT( key ) )
    {
	grid[ room ].Address( key );
    }

    // must be a COMMAND
    else
    {
	grid[ room ].Command( key );
    }
}


//
// 
//
int
returnMessage( const char* msg, UNIXSOCKADDR* retAddr )
{
    int len = strlen( msg ) + 1;
    if ( monitorUDGS->send( (char *)msg, len, retAddr ) != len )
    {
	perror( "returnMessage(): monitorUDGS->send()" );
	return ERROR;
    }

    return OK;
}

// 
//  what == TRUE || FALSE: monitor ON || OFF
//
int
monitorX10( int what, int room, int unit, UNIXSOCKADDR& addr )
{
    char s[ 128 ];

    // get lists, etc.
    SLList<UNIXSOCKADDR>* list;
    list = grid[ room ][ unit ].GetMonitorsList();
    
    ///if ( list == NULL ) { cerr << "NULL list" << endl; }
    ///else { cerr << "list: " << endl; printList( *list ); }

    if ( what == TRUE )
    {
	// check for list
	if ( list == NULL )
	{
	    list = new SLList<UNIXSOCKADDR>;
	    grid[ room ][ unit ].SetMonitorsList( list );
	}

	// append return address to list
	(*list).append( addr );
    }

    else  // what == FALSE
    {
	// search list for instance of addr, delete if found
	if ( (list == NULL) || (deleteFromList( *list, addr ) == 0) )
	{
	    return ERROR;
	}

	if ( (*list).empty() )
	{
	    delete list;
	    grid[ room ][ unit ].SetMonitorsList( NULL );
	}
    }

    // done
    ///if ( list == NULL ) { cerr << "NULL list" << endl; }
    ///else { cerr << "list: " << endl; printList( *list ); }
    return OK;
}


//
// 
//
int
reportX10( int room, int unit, const char* s )
{
    // get the appropriate list
    SLList<UNIXSOCKADDR>& list = *grid[ room ][ unit ].GetMonitorsList();

    // walk list sending message to each client
    for ( Pix p = list.first(), lp = 0; p != 0;  )
    {
	if ( returnMessage( s, &((UNIXSOCKADDR&)list( p )) ) < 0 )
	{
	    // client dead?
	    monitorX10( FALSE, room, unit, (UNIXSOCKADDR&) list( p ) );

	    // backup pointer to 'last pointer'
	    if ( lp )
	    {
		p = lp;			// use 'last pointer'
		list.next( p );		//  to get 'new pointer'
	    }
	    else
	    {
		p = list.first();	// p was 1st, get new 1st
	    }
	}
	else
	{
	    lp = p;		   // advance 'last pointer' to pointer
	    list.next( p );    // advance pointer to 'next pointer'
	}
    }

    return OK;
}


//
// 
//
int
deleteFromList( SLList<UNIXSOCKADDR>& list, UNIXSOCKADDR& addr )
{
    int count = 0;

    ///cerr << form( "dt: %s, list:\n", addr.addr.sun_path ); printList(list);

    // walk list looking for EACH INSTANCE OF addr
    for ( Pix p = list.first(), lp = 0; p != 0;  )
    {
	if ( !strcmp( list( p ).addr.sun_path, addr.addr.sun_path ) )
	{
	    // delete the entry
	    list.del_after( lp );

	    // backup pointer to 'last pointer'
	    if ( lp )
	    {
		p = lp;			// use 'last pointer'
		list.next( p );		//  to get 'new pointer'
	    }
	    else
	    {
		p = list.first();	// p was 1st, get new 1st
	    }

	    // inc count of deleted nodes
	    ++count;
	}

	else
	{

	    lp = p;			// advance 'last pointer' to pointer
	    list.next( p );		// advance pointer to 'next pointer'
	}
    }

    ///cerr << "list out:\n"; printList( list );

    // return count of deleted nodes
    return count;
}


//
// 
//
void
printList( SLList<UNIXSOCKADDR>& list )
{
    for ( Pix p = list.first(); p != 0; list.next( p ) )
    {
	cerr << form( "client: %s", list( p ).addr.sun_path ) << endl;
    }
    cerr << endl;
}


#endif  // NOTIFY


//
// the daemon process:
//  wait for IO from either the reader process or the client socket(s):
//   reader process:
//    process x10 data as appropriate
//   socket:
//    process commands as appropriate
//
void
daemon( const char* p )
{
    int			maxFd, count, unixDG;
    fd_set		fdSet;
    UnixDGSocket	udgSocket( (char *)p );
#if defined( NOTIFY )
    // yet another BANDAID
    monitorUDGS = &udgSocket;

    // and yet another...
    const char* name;
    for( int area = 0 ; area < 16 ; area++ )
    {
        for( int unit = 0 ; unit < 16 ; unit++ )
        {
	    extern KeyMap x10Map;
	    if ( x10Map.getName( name, area, unit ) )
	    {
		grid[ area ][ unit ].SetName( name );
	    }
        }
    }
#endif  // NOTIFY

#if defined( HAVE_VOICE )

    /// FIX ME when (rc)daemon becomes an object!!!
    vp = new Voice;
    vp->say( "x ten deemon running\n" );

#endif  // HAVE_VOICE

    // setup data for select()
    unixDG = udgSocket.getFD();

    FD_ZERO( &fdSet );
    maxFd = MAX( readerPipe, unixDG );

    // fds are zero based indices, turn into a count
    ++maxFd;

    // main processing loop
    while ( TRUE )
    {

	// wait for traffic
	FD_SET( readerPipe, &fdSet );
	FD_SET( unixDG, &fdSet );

	count = select( maxFd, &fdSet, (fd_set*)0, (fd_set*)0,
		        (struct timeval*)0 );
	if ( count == -1 )
	{
	    perror( "select() fail" );
            daemonTerminate();
        }

	if ( FD_ISSET( readerPipe, &fdSet ) )
	{
	    if ( readX10( readerPipe ) == FALSE )
	    {
		daemonTerminate();
	    }
	}

	if ( FD_ISSET( unixDG, &fdSet ) )
	{
	    if ( readUnixDG( &udgSocket ) == FALSE )
	    {
		daemonTerminate();
	    }
	}
    }
}


//
// terminate reader & writer processes & cleanup ports
//
void
daemonTerminate( void )
{
    pid_t	pid;

    if ( writerPid ) kill( writerPid, SIGTERM );
    if ( readerPid ) kill( readerPid, SIGTERM );

    while ( TRUE )
    {
	if ( wait( (int *)0 ) > 0 ) continue;
	if ( errno == ECHILD ) break;
    }

/// in destructor???
///    unlink( X10D_DG_PATH );

    markLog( "X10 Daemon stopped: " );			// illegal

#if defined( HAVE_VOICE )

    /// FIX ME when (rc)daemon becomes an object!!!
    vp->say( "x ten deemon stopping\n" );

#endif  // HAVE_VOICE

    exit( 0 );
}


//
// 
//
int
killX10d( void )
{
    char		s[ 80 ];
    const char*		myPath;
    const char*		itsPath;
    NamedStr		defaults;

    // get the socket paths from the RC defaults file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load file: %s" << RCD_STRINGS << endl;
	return ERROR;
    }
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	  || !(itsPath = defaults.getString( "X10D_DG_PATH" )) )
    {
	cerr << "couldn't get socket paths" << endl;
	return ERROR;
    }

    // get a socket to the x10d server
    UnixDGSocket udgSocket( (char*)myPath, (char*)itsPath );
    if ( udgSocket.getFD() == ERROR )
    {
	cerr << "couldn't get socket fd" << endl;
	return ERROR;
    }

    // send the message to the x10d daemon
    s[ 0 ] = X10KILL;
    if ( udgSocket.send( s, 1 ) != 1 )
    {
	perror( "killX10d sendto()" );
	return ERROR;
    }

    // close shop & go home
    return OK;
}


#if defined( NO_ZOMBIES_ALLOWED )
static void
childDeath( void )
{
    wait( (int *)0 );
}
#endif  // NO_ZOMBIES_ALLOWED


//
// process an incoming x10 command
//

static int
readX10( int rp )
{
    static int	lastSlot = -1;			// timeslot of last key
    static int	lRoom = -1;
    static int	lKey = -1;

    int		count;
    int		slot, mapIndex;
    u_char	buf[ READX10SIZE ];
    X10ACTION	*headAction, *action;

    // a VIRTUAL command
    errno = 0;
    if ( rp == -1 )
    {
	memcpy( buf, x10vbuf, READX10SIZE );
    }

    // a REAL command
    else if ( (count = read( rp, buf, READX10SIZE )) == -1 )
    {
	perror( "daemon() pipe read()" );
        return FALSE;
    }
    else if ( count != READX10SIZE )
    {
	cerr << "invalid packet size on pipe: " << count << endl;
	return FALSE;
    }

    room = buf[ 0 ];
    key = buf[ 1 ];

    //
    // we keep time slices in hundredths of a second,
    //  to a max of 25599 hundredths
    // we toss any key coming in within MIN_X10_SEPARATION of the last
    //
    slot = ((int)buf[ 3 ] * 100) + buf[ 2 ];

    if ( (key == lKey) && (room == lRoom) )
    {
	if ( slot < lastSlot )
	{
	    if ( (slot + (25599-lastSlot)) < MIN_X10_SEPARATION ) return TRUE;
	}
	else
	{
	    if ( (slot - lastSlot) < MIN_X10_SEPARATION ) return TRUE;
	}
    }

    lastSlot = slot;

#if defined( NOTIFY )
    if ( rp != -1 )    // physical data from 117volt bus
    {
	stateEngine( room, key );
    }
#endif  // NOTIFY

    //
    // print the area & unit/function, followed by the reconstructed time
    //
    if ( visual )
    {
	time_t	clock;
	time( &clock );
	clock &= ~0xff;
	clock |= buf[ 3 ];

	extern KeyMap x10Map;
	const char* s;
	if ( !x10Map.getName( s, room, key ) )
	{
	    char str[ 80 ];
	    sprintf( str, "%c %2d", room + 'A', key + 1 );
	    s = str;
	}
	cerr << form( "<- %-20s", s );
	cerr << ctime( &clock );
    }

#if defined( HAVE_VOICE )
    if ( verbal )
    {
	char s[ 80 ];
	sprintf( s, "room %s\n", roomNames[ room ] );
	vp->say( s );

	if ( key <= KEY_16 )
	{
	    vp->say( "key\n" );
	}

	sprintf( s, "%s\n", keyNames[ key ] );
	vp->say( s );
    }
#endif  // HAVE_VOICE

    // walk the action list, running each entry
    if ( headAction = keyMap[ RMKY_TO_MAP( room, key ) ] )
    {
	for ( action = headAction;  ; action = action->next )
	{
	    if ( action->magic == OBJECT_MAGIC )
	    {
		if ( (!action->test) || (action->test(action->tArgs) == TRUE) )
		{
		    action->object->action();
		}
	    }

	    if ( action->next == headAction ) break;
	}
    }

    // update UNITS history
    if ( IS_A_UNIT( key ) )
    {
	if ( IS_A_FUNC( lastKey[ room ] ) )
	{
	    unitsGroup[ room ] = (1 << key);
	}
	else
	{
	    unitsGroup[ room ] |= (1 << key);
	}
    }

    // update room/key history
    lastKey[ room ] = key;
    if ( IS_A_UNIT( key ) )
    {
	lastUnit[ room ] = key;
    }
    else
    {
	lastFunction[ room ] = key;
    }

    // update key debounce data
    lRoom = room;
    lKey = key;

    return TRUE;
}


int
writeX10( u_char type, u_char* data )
{
    u_char	buf[ WRITEX10SIZE + 1 ];

    buf[ 0 ] = type;
    buf[ 1 ] = data[ 0 ];
    buf[ 2 ] = data[ 1 ];
    buf[ 3 ] = data[ 2 ];
    if ( write( writerPipe, buf, WRITEX10SIZE+1 ) != WRITEX10SIZE+1 )
    {
	perror( "pipe write()" );
	return ERROR;
    }

#if defined( NOTIFY )
    stateEngine( buf[ 1 ], buf[ 2 ] );
#endif  // NOTIFY

    if ( type == X10EXEC )
    {
	extern KeyMap x10Map;
	const char* s;
	if ( !x10Map.getName( s, buf[ 1 ], buf[ 2 ] ) )
	{
	    char str[ 80 ];
	    sprintf( str, "%c %2d", buf[ 1 ] + 'A', buf[ 2 ] + 1 );
	    s = str;
	}

	cerr << form( "-> %-20s", s );
	time_t	clock;
	time( &clock );
	cerr << ctime( &clock );
    }

    return OK;
}


static int
readUnixDG( UnixDGSocket* udgs )
{
    int			cnt;
    u_char		msg[ MAX_PIPE_MSG ];
    X10ACTION		*action;
    UNIXSOCKADDR	retAddr;
    struct timeval	tv;

    retAddr.len = sizeof( retAddr.addr );
    if ( (cnt = udgs->recv( (char *)msg, &retAddr )) < 0 )
    {
	perror( "recvfrom()" );
	return FALSE;
    }

    // effective EOF, time to quit
    if ( cnt == 0 ) return FALSE;

    // process packet
    switch ( msg[ 0 ] ) {

    case X10EXEC:				// ship x10 data
    	if ( writeX10( X10EXEC, &msg[1] ) == ERROR )
	{
	    perror( "writeX10()" );
	    return FALSE;
	}
	return TRUE;

    case X10VIRTUAL:				// virtual x10 control
	gettimeofday( &tv, NULL );
	x10vbuf[ 0 ] = msg[1];
	x10vbuf[ 1 ] = msg[2];
	x10vbuf[ 2 ] = tv.tv_usec / 10000;	// 10 millisec resolution
	x10vbuf[ 3 ] = tv.tv_sec;
	return readX10( -1 );

    case X10DUAL:				// direct & mapped x10 control
	/// should we test args against ACTUAL hardware???
	if ( writeX10( X10EXEC, &msg[1] ) == ERROR )
	{
	    perror( "writeX10()" );
	    return FALSE;
	}
	gettimeofday( &tv, NULL );
	x10vbuf[ 0 ] = msg[1];
	x10vbuf[ 1 ] = msg[2];
	x10vbuf[ 2 ] = tv.tv_usec / 10000;	// 10 millisec resolution
	x10vbuf[ 3 ] = tv.tv_sec;
	return readX10( -1 );

    case X10KILL:
	daemonTerminate();
	exit( 1 );				// should never get here!

    case MAP:
	action = mapAction( (char *)msg+1 );
	switch ( (u_long)action ) {
	case ERROR:				// fatal ERROR
	    return FALSE;
	case FALSE:				// soft ERROR
	    action = (X10ACTION *)NULL;
	default:
	    msg[ 0 ] = (u_long)action & 0xff;
	    msg[ 1 ] = (u_long)action >> 8;
	    msg[ 2 ] = (u_long)action >> 16;
	    msg[ 3 ] = (u_long)action >> 24;
	    break;
	}
	if ( udgs->send( (char *)msg, CNTL_BSIZE, &retAddr ) != CNTL_BSIZE )
	{
	    perror( "MAP sendto()" );
	    return FALSE;
	}
	if ( verbose ) showActions( keyMap[ action->index ] );
	return TRUE;

    case UNMAP:
	action = (X10ACTION *)strtoul( (char *)msg+1, (char **)NULL, 16 );
	if ( rmvAction ( action ) == FALSE )
	{
	    action = (X10ACTION *)NULL;
	}
	msg[ 0 ] = (u_long)action & 0xff;
	msg[ 1 ] = (u_long)action >> 8;
	msg[ 2 ] = (u_long)action >> 16;
	msg[ 3 ] = (u_long)action >> 24;
	if ( udgs->send( (char *)msg, CNTL_BSIZE, &retAddr ) != CNTL_BSIZE )
	{
	    perror( "UNMAP sendto()" );
	    return FALSE;
	}
	if ( verbose && action ) showActions( action );
	return TRUE;

    default:
#if defined( ASCII_MESSAGES )
# if defined( NOTIFY )
	switch( parseMessage( (const char*)msg, retAddr ) )
# else
	switch( parseMessage( (const char*)msg ) )
# endif
	{
	case TRUE:
	    return TRUE;

	case FALSE:
	    return TRUE;

	case ERROR:
	default:
	    return FALSE;
	}
#endif  // ASCII_MESSAGES

	cerr << form( "Illegal command: 0x%02x\n", msg[ 0 ] ) << endl;
	return TRUE;
    }
}


# if defined( COMMENT )

 statement-group:
    <pattern> <action> [action-option]

  pattern:
    Keys: <key> [... key]
    Control: <key> <key>

   key:
    <any X-10 keycode>

  action:
    <action-type>

   action-type:
    Message: <socket address> <command> [args]
    System: <ip address> <command> [args]

   socket address:
    <udg socket>
    <tcp socket>		(NOT implimented yet)

   ip address:
    <internet address>		(NOT implimented yet)

  action-option:
    Priority: <priority-value>

   priority-value:
    <-127 thru +127, where -127 := highest priority, +127 := lowest priority>

# endif  // COMMENT


typedef struct mapTag {
    char	*name;
    char	value;
    char	type;
} MAPTAG;

MAPTAG keyWords[] = 
{
  { "Keys:",		VALUE_KEYS,	TYPE_PATTERN },
  { "Control:",		VALUE_CONTROL,	TYPE_PATTERN },
  { "Status:",		VALUE_STATUS,	TYPE_PATTERN },

  { "Message:",		VALUE_MESSAGE,	TYPE_ACTION },
  { "System:",		VALUE_SYSTEM,	TYPE_ACTION },

  { "Priority:",	VALUE_PRIORITY,	TYPE_OPTION },

  { (char*)NULL, 	(char)NULL,	(char)NULL }
};


int
lookupTag( char* name, MAPTAG* tags, char& value, char& type )
{
    for ( int x = 0; tags[ x ].type; ++x )
    {
	if ( strcmp( name, tags[ x ].name ) == 0 )
	{
	    value = tags[x].value;
	    type = tags[x].type;
	    return TRUE;
	}
    }

    return FALSE;
}


#define MAX_MAP_LENGTH		1024

static X10ACTION*
mapAction( char* s )
{
    char	command[ 64 ];
    char	actionArgs[ MAX_MAP_LENGTH ];
    short	prio;
    u_char	room, key;
    int		x, y;

    char 	patternValue = 0;
    int		patternIndex = 0;
    int		patternArgc = 0;

    char	actionValue = 0;
    int		actionIndex = 0;
    int		actionArgc = 0;

    short	priorityValue = 0;
    int		priorityIndex = 0;

    int		testType = 0;
    u_char*	testArgs = 0;

    extern	KeyMap x10Map;

    /// cerr << "map: '" << s << "'" << endl;

    // anal
    if ( (x = strlen( s )) > MAX_MAP_LENGTH )
    {
	cerr << "MAP string too long: " << x << endl;
	return (X10ACTION*)ERROR;
    }

    // break string up into tokens
    Tokens& tokens = *new Tokens( s );
    int argc = tokens.count();

    // examine tokens:
    //  find keywords
    //   record keyword type & position
    char value, type;
    for ( x = 0; x < argc; ++x )
    {
	if ( lookupTag( tokens[ x ], keyWords, value, type ) == TRUE )
	{
	    switch ( type )
	    {
	    case TYPE_PATTERN:
		patternValue = value;	patternIndex = x;	break;

	    case TYPE_ACTION:
		actionValue = value;	actionIndex = x;	break;

	    case TYPE_OPTION:
		priorityValue = value;	priorityIndex = x;	break;

	    default:
		cerr << "bad TYPE_" << type << endl;		goto fubar;
	    }
	}
    }

    // check for missing fields
    if ( patternValue == 0 ) { cerr << "missing PATTERN" << endl; goto fubar; }
    if ( actionValue == 0 ) { cerr << "missing ACTION" << endl; goto fubar; }

    // record what we know
    patternArgc = actionIndex - 1;
    actionArgc = (priorityIndex ? priorityIndex : argc) - (patternArgc + 2) ;

///    cerr << "pi: " << patternIndex << " ai: " << actionIndex
///         << "pac: " << patternArgc << " aac: " << actionArgc << endl;

    //
    // process the PATTERN fields:
    //
    switch ( patternValue )
    {

    case VALUE_KEYS:	// process 'Keys' argument(s)

	if ( (patternArgc >= 1 ) && (patternArgc <= MAX_PATTERN_KEYS ) )
	{
	    // we 'fire' on the last key of pattern
	    x10Map.getKey( tokens[ patternArgc ], room, key );

	    // there may be additional key(s) in pattern to store for test
	    if ( patternArgc > 1 )
	    {
		testType = TEST_KEYS;
		testArgs = new u_char[ (2 * (patternArgc-1)) + 1 ];
		testArgs[ 0 ] = 2 * (patternArgc-1);
		for ( x = 1, y = 1; x < patternArgc; ++x, y += 2 )
		{
		    x10Map.getKey( tokens[ x ],
				    testArgs[ y ], testArgs[ y+1 ] );
		}
	    }
	}

	else
	{
	    cerr << "VALUE_KEYS: bad arg count: " << patternArgc << endl;
	    goto fubar;
	}

	break;


    case VALUE_CONTROL:	// process 'Control' argument(s)

	if ( patternArgc == 2 )
	{
	    // we 'fire' on the 2nd key of pattern
	    x10Map.getKey( tokens[ 2 ], room, key );

	    // we store 1st key for test
	    testType = TEST_CONTROL;
	    testArgs = new u_char[ 3 ];
	    testArgs[ 0 ] = 2;
	    x10Map.getKey( tokens[ 1 ], testArgs[ 1 ], testArgs[ 2 ] );
	}

	else
	{
	    cerr << "VALUE_CONTROL: bad arg count: " << patternArgc << endl;
	    goto fubar;
	}

	break;


    default:

	cerr << "bad patternValue: " << patternValue << endl;
	goto fubar;
    }


    //
    // process the ACTION fields:
    //

    // copy address, command, args
    actionArgs[ 0 ] = '\0';
    for ( x = actionIndex + 1, y = x + actionArgc; x < y; ++x )
    {
	strcat( actionArgs, tokens[ x ] ); strcat( actionArgs, " " );
    }


    //
    // process the PRIORITY fields:
    //

    if ( priorityValue )
    {
	prio = atoi( tokens[ priorityIndex + 1 ] );
	prio += DEFAULT_PRIORITY;
    }

    else
    {
	prio = DEFAULT_PRIORITY;
    }


    // return storage to system
    delete &tokens;

    // 
    if ( verbose )
    {
	cerr << (int)room << ", " << (int)key << ", " << (int)prio;
	cerr << ", testType: " << testType;
	if ( testArgs )
	  for ( x = 1; x <= testArgs[ 0 ]; ++x )
	      cerr << ", " << (int)testArgs[ x ];
	cerr << ", " << (int)actionValue;
	cerr << ", " << actionArgs << endl;
    }

#if 1
    X10ACTION* status;
    status = addAction( room, key, prio,
			    testType, testArgs,
			    actionValue, actionArgs );

    // return memory
    delete testArgs;

    return status;
#else
    delete testArgs;
    return (X10ACTION*)FALSE;
#endif

fubar:
    cerr << "fubar, returning ERROR" << endl;
    delete &tokens;
    return (X10ACTION*)ERROR;
}


//
// 
//
void
clearMap( void )
{
    for ( int x = 0; x < KEYMAPSIZE; ++x )
    {
	keyMap[ x ] = (X10ACTION*)0;
    }
}


//
// add a test/action pair to list
//
//  we have:
//   area, key, priority,
//   test type, (binary) test args,
//   action type, (ascii) action args,
//
X10ACTION*
addAction( int r, int k, int p,
	   int tType, u_char* tArgs,
	   int aType, char* aArgs )
{
    int		remote;
    int		index;
    X10ACTION*	ap;
    X10ACTION*	action;
    const char*	myPath;
    const char*	itsPath;

#if 0
    cerr << r << ", " << k << ", " << p;
    cerr << ", tType: " << tType;
    if ( tArgs )
      for ( int x = 1; x <= tArgs[ 0 ]; ++x )
	cerr << ", " << (int)tArgs[ x ];
    cerr << ", " << (int)aType;
    cerr << ", " << aArgs << endl;
#endif

    // break string up into address & message
    Tokens& tokens = *new Tokens( (char*)aArgs, 2 );
    int argc = tokens.count();

    if ( remote = (strcmp( "-", tokens[ 0 ] ) != 0) )
    {
	/// FIX ME
	/// eventually there will be a 'resolver' for msg addresses
	// get the socket paths from the RC defaults file
	if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	      || !(itsPath = defaults.getString( tokens[ 0 ] )) )
	{
	    delete &tokens;
	    cerr << "addAction() couldn't get socket paths\n";
	    return (X10ACTION *)ERROR;
	}
    }


    // create an action object
    Action*	object;
    switch ( aType )
    {

    case VALUE_MESSAGE:
	if ( remote )
	  object = new Message( tokens[ 1 ], myPath, itsPath );
	else
	  object = new X10Action( tokens[ 1 ] );
	break;

    case VALUE_SYSTEM:
	if ( remote )
#if defined( READY )
	  object = new System( tokens[ 1 ], myPath, itsPath );
#else
        {
	    cerr << "remote 'System' NOT supported yet!" << endl;
	    delete &tokens;
	    return (X10ACTION *)ERROR;
	}
#endif
	else
	  object = new System( tokens[ 1 ] );
	break;

    default:
	cerr << "addAction() bad Action!" << endl;
	delete &tokens;
	return (X10ACTION *)ERROR;
    }

    // return memory
    delete &tokens;

    // create the action structure
    action = new X10ACTION;

    action->magic = OBJECT_MAGIC;
    action->object = object;

    switch ( tType )
    {
    case TEST_KEYS:
	action->test = testKeys;
	break;
    case TEST_CONTROL:
	action->test = testControl;
	break;
    default:
	action->test = NULL;
	break;
    }

    // get permanent copy of test args
    if ( tArgs )
    {
	action->tArgs = new u_char[ tArgs[ 0 ] + 1 ];
	memcpy( action->tArgs, tArgs, (int)tArgs[ 0 ] + 1 );
    }

    action->flags = 0;
    action->group = 0;
    action->priority = p;
    action->index = RMKY_TO_MAP( r, k );

    // link the action structure into the linked list
    index = action->index;
    if ( keyMap[ index ] )
    {
	// link @ priority
	ap = keyMap[ index ];

	do
	{
	    if ( ap->priority > p ) break;
	}
	while ( (ap = ap->next) != keyMap[ index ] );

	if ( (ap == keyMap[ index ]) && (p < ap->priority) )
	{
	    keyMap[ index ] = action;
	}

	action->last = ap->last;
	action->last->next = action;
	action->next = ap;
	ap->last = action;
    }

    else
    {
	keyMap[ index ] = action;
	action->next = action->last = action;
    }

    return action;
}


//
// 
//
static int
rmvAction( X10ACTION *action )
{
    if ( action->magic != OBJECT_MAGIC ) return FALSE;	// test...

    if ( action->next == action )			// only entry
    {
	keyMap[ action->index ] = (X10ACTION *)NULL;	// NULL out the head
    }

    else						// other entries
    {
	if ( keyMap[ action->index ] == action )	// action is head entry
	{
	    keyMap[ action->index ] = action->next;	// head now head->next
	}
	action->last->next = action->next;		// unlink from list
	action->next->last = action->last;
    }

    // free any memory malloc()ed for this action
    if ( action->tArgs ) delete action->tArgs;

    delete action;

    return TRUE;
}


//
// 
//
static void
showActions( X10ACTION *action )
{
    X10ACTION *ap = action;

    if ( action == (X10ACTION *)NULL ) return;

    do
    {
	cerr << form( "action @ 0x%08x, prio: %d\n",
		      ap, ap->priority ) << endl;
    } while ( (ap = ap->next) != action );

    cerr << endl;
}


//
// test for series of key presses
//
int
testKeys( u_char* k )
{
    /// BYTE 0 holds count for future, longer strings

    // test lastKey[ room ] against target key
    if ( lastKey[ k[ 1 ] ] == k[ 2 ] )
    {
	return TRUE;
    }
    else
    {
	return FALSE;
    }
}


//
// test for X-10 unit addressed functions
//
static int
testControl( u_char* k )
{
    /// BYTE 0 holds count for future, longer strings

    // test unitGroup[ room ] against target key
    if ( unitsGroup[ k[ 1 ] ] & (1 << k[ 2 ]) )
    {
	return TRUE;
    }
    else
    {
	return FALSE;
    }
}


///////////////////////////////////////////////////////////////////////////////
#if defined( ASCII_MESSAGES )

void
fillx10vbuf( const u_char* buf )
{
    struct timeval	tv;

    gettimeofday( &tv, NULL );
    x10vbuf[ 0 ] = buf[ 0 ];
    x10vbuf[ 1 ] = buf[ 1 ];
    x10vbuf[ 2 ] = tv.tv_usec / 10000;	// 10 millisec resolution
    x10vbuf[ 3 ] = tv.tv_sec;
}

//
// parse ascii message into binary X10 command
//
static int
#if defined( NOTIFY )
parseMessage( const char* args, UNIXSOCKADDR retAddr )
#else
parseMessage( const char* args )
#endif  // NOTIFY
{
    /// FIX ME
    /// merge with common code in class X10Action

    // break string up into individual words
    Tokens& tokens = *new Tokens( (char*)args );
    int argc = tokens.count() - 1;

    // check command
    int	command;
    if ( strcmp( tokens[ 0 ], "dual" ) == 0 )
    {
	command = X10DUAL;
    }

    else if ( strcmp( tokens[ 0 ], "emit" ) == 0 )
    {
	command = X10EXEC;
    }

    else if ( strcmp( tokens[ 0 ], "virt" ) == 0 )
    {
	command = X10VIRTUAL;
    }

    else if ( strcmp( tokens[ 0 ], "toggleVerbal" ) == 0 )
    {
	verbal = verbal ? 0 : 1;
	delete &tokens;
	return TRUE;
    }

#if defined( NOTIFY )
    else if ( (strcmp( tokens[ 0 ], "monitor" ) == 0) )
    {
	// expected command line:
	//	<monitor> <on|off> <area.unit> [notification address]
	//	t[0]      t[1]     t[2]        t[3]
	// argc == 2 || 3

	// check arg count
	int count = argc + 1;
	if ( (count == 3) || (count == 4) )
	{
	    // determine return address for notification
	    UNIXSOCKADDR addr;
	    if ( count == 4 ) UnixDGSocket::makeAddress( addr, tokens[ 3 ] );
	    else addr = retAddr;

	    // decide if monitor will be ON or OFF
	    int action;
	    if ( strcmp( tokens[ 1 ], "on" ) == 0 )	  action = TRUE;
	    else if ( strcmp( tokens[ 1 ], "off" ) == 0 ) action = FALSE;
	    else					  goto failure;

	    /// parse tokens[ 2 ] for room & unit
	    extern KeyMap x10Map;	/// FIX ME
	    u_char room, unit;
	    if ( x10Map.getKey( tokens[ 2 ], room, unit ) == FALSE )
	    {
		cerr << "getKey FAILURE" << endl;
		goto failure;
	    }

	    // qualify unit
	    if ( unit >= 16 )
	    {
		cerr << "bad unit: %d " << unit << endl;
		goto failure;
	    }

	    // setup the monitor action
	    if ( monitorX10( action, room, unit, addr ) == ERROR )
	    {
		goto failure;
	    }

	    // done
	    delete &tokens;
	    returnMessage( OK_MSG, &retAddr );
	    return TRUE;
	}

 failure:
	cerr << "monitorX10 FAILURE" << endl;
	returnMessage( FAIL_MSG, &retAddr );
	delete &tokens;
	return FALSE;
    }
#endif  // NOTIFY

    else
    {
	cerr << "parseMessage(), bad command:" << tokens[ 0 ] << endl;
	delete &tokens;
	return FALSE;
    }

    // group contains: [room, key, repeat]
    if ( argc < 1 )
    {
	cerr << "parseMessage(), missing args" << endl;
	delete &tokens;
	return FALSE;
    }

    // 
    u_char* bArgs = new u_char[ argc * 3 ];

    // skip command, decode and store group(s)
    int index = 0;
    for ( int x = 1; x <= argc; ++x )
    {
	extern KeyMap x10Map;	/// FIX ME
	x10Map.getKey( tokens[ x ], bArgs[ index ], bArgs[ index+1 ] );
	index += 2;
	bArgs[ index++ ] = REPEAT_COUNT;
    }

    // return storage to system
    delete &tokens;

    // skip command, decode and store group(s)
    int status;
    for ( int x = 0; x < argc; ++x )
    {
	switch ( command )
	{
	case X10VIRTUAL:
	    fillx10vbuf( &bArgs[ x * 3 ] );
	    status = readX10( -1 );
	    break;

	case X10DUAL:
	    if ( writeX10( X10EXEC, &bArgs[ x * 3 ] ) == ERROR )
	    {
		perror( "writeX10()" );
		delete bArgs;
		return ERROR;
	    }
	    fillx10vbuf( &bArgs[ x * 3 ] );
	    status = readX10( -1 );
	    break;

	case X10EXEC:
	    if ( writeX10( X10EXEC, &bArgs[ x * 3 ] ) == ERROR )
	    {
		perror( "writeX10()" );
		delete bArgs;
		return ERROR;
	    }
	    break;

	default:
	    cerr << "parseMessage(): bArgs" << endl;
	    delete bArgs;
	    return FALSE;
	}
    }

    delete bArgs;
    return TRUE;
}


#endif  // ASCII_MESSAGES
