//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10reader.cc,v 1.3 1996/03/31 01:56:34 smp Exp smp $";


//
// x10 reader process
//

#include <stdio.h>
#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"

#include "x10d.hh"
#include "x10daemon.hh"
#include "x10convert.hh"

#if defined( DEV_TW )

// nothing special needed

#elif defined( SERPLIX )

#include "serplix.hh"

#elif defined( LYNX10 )

// nothing special needed

#else

# error You have to add hardware specific code to 'rcreader.c'!

#endif  // DEV_TW || SERPLIX || LYNX10 || ???


static void	readerTerminate( void );


//
// the reader process
//    pass x10 data to parent daemon
//
void
reader( int wPipe
#if defined( SERPLIX )
                 , int wId
#endif  // SERPLIX
)
{
    int			fd, flipFlop = 0;
    u_char		c, buf[ READX10SIZE ];
    const char*		file;
    struct sigaction	action;
    struct timeval	tv;

    fclose( stdin );
    fclose( stdout );

    markLog( "Reader started: " );

    // setup signal handlers
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;
#if 0 /**(defined(BSD) && (BSD >= 199306))*/
    action.sa_handler = readerTerminate;
#else
    action.sa_handler = (PFIV)readerTerminate;
#endif

    sigaction( SIGTERM, &action, (struct sigaction *)NULL );
    sigaction( SIGINT, &action, (struct sigaction *)NULL );
    sigaction( SIGQUIT, &action, (struct sigaction *)NULL );


    // open port
    if ( !(file = defaults.getString( "READERPORT" )) )
    {
	cerr << "couldn't find READERPORT" << endl;
	readerTerminate();
    }

    if ( (fd = open( file, O_RDONLY /**|O_NONBLOCK*/ )) < 0 )
    {
	perror( "open( X10 READERPORT )" );
	readerTerminate();
    }


    //
    // main loop for reader
    //
    while ( TRUE )
    {

#if defined( DEV_TW )

//
// From tw.c:
//

 // Reception is done using read().
 // The driver produces a series of three-character packets.
 // In each packet, the first character consists of flags,
 // the second character is a four bit house code (0-15),
 // and the third character is a five bit key/function code (0-31).
 // The flags are the following:

#define TW_RCV_LOCAL	1  // The packet arrived during a local transmission
#define TW_RCV_ERROR	2  // An invalid/corrupted packet was received

	if ( read( fd, buf, 3 ) < 3 )
	{
	    readerTerminate();
	}

	// 
	if ( buf[ 0 ] & TW_RCV_ERROR )
	{
	    continue;
	}

	// 
	if ( buf[ 0 ] & TW_RCV_LOCAL )
	{
	    continue;
	}

	// bump past ERROR code
	buf[ 0 ] = buf[ 1 ];
	buf[ 1 ] = buf[ 2 ];

	// timestamp packet
	gettimeofday( &tv, NULL );
	buf[ 2 ] = tv.tv_usec / 10000;		// 10 millisec resolution
	buf[ 3 ] = tv.tv_sec;

	// send packet to x10daemon
	if ( write( wPipe, buf, READX10SIZE ) != READX10SIZE )
	{
	    readerTerminate();
	}

#elif defined( SERPLIX )

	// signal the writer for each ACK byte from serplix
	// read serplix port for x10 data, timestamp it

	read( fd, &c, 1 );			// get the byte
	if ( c & ACK_BIT )			// its an ACK
	{
	    kill( wId, SIGUSR1 );
	}
	else					// its x10 data
	{
	    if ( !flipFlop )			// 1st half
	    {
		// get time stamp
		gettimeofday( &tv, NULL );
		buf[ 2 ] = tv.tv_usec / 10000;	// 10 millisec resolution
		buf[ 3 ] = tv.tv_sec;

		flipFlop = 1;
		buf[ 0 ] = x10ToArea( c );
	    }
	    else				// 2nd half
	    {
		flipFlop = 0;
		buf[ 1 ] = x10ToKey( c );
		if ( write( wPipe, buf, READX10SIZE ) != READX10SIZE )
		{
		    readerTerminate();
		}
	    }
	}

#elif defined( LYNX10 )

	// Read about a line's worth of data from the LynX-10.  The
	// last character we ignore; it's just a newline.
	int count;
	if ( (count = read( fd, buf, 5 )) == -1 )
        {
	    readerTerminate();
	}

	// All LynX-10 response codes start with `X0' or `X1'.  If
	// there's no `X0' or `X1', then ignore this read altogether.
        if ( ! ( buf[0] == 'X' && ( buf[1] == '0' || buf[1] == '1' ) ) )
        {        
	    continue;
	}

	// The house code is always the third byte.  Convert from
	// LynX-10's ASCII hex to an areaCode.
	buf[0] = lynX10ToArea( buf[2] );

	// The unit code is the fourth byte.  If the second byte is
	// `0', then the unit code refers to a unit ID.  If it's `1',
	// it's a command code.
	buf[1] = lynX10ToKey( buf[1] == '1', buf[3] );

	// Now timestamp it.
	gettimeofday( &tv, NULL );
	buf[ 2 ] = tv.tv_usec / 10000;		// 10 millisec resolution
	buf[ 3 ] = tv.tv_sec;

	// And send it on its way.
	if ( write( wPipe, buf, READX10SIZE ) != READX10SIZE )
	{
	    readerTerminate();
	}

#endif  // DEV_TW || SERPLIX || LYNX10

    }
}


static void
readerTerminate( void )
{
    markLog( "Reader stopped: " );		// Illegal

    exit( 0 );
}
