//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10writer.cc,v 1.3 1996/03/31 01:56:34 smp Exp smp $";


//
// x10 writer process
//

#include <stdio.h>
#include <iostream.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/file.h>

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"

#include "x10.hh"
#include "x10d.hh"
#include "x10daemon.hh"
#include "x10convert.hh"

#if defined( DEV_TW )

// nothing special needed

#elif defined( SERPLIX )

#include "serplix.hh"

#elif defined( LYNX10 )
  
// nothing special needed

#else

# error You have to add hardware specific code to 'x10writer.c'!

#endif  // DEV_TW || SERPLIX || LYNX10 || ???



#if defined( SERPLIX )
static int	initSerial( int );
static void	initSerplix( int );
#endif  // SERPLIX

#if defined( LYNX10 )
static int      initSerial( int );
#endif

static void	ack( void );
static void	writerTerminate( void );


//
// the writer process
//
void
writer( int rPipe )
{
    int			status, fd;
    const char*		file;
    sigset_t		set, oset;
    struct sigaction	action;
    u_char		buf[ WRITEX10SIZE + 1 ];

    fclose( stdin );
    fclose( stdout );

    markLog( "Writer started: " );

    // setup signal handlers
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;

#if 0 /**(defined(BSD) && (BSD >= 199306))*/
    action.sa_handler = ack;
#else
    action.sa_handler = (PFIV)ack;
#endif

    sigaction( SIGUSR1, &action, (struct sigaction *)NULL );

#if 0 /**(defined(BSD) && (BSD >= 199306))*/
    action.sa_handler = writerTerminate;
#else
    action.sa_handler = (PFIV)writerTerminate;
#endif

    sigaction( SIGTERM, &action, (struct sigaction *)NULL );
    sigaction( SIGINT, &action, (struct sigaction *)NULL );
    sigaction( SIGQUIT, &action, (struct sigaction *)NULL );

    // open port
    if ( !(file = defaults.getString( "WRITERPORT" )) )
    {
	cerr << "couldn't find WRITERPORT" << endl;
	writerTerminate();
    }


#if defined( DEV_TW )

    if ( (fd = open( file, O_WRONLY )) < 0 )
    {
	perror( "open( X10 WRITERPORT )" );
	writerTerminate();
    }

#elif defined( SERPLIX )

    if ( (fd = open( file, O_RDWR /**|O_NONBLOCK*/ )) < 0 )
    {
	perror( "open( X10 WRITERPORT )" );
	writerTerminate();
    }

    if ( initSerial( fd ) == ERROR )
    {
	perror( "initSerial( fd )" );
	writerTerminate();
    }

    // initialize the hardware
    initSerplix( fd );

    // prevent race conditions between reader & writer
    sigemptyset( &set );
    sigaddset( &set, SIGUSR1 );
    sigprocmask( SIG_BLOCK, &set, &oset );

#elif defined( LYNX10 )

    if ( (fd = open( file, O_RDWR )) < 0 )
    {
        perror( "open( X10 WRITERPORT )" );
	writerTerminate();
    }
    if ( initSerial( fd ) == ERROR )
    {
        perror( "initSerial( fd )" );
	writerTerminate();
    }

#endif  // DEV_TW || SERPLIX || LYNX10

    //
    // main loop for writer, read commands from daemon, send to ???
    //
    while ( TRUE )
    {
	if ( (status = read( rPipe, buf, WRITEX10SIZE+1 )) != WRITEX10SIZE+1 )
	{
	    cerr << form( "rcwriter(), read status: %d", status ) << endl;
	    writerTerminate();
	}

	switch ( buf[ 0 ] ) {
	case X10EXEC:

#if defined( DEV_TW )

//
// From tw.c:
//

 // Transmission is done by calling write() to send three byte packets of data.
 // The first byte contains a four bit house code (0=A to 15=P).
 // The second byte contains five bit unit/key code (0=unit 1 to 15=unit 16,
 // 16=All Units Off to 31 = Status Request).  The third byte specifies
 // the number of times the packet is to be transmitted without any
 // gaps between successive transmissions.  Normally this is 2, as per
 // the X-10 documentation;
	    buf[ 0 ] = buf [ 1 ];
	    buf[ 1 ] = buf[ 2 ];
	    buf[ 2 ] = 2;
	    write( fd, buf, 3 );

#elif defined( SERPLIX )

	    // buf -> 0: X10EXEC,
	    //        1: HOUSECODE,
	    //        2: UNIT/CMND,
	    //        3: repeat count (normally 0)
	    buf[ 1 ] = areaToX10( buf[ 1 ] );
	    buf[ 2 ] = keyToX10( buf[ 2 ] );
	    write( fd, &buf[ 1 ], 1 );
	    sigsuspend( &oset );			// wait for ACK signal
	    write( fd, &buf[ 2 ], 1 );
	    sigsuspend( &oset );			// wait for ACK signal
	    write( fd, &buf[ 3 ], 1 );
	    sigsuspend( &oset );			// wait for ACK signal

#elif defined( LYNX10 )

	    // We need the output packet to look like:
	    //   buf -> 0 = `X'
	    //          1 = type, `0' for addressing, `1' for command
	    //          2 = house in hex
	    //          3 = unit or command, in hex
	    int house;
	    unsigned char type, unit;

	    house = areaToLynX10( buf[ 1 ] );
	    unit = buf[ 2 ];
	    keyToLynX10( &type, &unit );

	    buf[ 0 ] = 'X';
	    buf[ 1 ] = type;
	    buf[ 2 ] = house;
	    buf[ 3 ] = unit;

	    write( fd, buf, 4 ); // bug: assumes success

#endif // DEV_TW || SERPLIX  || LYNX10

	    break;

	default:
	    cerr << form( "bad command to writer(): %d", buf[0] ) << endl;
	}
    }
}


static void
ack( void )
{
    // nothing to do but wait...
}


static void
writerTerminate( void )
{
    markLog( "Writer stopped: " );			// Illegal

    exit( 0 );
}


#if defined( SERPLIX )
//
// the serial port initialization function
//
static int
initSerial( int fd )
{
    struct termios	newTerm;

    if ( tcgetattr( fd, &newTerm ) == ERROR ) return ERROR;

    newTerm.c_cc[VMIN] = 1;
    newTerm.c_cc[VTIME] = 0;
    newTerm.c_iflag &= ~( BRKINT|
			  IGNPAR|PARMRK|INPCK|ISTRIP|
			  INLCR|IGNCR|ICRNL|IXON );

    newTerm.c_oflag &= ~(OPOST);

    newTerm.c_cflag &= ~( CSIZE|CSTOPB|HUPCL|PARENB );
    newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

    newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			  ISIG|
			  ICANON|IEXTEN|
			  NOFLSH|
			  TOSTOP);

    // get the desired baudrate
    const char*	baud;
    if ( !(baud = defaults.getString( "READERBAUD" )) )
    {
	cerr << "couldn't get READERBAUD" << endl;
	return ERROR;
    }
    if ( cfsetispeed( &newTerm, atoi( baud ) ) == ERROR ) return ERROR;

    if ( !(baud = defaults.getString( "WRITERBAUD" )) )
    {
	cerr << "couldn't get WRITERBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( baud ) ) == ERROR ) return ERROR;

    if ( tcsetattr( fd, TCSANOW, &newTerm ) == ERROR ) return ERROR;

    return OK;
}


//
// initialize the serial port PLIX device
//
static void
initSerplix( int fd )
{
    char		b0 = 0;
    char		b31 = 31;
    unsigned char	ack;

    //
    // initialize the plix chip,
    //  we assume that there is NO outside world x10 activity during this call
    //
top:
    for ( int x = 0; x < 3; ++x )
    {
	write( fd, &b0, 1 );
	read( fd, &ack, 1 );
	if ( !(ack & ACK_BIT) ) goto top;
    }

    write( fd, &b31, 1 );
    read( fd, &ack, 1 );
    if ( !(ack & ACK_BIT) ) goto top;
}
#endif	// SERPLIX


#if defined( LYNX10 )
//
// the serial port initialization function
//
static int
initSerial( int fd )
{
    struct termios	newTerm;

    if ( tcgetattr( fd, &newTerm ) == ERROR ) return ERROR;

    newTerm.c_iflag |= IGNBRK|IGNPAR|IXON|IXOFF;
    newTerm.c_iflag &= ~( BRKINT|PARMRK|INPCK|ISTRIP|IMAXBEL|IXANY );

    newTerm.c_oflag &= ~OPOST;

    newTerm.c_cflag |= CS8|CREAD|CLOCAL;
    newTerm.c_cflag &= ~( CSTOPB|PARENB|HUPCL|CCTS_OFLOW|CRTS_IFLOW|MDMBUF);

    newTerm.c_lflag |= ICANON;
    newTerm.c_lflag &= ~( ECHOKE|ECHOE|ECHO|ECHONL|ECHOPRT|ECHOCTL
			  |ISIG|ALTWERASE|IEXTEN|EXTPROC
			  |TOSTOP|FLUSHO|NOKERNINFO|PENDIN|NOFLSH );

    newTerm.c_cc[VERASE] = 0;
    newTerm.c_cc[VKILL]  = 0;
    newTerm.c_cc[VMIN]   = 1;
    newTerm.c_cc[VTIME]  = 0;

    // get the desired baudrate
    const char*	baud;
    if ( !(baud = defaults.getString( "READERBAUD" )) )
    {
	cerr << "couldn't get READERBAUD" << endl;
	return ERROR;
    }
    if ( cfsetispeed( &newTerm, atoi( baud ) ) == ERROR ) return ERROR;

    if ( !(baud = defaults.getString( "WRITERBAUD" )) )
    {
	cerr << "couldn't get WRITERBAUD" << endl;
	return ERROR;
    }
    if ( cfsetospeed( &newTerm, atoi( baud ) ) == ERROR ) return ERROR;

    if ( tcsetattr( fd, TCSANOW, &newTerm ) == ERROR ) return ERROR;

    return OK;
}
#endif  // LYNX10
