//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10map.c,v 1.1 1996/02/11 22:42:35 smp Exp smp $";


//
// x10map.cc
//

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "util/misc/misc.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/namedstr.hh"
#include "rc/daemon/rcserver.hh"
#include "rc/x10d/x10daemon.hh"


static int	verbose = 0;

static int	map( FILE* );
static int	getLine( char*, FILE* );
static int	doArgs( int, char ** );


main( int argc, char *argv[] )
{
    // get args
    if ( doArgs( argc, argv ) == ERROR ) exit( 1 );

    // body of client
    return map( stdin );
}


static int
map( FILE *fp )
{
    int		count, status;
    char	*p;
    u_char	request[ MAX_PIPE_MSG ];
    u_char	reply[ MAX_DG_MSG ];
    u_int	handle;
    const char*	myPath;
    const char*	itsPath;
    NamedStr	defaults;

    // get the socket paths from the RC defaults file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << form( "couldn't load necessary file: %s",
		      RCD_STRINGS ) << endl;
	exit( 1 );
    }
    if ( !(myPath = defaults.getString( "RCD_DG_TMP" ))
	|| !(itsPath = defaults.getString( "X10D_DG_PATH" )) )
    {
	cerr << form( "couldn't get socket paths" ) << endl;
	exit( 1 );
    }

    UnixDGSocket	udgSocket( (char*)myPath, (char*)itsPath );
    if ( udgSocket.getFD() == ERROR ) return BAD;

    while ( getLine( (p = (char*)request+1), fp ) )
    {

//
// here we would use our LEX machine to create the x10 defined grammer
// from the custom grammer input
//

        // remove whitespace
	while ( isspace( *p ) ) ++p;

	// skip comments
	if ( (*p == '\0') || (*p == '#') ) continue;

	// add appropriate header to request
	if ( (*p == 'u') || (*p == 'U') )
	{
	    for ( ++p; isspace( *p ); ++p );		// remove whitespace
	    *(p-1) = UNMAP;
	}
	else
	{
	    *(p-1) = MAP;
	}
//
// end of custom grammer engine
//

	// send x10 grammer to x10 daemon
	count = strlen( p ) + 2;
reSend:
	status = udgSocket.send( p-1, count );

	switch ( status )
	{
	case -1:
	    if ( errno == ENOBUFS ) goto reSend;
	    perror( "x10map, sendto()" );
	    return BAD;
	default:
	    if ( status != count )
	    {
		cerr << form( "status: %d", status );
		perror( " <- unexpected count, x10map sendto()" );
		cerr << endl;
		return BAD;
	    }
	}

	// get a response from the x10 daemon
	if ( (count = udgSocket.recv( (char *)reply )) < 0 )
	{
	    perror( "recvfrom()" );
	    return BAD;
	}
	handle = reply[ 0 ] & 0xff;
	handle |= (u_long)reply[ 1 ] << 8;
	handle |= (u_long)reply[ 2 ] << 16;
	handle |= (u_long)reply[ 3 ] << 24;

	/// test the result

	// 'return' the result
	if ( verbose )
	{
	    if ( (*(p-1)) == MAP )
	    {
		cout << form( "map handle: 0x%08x\n", handle );
	    }
	    else
	    {
		cout << form( "unmap returned: 0x%08x\n", handle );
	    }
	}
    }

    return OK;
}


static int
getLine( char* s, FILE* fp )
{
    int		rem = MAX_PIPE_MSG - 1;
    char*	p;

    while ( rem && fgets( s, rem, fp ) )
    {
	if ( p = strrchr( s, '\n' ) ) *p = '\0';
	else return NULL;

	if ( *(p-1) == '\\' )
	{
	    rem -= (strlen( p ) - 1);
	    s = p - 1;
	}
	else return TRUE;
    }

    return NULL;
}


//
// process command line arguments
//
static int
doArgs( int argc, char *argv[] )
{
    int		index;

    // process optional arguments
    for ( index = 1; index < argc; ++index )
    {
	if ( argv[ index ][ 0 ] == '-' )
	{
	    switch ( argv[ index ][ 1 ] )
	    {
	    case 'v':
		++verbose;
	        break;

	    default:
		cerr << form( "usage: %s [-v]", argv[ 0 ] ) << endl;
		return ERROR;
	    }
	}
	else break;
    }

    return OK;
}
