//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id$";


#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/ToggleB.h>
#include <Mrm/MrmPublic.h>

// rc client & server hook routines
#include "rc/daemon/rcserver.hh"
#include "rc/x10client/x10client.hh"
#include "rc/x10d/x10.hh"
#include "rc/x10d/x10daemon.hh"

#define X10UNIT(X)	(((X) - 1) / 2)

// number of rocker & pushbutton pairs supported
#define NUM_ROCKERS	0
#define NUM_PUSHPAIRS	0
#define NUM_PUSH	22

// instance & class names for X resource access
static char* appName = "x10maxi";
static char* appClass = "X10maxi";

// Motif Resource Manager stuff
static MrmHierarchy mrmId;
static char *mrmFile[]={"/usr/X11R6/lib/X11/uid/x10maxi.uid"};
static MrmCode mrmClass;


// callback functions
static void areaCB( Widget, int*, XtPointer );
static void pushBCB( Widget, int*, XmToggleButtonCallbackStruct* );
static void verbalCB( Widget, XtPointer, XtPointer );
static void exitCB( Widget, XtPointer, XtPointer );

// callback registration list for MRM
static MRMRegisterArg mrmNames[] = {
  { "areaCB",	(XtPointer)areaCB },
  { "pushBCB",	(XtPointer)pushBCB },
  { "verbalCB",	(XtPointer)verbalCB },
  { "exitCB",	(XtPointer)exitCB }
};

//
// set the fallback resources BY CLASS
//
static String fallbackResources[] = {
"X10maxi*XmText.columns: 10",
"X10maxi*XmTextField.columns: 10",
"X10maxi*fontList: *-*-Helvetica-medium-r-*-*-*-100-*-*-p-*-*-*",
"X10maxi*titleLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"X10maxi*labelLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"X10maxi*menuBar*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"X10maxi*popupMenu*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"X10maxi*background: grey30",
"X10maxi*XmToggleButton*background: grey60",
"X10maxi*menuBar*XmPushButton*background: white",
"X10maxi*XmPushButton*background: ghostwhite",
"X10maxi*XmFrame.XmLabel.foreground: yellow",
NULL
};

static XtAppContext	appContext;
static Widget		shell;

static int		x10Command;
static int		x10Area;
static int		x10UnitBase;

static void		initializeX10( void );
static void		initializeButtons( void );


typedef struct {
    char*		label;		// button label
    int			area;		// software house code
    unsigned char	base;		// zero-based base
} areaTag;

// label to x10 area associations for area menu
// NOTE: these MUST agree with callback args in x10maxi.uil
areaTag areaList[] = {
    { "Front Room",	AREA_F,	0 },
    { "Dining Room",	AREA_G, 4 },
    { "Kitchen",	AREA_H, 0 },
    { "Bath Room",	AREA_I, 0 },
    { "Bed Room",	AREA_K, 0 },
    { "Office",		AREA_J,	0 },
    { "Front Porch",	AREA_C, 0 },
    { "Garage",		AREA_D, 0 },
    { "Alarm",		AREA_A, 0 },
    { "Virtual",	AREA_M, 0 },
    { NULL,		-1 }
};


int
main( int argc, char **argv )
{
    Widget	appMain;
    Display*	display;
    Arg		args[2];

    // setup communications with the rc Daemon
    if ( attachDaemon() )
    {
	fprintf( stderr, "attachDaemon() failed!\n" );
	exit( 1 );
    }
    initializeX10();

    // initialize X
    XtToolkitInitialize();
    MrmInitialize();

    // top level X setup
    appContext = XtCreateApplicationContext();
    XtAppSetFallbackResources( appContext, fallbackResources );
    display = XtOpenDisplay( appContext, NULL, appName, appClass,
			     NULL, 0, &argc, argv );
    if ( display == NULL )
    {
	fprintf( stderr, "%s:  Can't open display\n", argv[0] );
	exit( 1 );
    }

    // create the top level shell
    XtSetArg( args[0], XmNargc, argc );
    XtSetArg( args[1], XmNargv, argv );
    shell = XtAppCreateShell( appName, appClass,
			      applicationShellWidgetClass, display, args, 2 );

    // initialize the MRM engine
    if (MrmOpenHierarchy( 1, mrmFile, NULL, &mrmId ) != MrmSUCCESS) exit(0);
    MrmRegisterNames( mrmNames, XtNumber(mrmNames) );
    ///MrmRegisterNames( mrmWidgets, XtNumber(mrmWidgets) );
    MrmFetchWidget( mrmId, "appMain", shell, &appMain, &mrmClass );
    XtManageChild( appMain );

    // make it all happen
    XtRealizeWidget( shell );

    // ...if only this was done with threads...
    XtAppMainLoop( appContext );
}


//
// setup an assummed x10 state
//
static void
initializeX10( void )
{
    x10Command = X10DUAL;
    x10Area = areaList[ 0 ].area;
    x10UnitBase = areaList[ 0 ].base;
}


//
// process the area menu
//
static void
areaCB( Widget w, int* clientData, XtPointer widgetData )
{
    x10Area = areaList[ *((int*)clientData) ].area;
    x10UnitBase = areaList[ *((int*)clientData) ].base;
}


//
// process the pushbuttons
//
static void
pushBCB( Widget w, int* id, XmToggleButtonCallbackStruct* cb )
{
    int	command;

    switch ( *id ) {
    case 1: command = KEY_1;			break;
    case 2: command = KEY_2;			break;
    case 3: command = KEY_3;			break;
    case 4: command = KEY_4;			break;
    case 5: command = KEY_5;			break;
    case 6: command = KEY_6;			break;
    case 7: command = KEY_7;			break;
    case 8: command = KEY_8;			break;
    case 9: command = KEY_9;			break;
    case 10: command = KEY_10;			break;
    case 11: command = KEY_11;			break;
    case 12: command = KEY_12;			break;
    case 13: command = KEY_13;			break;
    case 14: command = KEY_14;			break;
    case 15: command = KEY_15;			break;
    case 16: command = KEY_16;			break;
    case 17: command = KEY_ON;			break;
    case 18: command = KEY_OFF;			break;
    case 19: command = KEY_BRIGHT;		break;
    case 20: command = KEY_DIM;			break;
    case 21: command = KEY_ALL_LIGHTS_ON;	break;
    case 22: command = KEY_ALL_UNITS_OFF;	break;
    default: return;
    }

    sendX10Command( x10Command, x10Area, command );
}


//
// verbal button in menu bar
//
static void
verbalCB( Widget w, XtPointer cd, XtPointer cb )
{
    sendX10Command( x10Command, AREA_P, KEY_14 );
}


//
// exit button in menu bar
//
static void
exitCB( Widget w, XtPointer cd, XtPointer cb )
{
    exit( 0 );
}
