//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: x10mini.c,v 1.6 1995/02/07 04:23:11 smp Exp smp $";


#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/ToggleB.h>
#include <Mrm/MrmPublic.h>

// rc client & server hook routines
#include "rc/daemon/rcserver.hh"
#include "rc/x10client/x10client.hh"
#include "rc/x10d/x10.hh"
#include "rc/x10d/x10daemon.hh"


#define X10UNIT(X)	(((X) - 1) / 2)

// number of rocker & pushbutton pairs supported
#define NUM_ROCKERS		4
#define NUM_PUSHPAIRS	2

// instance & class names for X resource access
static char* appName = "x10mini";
static char* appClass = "X10mini";

// Motif Resource Manager stuff
static MrmHierarchy mrmId;
static char *mrmFile[]={"/usr/X11R6/lib/X11/uid/x10mini.uid"};
static MrmCode mrmClass;


// widget registration list for MRM
Widget rockers[ NUM_ROCKERS ][ 2 ];
static MRMRegisterArg mrmWidgets[] = {
  { "_rocker1a", (XtPointer)&rockers[ 0 ][ 0 ] },
  { "_rocker1b", (XtPointer)&rockers[ 0 ][ 1 ] },
  { "_rocker2a", (XtPointer)&rockers[ 1 ][ 0 ] },
  { "_rocker2b", (XtPointer)&rockers[ 1 ][ 1 ] },
  { "_rocker3a", (XtPointer)&rockers[ 2 ][ 0 ] },
  { "_rocker3b", (XtPointer)&rockers[ 2 ][ 1 ] },
  { "_rocker4a", (XtPointer)&rockers[ 3 ][ 0 ] },
  { "_rocker4b", (XtPointer)&rockers[ 3 ][ 1 ] }
};


// callback functions
static void areaCB( Widget, int*, XtPointer );
static void rockerCB( Widget, int*, XmToggleButtonCallbackStruct* );
static void pushBCB( Widget, int*, XmToggleButtonCallbackStruct* );
static void registerCB( Widget, XtPointer, XtPointer );
static void verbalCB( Widget, XtPointer, XtPointer );
static void exitCB( Widget, XtPointer, XtPointer );

// callback registration list for MRM
static MRMRegisterArg mrmNames[] = {
  { "registerCB",	(XtPointer)registerCB },
  { "areaCB",		(XtPointer)areaCB },
  { "rockerCB",	(XtPointer)rockerCB },
  { "pushBCB",	(XtPointer)pushBCB },
  { "verbalCB",	(XtPointer)verbalCB },
  { "exitCB",		(XtPointer)exitCB }
};

//
// set the fallback resources BY CLASS
//
static String fallbackResources[] = {
"X10mini*XmText.columns: 10",
"X10mini*XmTextField.columns: 10",
"X10mini*fontList: *-*-Helvetica-medium-r-*-*-*-100-*-*-p-*-*-*",
"X10mini*titleLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"X10mini*labelLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"X10mini*menuBar*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"X10mini*popupMenu*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"X10mini*background: grey30",
"X10mini*XmToggleButton*background: grey60",
"X10mini*menuBar*XmPushButton*background: white",
"X10mini*XmPushButton*background: tan",
"X10mini*XmFrame.XmLabel.foreground: yellow",
"X10mini*selectColor: ghostWhite",
NULL
};

static XtAppContext	appContext;
static Widget		shell;

static int		x10Command;
static int		x10Area;
static int		x10UnitBase;

static void		initializeX10( void );
static void		initializeButtons( void );


typedef struct {
    char*		label;		// button label
    int			area;		// software house code
    unsigned char	base;		// zero-based base
} areaTag;

// label to x10 area associations for area menu
// NOTE: these MUST agree with callback args in x10mini.uil
areaTag areaList[] = {
    { "Front Room",	AREA_F,	0 },
    { "Dining Room",	AREA_G, 4 },
    { "Kitchen",	AREA_H, 0 },
    { "Bath Room",	AREA_I, 0 },
    { "Bed Room",	AREA_K, 0 },
    { "Office",		AREA_J,	0 },
    { "Front Porch",	AREA_C, 0 },
    { "Garage",		AREA_D, 0 },
    { "Alarm",		AREA_A, 0 },
    { "Virtual",	AREA_M, 0 },
    { NULL,		-1 }
};


int
main( int argc, char **argv )
{
    Widget	appMain;
    Display*	display;
    Arg		args[2];

    // setup communications with the rc Daemon
    if ( attachDaemon() )
    {
	fprintf( stderr, "attachDaemon() failed!\n" );
	exit( 1 );
    }
    initializeX10();

    // initialize X
    XtToolkitInitialize();
    MrmInitialize();

    // top level X setup
    appContext = XtCreateApplicationContext();
    XtAppSetFallbackResources( appContext, fallbackResources );
    display = XtOpenDisplay( appContext, NULL, appName, appClass,
			     NULL, 0, &argc, argv );
    if ( display == NULL )
    {
	fprintf( stderr, "%s:  Can't open display\n", argv[0] );
	exit( 1 );
    }

    // create the top level shell
    XtSetArg( args[0], XmNargc, argc );
    XtSetArg( args[1], XmNargv, argv );
    shell = XtAppCreateShell( appName, appClass,
			      applicationShellWidgetClass, display, args, 2 );

    // initialize the MRM engine
    if (MrmOpenHierarchy( 1, mrmFile, NULL, &mrmId ) != MrmSUCCESS) exit(0);
    MrmRegisterNames( mrmNames, XtNumber(mrmNames) );
    MrmRegisterNames( mrmWidgets, XtNumber(mrmWidgets) );
    MrmFetchWidget( mrmId, "appMain", shell, &appMain, &mrmClass );
    XtManageChild( appMain );

    // make it all happen
    XtRealizeWidget( shell );

    // ...if only this was done with threads...
    XtAppMainLoop( appContext );
}


//
// setup an assummed x10 state
//
static void
initializeX10( void )
{
    x10Command = X10DUAL;
    x10Area = areaList[ 0 ].area;
    x10UnitBase = areaList[ 0 ].base;
}


//
// process the area menu
//
static void
areaCB( Widget w, int* clientData, XtPointer widgetData )
{
    x10Area = areaList[ *((int*)clientData) ].area;
    x10UnitBase = areaList[ *((int*)clientData) ].base;
}


//
// the on/off rocker buttons
//
static void
rockerCB( Widget w, int* id, XmToggleButtonCallbackStruct* cb )
{
    int unit = X10UNIT( *id );

    sendX10Command( x10Command, x10Area, unit + x10UnitBase );

    if ( *id & 1 )    // turn the device on
    {
	sendX10Command( x10Command, x10Area, KEY_ON );
    }
    else		// turn the device off
    {
	sendX10Command( x10Command, x10Area, KEY_OFF );
    }

    // set this half of rocker
    XmToggleButtonSetState( w, True, False );

    // reset the opposite half of rocker
    XmToggleButtonSetState( rockers[ unit ][ *id & 1 ], False, False );
}


//
// process the tan pushbuttons
//
static void
pushBCB( Widget w, int* id, XmToggleButtonCallbackStruct* cb )
{
    int	command;

    switch ( *id )
    {
    case 1: command = KEY_BRIGHT;		break;
    case 2: command = KEY_DIM;			break;
    case 3: command = KEY_ALL_LIGHTS_ON;	break;
    case 4: command = KEY_ALL_UNITS_OFF;	break;
    default: return;
    }

    sendX10Command( x10Command, x10Area, command );
}


//
// register important widgets @ creation time for later access
//
static void
registerCB( Widget w, XtPointer clientData, XtPointer callData )
{
    *((Widget*)clientData) = w;
}


//
// verbal button in menu bar
//
static void
verbalCB( Widget w, XtPointer cd, XtPointer cb )
{
    sendX10Command( x10Command, AREA_P, KEY_14 );
}


//
// exit button in menu bar
//
static void
exitCB( Widget w, XtPointer cd, XtPointer cb )
{
    exit( 0 );
}
