//
// Copyright 1995,1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHOR (Steve Passe)
// is not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHOR makes no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHOR
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// AUTHOR:     Steve Passe <smp@csn.net>
//

#define DEBUG
#define FLATWHEELS


//
// xmltherm.cc, monitor MicroLAN(tm) temp probes
//

static char rcsid[] = "$Id: xmltherm.cc,v 1.3 1997/01/09 20:27:09 smp Exp smp $";

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

// Xlib headers
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

// Standard Xt headers
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

// FWF Widget Set headers
#include <Xfwf/ThWheel.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>

// Athena Widget Set headers
#include <X11/Xaw/Form.h>

// Control Panel Widget Set headers
#include <X11/Xc/Xc.h>
#include <X11/Xc/Lcd2.h>


// header for this process
#include "xmltherm.hh"

#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/udgsocket.hh"
#include "rc/daemon/rcserver.hh"

struct DEVNAMES {
    const char*		name;
} devNames[] =
{
  { "<noname>"			},
  { "greenhouse.loshade"	},
  { "greenhouse.hisun"		},
  { "compcloset.hi"		},
  { "compcloset.mid"		},
  { "outside.northshade"	},
  { "outside.southsun"		},
  { "compcloset.lo"		},
  { "attic.center"		}
};


// Widgets
Widget TopLevel;
Widget		MainBox;
display_t		gauge[ MAX_CHANNELS ];


static String fallback_resources[] = {

    ".channelCount:			2",
    ".channelList:			",

    // per channel stuff
    "*Form.Label.background:		" HI_BACKGROUND,
#if defined( FLATWHEELS )
    "*Form.Label.width:			250",
#else
    "*Form.Label.width:			215",
#endif

    // lcd displays
    "*Lcd2.shadeDepth:			2",
    "*Lcd2.background:			" LO_BACKGROUND,
    "*Lcd2.valueForeground:		orange",
    "*Lcd2.valueBackground:		black",
    "*Lcd2.width:			150",
    "*Lcd2.height:			65",
    "*Lcd2.dataType:			lval",
    "*Lcd2.upperBound:			300",
    "*Lcd2.lowerBound:			40",
    "*Lcd2.value:			60",
    "*Lcd2.valueLength:			3",
    "*Lcd2.interval:			30000",		// 30 seconds

    // ALARM1/ALARM2 formes
#if defined( FLATWHEELS )
    "*Form.Form.Label.internalHeight:	1",
#endif
    "*Form.Form.Label.width:		29",
    "*Form.Form.Label.value:		---",
    "*Form.Form.Label.background:	" HI_BACKGROUND,
///    "*XfwfThumbWheel.vertical:		False",
    "*XfwfThumbWheel.minValue:		-1",
    "*XfwfThumbWheel.maxValue:		1000",
    "*XfwfThumbWheel.value:		-1",
    "*XfwfThumbWheel.step:		1",

    // all others
    "*background:			" DEFAULT_BACKGROUND,
    NULL,
};


static int channelCount = 1;

typedef struct {
	int	channels;
	char	*list;
} appResources_t, *appResourcesPtr;

static appResources_t	appResources;

static XtResource	resources[] = {
#define offset(field)		XtOffset(appResourcesPtr, field)
	{ XtNchannelCount, XtCChannelCount, XtRInt, sizeof(int), 
		offset(channels), XtRImmediate, (XtPointer) 2 },
	{ XtNchannelList, XtCChannelList, XtRString, sizeof(char *),
		offset(list), XtRImmediate, "" },
#undef offset
};

static XrmOptionDescRec options[] = {
	{ "-c",	".channelCount",	XrmoptionSepArg, (caddr_t) 2 },
	{ "-l",	".channelList",		XrmoptionSepArg, (caddr_t) "" },
};


//
// private functions
//
static void LcdCallback( Widget, XtPointer, XcData );
static void WheelAlarm1Callback( Widget, XtPointer, XtPointer );
static void WheelAlarm2Callback( Widget, XtPointer, XtPointer );
static void WheelRateCallback( Widget, XtPointer, XtPointer );

//
static int getSample( const char* /*int*/ channel );

static void
buildAdj( char* name, int instance,
	  adjustor_t* ap,
	  Widget parent, Widget ftop, Widget fleft );

// create the (ONE and ONLY ONE) instance of the client object
MLControl* mlcPtr;



///int input, output;


//
// 
//
void
main( int argc, String *argv )
{
    XtAppContext app_context;
    Display *display;
    Arg args[ 10 ];
    char s[ 133 ];
    int value;
    int list[ MAX_CHANNELS + 1 ];
    int x, xx;
    int i;

    // create the (ONE and ONLY ONE) instance of the daemon object
    MLControl mlc( argv[ 0 ] );
    mlcPtr = &mlc;

    // initialize client
    if ( mlc.initialize( "MLD_DG_PATH", (char*)NULL )
	  == ERROR )
    {
	cerr << "mlcontrol failure!" << endl;
	exit( 1 );
    }

    // create top level shell
    TopLevel = XtVaAppInitialize( &app_context, "Xmltherm",
				  options, 2, &argc, argv,
				  fallback_resources, NULL);

    // process remaining commandline args
    XtGetApplicationResources(TopLevel, &appResources, resources,
			      XtNumber(resources), 0, 0);

    if ( (channelCount = appResources.channels) > MAX_CHANNELS )
    {
	channelCount = MAX_CHANNELS;
    }
    
    if ( appResources.list && *appResources.list )
    {
	x = sscanf( appResources.list, "%d,%d,%d,%d,%d,%d,%d,%d",
	        &list[0], &list[1], &list[2], &list[3],
	        &list[4], &list[5], &list[6], &list[7] );
	if ( x > MAX_CHANNELS )
	{
	    fprintf( stderr, "bad channel list\n" );
	    exit(1);
	}

	channelCount = x;
	while ( x < MAX_CHANNELS )
	{
	    list[x++] = (int)NULL;
	}
    }
    else
    {
	for ( x = 0; x < channelCount; ++x )
	{
	    list[ x ] = x+1;
	}
    }

#if defined( DEBUGxxx )
    fprintf( stderr, "channels:" );
    for ( x = 0; x < MAX_CHANNELS; ++x )
    {
	if ( list[x] ) fprintf( stderr, " %d", list[x] );
    }
    fprintf( stderr, "\n" );
#endif // DEBUG

    // general X stuff
    display = XtDisplay(TopLevel);
 
#if defined( ADD_INPUT )
    // 
    XtAppAddInput( app_context, temperature.getInputFD(),
		   XtInputReadMask, update, (XtPointer)NULL );
#endif  // ADD_INPUT

    // set resources for the toplevel shell
#if 0
    i = 0;
    XtSetArg(args[i], XtNminWidth, 250);  i++;
    XtSetValues(TopLevel, args, i);
#endif

    // the top level container
    MainBox = XtVaCreateManagedWidget("mainbox", boxWidgetClass,
				      TopLevel, NULL);
    i = 0;
    XtSetArg(args[i], XtNdefaultDistance, 0);  i++;
    XtSetValues(MainBox, args, i);

    i = 0;
    XtSetArg(args[i], XtNvSpace, 0); i++;
    XtSetArg(args[i], XtNhSpace, 0); i++;
    XtSetValues(MainBox, args, i);


    // create forms, lcd displays, wheels, labels, etc.
    for ( xx = 0; xx < channelCount; ++xx )
    {
	// get indexes from ordered list
	x = list[xx]-1;

	//
	// outer form
	//
	sprintf( s, "form_%d", x+1 );
	gauge[ x ].form = XtVaCreateManagedWidget(s, formWidgetClass,
					   MainBox, NULL);
	i = 0;
	XtSetArg(args[i], XtNvertDistance, 0); i++;
	XtSetArg(args[i], XtNhorizDistance, 0); i++;
	XtSetArg(args[i], XtNdefaultDistance, 0); i++;
	XtSetValues(gauge[ x ].form, args, i);

	//
	// channel label
	//
	sprintf( s, "channel_%d", x+1 );
	gauge[ x ].label = XtVaCreateManagedWidget(s, labelWidgetClass,
					     gauge[ x ].form, NULL);

#if 0
	//
	// channel device
	//
	gauge[ x ].device = x + 1;
#endif

	//
	// channel display
	//
	sprintf( s, "lcd_%d", x+1 );
	gauge[ x ].lcd = XtVaCreateManagedWidget(s, xcLcd2WidgetClass,
					 gauge[ x ].form, NULL);
	i = 0;
	XtSetArg(args[i], XtNfromVert, gauge[ x ].label); i++;
	XtSetValues(gauge[ x ].lcd, args, i);

	//
	// build ALARM1 form
	//
	buildAdj( "alarm1", x, &gauge[x].alarm1,
		  gauge[x].form, gauge[x].label, gauge[x].lcd );

	/// temp hack
	gauge[x].alarm1.type = -1;

	WheelAlarm1Callback( gauge[ x ].alarm1.wheel, (XtPointer)x, NULL );

	//
	// build ALARM2 form
	//
	buildAdj( "alarm2", x, &gauge[x].alarm2,

#if defined( FLATWHEELS )
		  gauge[x].form, gauge[x].alarm1.form, gauge[x].lcd );
#else
		  gauge[x].form, gauge[x].label, gauge[x].alarm1.form );
#endif

	/// temp hack
	gauge[x].alarm2.type = 1;

	WheelAlarm2Callback( gauge[ x ].alarm2.wheel, (XtPointer)x, NULL );

	//
	// build RATE form
	//
	buildAdj( "rate", x, &gauge[x].rate,

#if defined( FLATWHEELS )
		  gauge[x].form, gauge[x].alarm2.form, gauge[x].lcd );
#else
		  gauge[x].form, gauge[x].label, gauge[x].alarm2.form );
#endif

	WheelRateCallback( gauge[ x ].rate.wheel, (XtPointer)x, NULL );

	//
	// register callback functions
	//
	XtAddCallback( gauge[x].lcd, XcNupdateCallback,
		      (XtCallbackProc)LcdCallback, (XtPointer)x );
	XtAddCallback( gauge[x].alarm1.wheel, XtNscrollCallback,
		      (XtCallbackProc)WheelAlarm1Callback, (XtPointer)x );
	XtAddCallback( gauge[x].alarm2.wheel, XtNscrollCallback,
		      (XtCallbackProc)WheelAlarm2Callback, (XtPointer)x );
	XtAddCallback( gauge[x].rate.wheel, XtNscrollCallback,
		      (XtCallbackProc)WheelRateCallback, (XtPointer)x );

	// all guages use the same process for sampling
#if 0
        gauge[x].proc = &temperature;
#endif
    }

    XtRealizeWidget(TopLevel);
    XtAppMainLoop(app_context);
}


//
// 
//
static void
buildAdj( char* name, int instance,
	  adjustor_t* ap,
	  Widget parent, Widget ftop, Widget fleft )
{
    Arg args[ 10 ];
    int i;
    char s[ 64 ];

    ++instance;

    sprintf( s, "%sA_%d", name, instance );
    ap->form = XtVaCreateManagedWidget(s, formWidgetClass, parent, NULL);

    // define placement of container
    i = 0;
    XtSetArg( args[i], XtNvertDistance, 0 ); i++;
    XtSetArg( args[i], XtNhorizDistance, 0 ); i++;
    XtSetArg( args[i], XtNdefaultDistance, 0 ); i++;
    XtSetArg( args[i], XtNfromVert, ftop ); i++;
    XtSetArg( args[i], XtNfromHoriz, fleft ); i++;
    XtSetValues( ap->form, args, i );

    // create adjustor label
    sprintf( s, "%sL_%d", name, instance );
    ap->label = XtVaCreateManagedWidget( s, labelWidgetClass,
					 ap->form, NULL );

    // create adjustor wheel
    sprintf( s, "%sW_%d", name, instance );
    ap->wheel = XtVaCreateManagedWidget( s, xfwfThumbWheelWidgetClass,
					 ap->form, NULL );

    // plug default values into structure
    XtVaGetValues( ap->wheel, XtNminValue, &(ap->min), NULL );
    XtVaGetValues( ap->wheel, XtNmaxValue, &(ap->max), NULL );
    XtVaGetValues( ap->wheel, XtNvalue, &(ap->val), NULL );

    i = 0;
#if defined( FLATWHEELS )
    XtSetArg( args[i], XtNfromHoriz, ap->label ); i++;
#else
    XtSetArg( args[i], XtNfromVert, ap->label ); i++;
#endif
    XtSetValues( ap->wheel, args, i );
}


//
// updateCallback for the Lcd widgets
//
static
void LcdCallback( Widget w, XtPointer client_data, XcData call_data )
{
    int x, alarm1, alarm2;
    char s[ 64 ];
    XcVType value;

    x = (int)client_data;

    strcpy( s, OK_FCOLOR );
    alarm1 = alarm2 = 0;

    // get a sample
#if 0
    /// FIXME: get this into the .ad file
    value.lval = (long)getSample( gauge[ x ].deviceList[?] );
#else
///    int xxx[] = { 6,5,2,1,3,4,7,8 };
    const char* foo[] = {
      { "outside.southsun" },
      { "outside.northshade" },
      { "greenhouse.hisun" },
      { "greenhouse.loshade" },
      { "compcloset.hi" },
      { "compcloset.mid" },
      { "compcloset.lo" },
      { "attic.centertherm" },
    };
    value.lval = (long)getSample( foo[ x ] );
#endif
    /// FIXME: handle ERRORs

    // determine if ALARM1 needed
    XtVaGetValues( gauge[ x ].alarm1.wheel, XtNvalue, &alarm1, NULL );
    if ( ((gauge[ x ].alarm1.type < 0) && (value.lval <= alarm1))
	 || ((gauge[ x ].alarm1.type > 0) && (value.lval >= alarm1)) )
    {
	alarm1 = 1;
	strcpy( s, ALARM1_FCOLOR );
    }

    // determine if ALARM2 needed
    XtVaGetValues( gauge[ x ].alarm2.wheel, XtNvalue, &alarm2, NULL );
    if ( ((gauge[ x ].alarm2.type < 0) && (value.lval <= alarm2))
	 || ((gauge[ x ].alarm2.type > 0) && (value.lval >= alarm2)) )
    {
	alarm2 = 1;
	strcpy( s, ALARM2_FCOLOR );
    }

    // update lcd color
    XtVaSetValues( gauge[ x ].lcd,
		  XtVaTypedArg, XcNvalueForeground, XtRString, s, sizeof(s)+1,
		  NULL );

    // return the new value to lcd
    call_data->value.lval=value.lval; 

    ///
    /// process alarm1 & alarm2
    ///

}


//
// 
//
static
void WheelAlarm1Callback( Widget w, XtPointer client_data, XtPointer call_data )
{
    int x = (int)client_data;
    int value;
    char s1[64];
    char s2[64];

    // get new value of wheel
    XtVaGetValues( w, XtNvalue, &value, NULL );

    if ( (value > gauge[ x ].alarm1.min) && (value < gauge[ x ].alarm1.max) )
    {
	sprintf( s1, "%03d", value );
	strcpy( s2, ALARM1_COLOR );
    }
    else
    {
	strcpy( s1, NIL_LABEL );
	strcpy( s2, NIL_COLOR );
    }

    // update label
    XtVaSetValues( gauge[ x ].alarm1.label,
		  XtNlabel, s1,
		  XtVaTypedArg, XtNbackground, XtRString, s2, sizeof(s2)+1,
		  NULL );
}


//
// 
//
static
void WheelAlarm2Callback( Widget w, XtPointer client_data, XtPointer call_data )
{
    int x = (int)client_data;
    int value;
    char s1[64];
    char s2[64];


    XtVaGetValues( w, XtNvalue, &value, NULL );

    if ( (value > gauge[ x ].alarm1.min) && (value < gauge[ x ].alarm2.max) )
    {
	sprintf( s1, "%03d", value );
	strcpy( s2, ALARM2_COLOR );
    }
    else
    {
	strcpy( s1, NIL_LABEL );
	strcpy( s2, NIL_COLOR );
    }

    // update label
    XtVaSetValues( gauge[ x ].alarm2.label,
		  XtNlabel, s1,
		  XtVaTypedArg, XtNbackground, XtRString, s2, sizeof(s2)+1,
		  NULL );
}


//
// 
//
static
void WheelRateCallback( Widget w, XtPointer client_data, XtPointer call_data )
{
    int x = (int)client_data;
    int value;
    char s1[64];
    char s2[64];

    // get new value of wheel
    XtVaGetValues( w, XtNvalue, &value, NULL );

    if ( (value > gauge[ x ].rate.min) && (value < gauge[ x ].rate.max) )
    {
	sprintf( s1, "%03d", value );
	strcpy( s2, RATE_COLOR );
    }
    else
    {
	strcpy( s1, NIL_LABEL );
	strcpy( s2, NIL_COLOR );
    }

    XtVaSetValues( gauge[ x ].rate.label,
		  XtNlabel, s1,
		  XtVaTypedArg, XtNbackground, XtRString, s2, sizeof(s2)+1,
		  NULL );

    // update sample rate
    /// TODO: make this a log scale in software
    value *= 1000;
    XtVaSetValues( gauge[ x ].lcd,
		  XcNinterval, value,
		  NULL );
}


//
// 
//
int
getSample( const char* /*int*/ channel )
{

#if !defined( FAKE_SAMPLE )

    // send the message to the daemon
    int count;
    char cmd[ 256];
///    sprintf( cmd, "f %d", channel );
    sprintf( cmd, "F %s", channel );
    int length = strlen( cmd ) + 1;
    while ( (count = mlcPtr->writeDaemonMsg( cmd, length )) != length )
    {
	if ( count == -1 )
	{
	    if ( errno == ENOBUFS ) continue;
	    perror( "writeDaemonMsg() ERROR" );
	}
	else
	{
	    cerr << "writeDaemonMsg() count: " << count << endl;
	}
	return ERROR;
    }

    char message[ MAX_DG_MSG ];
    if ( mlcPtr->readDaemonMsg( message ) < 0 )
    {
	perror( "readDaemonMsg()" );
	return ERROR;
    }

    // FIXME: kludge
    char* p;
///    if ( p = strchr( message, '#' ) )
    if ( p = strrchr( message, ':' ) )
    {
#if 0
	char s[ 256 ];
	sprintf( s, "The %s temp is now %s degrees.\n",
		devNames[ atoi( p+1 ) ].name,
		p+4 );
	cout << s;
#endif
	return atoi( p+1 );
    }

    return ERROR;


#else  // FAKE_SAMPLE

    int r;

    fprintf( stderr, "sampling...\n" );

    r = rand() & 0x1f;			// 0 -> 31
    r -= 15;				// -15 -> 16
    r += 100;

    return r;

#endif  // !FAKE_SAMPLE
}


#if defined( ADD_INPUT )
//
// 
//
int
update( XtPointer w, int *source, XtInputId *id )
{

}
#endif  // ADD_INPUT


#if !defined( XADC )

// Callback for the Quit button: simply quits the application.
void Quit(w, client_data, call_data)
Widget w;
caddr_t client_data, call_data;
{
   XtDestroyWidget(TopLevel);
   exit(0);
}


// Callback for the Reset button: resets Lcd1's counter value to 0.
void Reset_lcd(w, lcd, call_data)
Widget w;
Widget lcd; 
caddr_t call_data;
{
XcVType value;

   value.lval = 0L;
   XcLcdUpdateValue(lcd, &value);

}

#endif // !XADC
