//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: xremote.c,v 1.10 1995/11/05 04:36:10 smp Exp smp $";


#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

// Xlib headers
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

// Standard Xt headers
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

// Athena Widget Set headers
#if 0
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
#endif  // 0

// FWF Widget Set headers
#include <Xfwf/Board.h>
#include <Xfwf/Label.h>
#include <Xfwf/Button.h>
#include <Xfwf/PullDown.h>
#include <Xfwf/OptButton.h>
#include <Xfwf/TextMenu.h>
#include <Xfwf/RowCol.h>

// rc client & server hook routines
#include "util/misc/misc.hh"
#include "util/misc/namedstr.hh"
#include "util/misc/udgsocket.hh"
#include "util/misc/tokens.hh"
#include "rc/daemon/system.hh"
#include "rc/daemon/message.hh"
#include "rc/daemon/rcserver.hh"


#define USE_ACCELERATORS_NOT

#define SKIPWS(P)		while( isspace((*P)) ) ++P

// path for device specific resource files
#define RES_BASE		BASE "/app-defaults/"

// magic label for system() objects
#define SYSTEM_ADDRESS		"SYSTEM"

#define BUTTON_RES_TMPL		"%s_b%02d"
#define MESSAGE_RES_TMPL	"%s_m%02d"

// maximum number of buttons per device
#define NUM_BUTTONS		100

// maximum number of devices
#define NUM_DEVICES		16
#define DEFAULT_DEVICE		"1"

// instance & class names for X resource access
const static char* appName = "xremote";
const static char* appClass = "Xremote";



//
// application resource setup
//
#define	XtNxDevice	"xDevice"
#define	XtCXDevice	"XDevice"
#define	XtNxDeviceList	"xDeviceList"
#define	XtCXDeviceList	"XDeviceList"

#define	XtNxVerbal	"xVerbal"
#define	XtCXVerbal	"XVerbal"

typedef struct {
    char*	xDevice;
    char*	xDeviceList;
    int		xVerbal;
} appResources_t, *appResourcesPtr;

static appResources_t	appResources;

static XtResource	resources[] = {
#define offset(field)		XtOffset(appResourcesPtr, field)

	{ XtNxDevice, XtCXDevice, XtRString, sizeof(char*),
		offset(xDevice), XtRImmediate, (XtPointer)DEFAULT_DEVICE },

	{ XtNxDeviceList, XtCXDeviceList, XtRString, sizeof(char*),
		offset(xDeviceList), XtRImmediate, (XtPointer)NULL },

	{ XtNxVerbal, XtCXVerbal, XtRInt, sizeof(int), 
		offset(xVerbal), XtRImmediate, (XtPointer)0 },

#undef offset
};

static XrmOptionDescRec options[] = {
	{ "-d",	".xDevice",	XrmoptionSepArg, (caddr_t) DEFAULT_DEVICE },
	{ "-l",	".xDeviceList",	XrmoptionSepArg, (caddr_t) NULL },
	{ "-v",	".xVerbal",	XrmoptionSepArg, (caddr_t) 0 },
};


//
// global variables
//
static int	xDevice = 0;
static int	xVerbal = 0;

//
// data for pulldown menu
//
typedef struct {
    int		index;		// index in array
    char*	label;		// ascii label
    char*	resFile;	// resource file
} deviceTag;


// label to x device associations for device menu
deviceTag deviceList[ NUM_DEVICES + 1 ] = {
    { -1, NULL,	NULL }
};

typedef struct {
    WidgetList	w;		// an array of Widget
    Action**	a;		// an array of Action
    int		c;		// count of each array
    int		x;		// width
    int		y;		// heigth
} device;

device		devices[ NUM_DEVICES ];

// button data
typedef struct {
    Widget	w;
    Action*	a;
} button;

button		buttons[ NUM_BUTTONS ];

Widget		topLevel;

// connection stuff
static NamedStr	defaults;
static Action*	addMessage( char* args );


// callback functions
static void optionsCB( Widget, XtPointer, XtPointer );
static void deviceCB( Widget, XtPointer, XtPointer );
static void buttonCB( Widget w, XtPointer clientData, XtPointer callData );

// other local functions

static int	buildDeviceList( appResourcesPtr appRes, deviceTag* list );
static int	loadDeviceResources( Widget w, deviceTag* list );
static int	buildDeviceMenu( Widget menu, deviceTag* list, int& index );
static void	setActiveMenu( Widget menu, deviceTag* list );

static int	buildButtonBoard( Widget ButtonBoard, deviceTag* deviceList );
static int	createButton( const char* deviceName, Widget parent,
			      int number, XtCallbackProc callBack );

//
// 
//
int
main( int argc, char **argv )
{
    XtAppContext	appContext;
    Widget		mainBox;
    Widget		mainMenu;
    Widget		optionsMenu;
    Widget		deviceMenu;
    Widget		ButtonBoard;

    // get the socket paths from the RC defaults file
    if ( defaults.readStrings( RCD_STRINGS ) == ERROR )
    {
	cerr << "couldn't load file: %s" << RCD_STRINGS << endl;
	exit( 1 );
    }

    // create top level shell
    topLevel = XtVaAppInitialize( &appContext, appClass,
				  options, XtNumber(options),
				  &argc, argv, NULL, NULL );

    // process remaining commandline args
    XtGetApplicationResources( topLevel,
			       &appResources, resources, XtNumber(resources),
			       0, 0 );

    // process appResources
    xVerbal = appResources.xVerbal;

    // desired default device
    xDevice = atoi( appResources.xDevice ) - 1;

    if ( buildDeviceList( &appResources, deviceList ) == ERROR )
    {
	cerr << "BAD device list, missing Xremote app-default?" << endl;
	exit( 1 );
    }

    // load device specific rdbs
    loadDeviceResources( topLevel, deviceList );

    // the top level container
    mainBox = XtVaCreateManagedWidget( "mainBox", xfwfBoardWidgetClass,
				       topLevel, NULL );

    //
    // build the menu bar
    //
    mainMenu = XtVaCreateManagedWidget( "menu", xfwfBoardWidgetClass,
					mainBox, NULL );

    optionsMenu = XtVaCreateManagedWidget( "oMenu", xfwfPullDownWidgetClass,
					   mainMenu, NULL );

    XtAddCallback( optionsMenu, XtNactivate, optionsCB, NULL );

    // create the device menu
    deviceMenu = XtVaCreateManagedWidget( "dMenu", xfwfOptionButtonWidgetClass,
					  mainMenu, NULL );

    XtAddCallback( deviceMenu, XtNactivate, deviceCB, NULL);

    // build device menu
    if ( buildDeviceMenu( deviceMenu, deviceList, xDevice ) == ERROR )
    {
	cerr << "BAD menu data" << endl;
	exit( 1 );
    }

#if defined( USE_ACCELERATORS )
    XtInstallAllAccelerators( topLevel, topLevel );
    XtInstallAllAccelerators( optionsMenu, topLevel );
#endif  // USE_ACCELERATORS

    //
    // button container
    //
    ButtonBoard = XtVaCreateManagedWidget( "keys", xfwfBoardWidgetClass,
					   mainBox, NULL );

    // create the button arrays
    cerr << "Building buttons, be patient, this takes time...";
    buildButtonBoard( ButtonBoard, deviceList );
    cerr << "done!" << endl;

    // activate only those devices that built correctly
    setActiveMenu( deviceMenu, deviceList );

    // change size
    Arg args[ 2 ];
    int i = 0;
    XtSetArg( args[i], XtNwidth, devices[ xDevice ].x );  i++;
    XtSetArg( args[i], XtNheight, devices[ xDevice ].y );  i++;
    XtSetValues( topLevel, args, i );

    XtManageChildren( devices[ xDevice ].w, devices[ xDevice ].c );

    // make it all happen
    XtRealizeWidget( topLevel );

    // ...if only this was done with threads...
    XtAppMainLoop( appContext );
}


//
// 
//
static int
buildDeviceList( appResourcesPtr appRes, deviceTag* list )
{
    int index = 0;

    if ( appRes->xDeviceList == NULL ) return ERROR;

    // get string into a local buffer
    char* s = new char[ strlen(appRes->xDeviceList) + 1 ];
    strcpy( s, appRes->xDeviceList );

    // get the first label
    char* p = strtok( s, "," );
    SKIPWS( p );

    // process each of count possible values
    for ( int x = 0; x < NUM_DEVICES; ++x )
    {
	// set the index
	list[ x ].index = x;

	// assign the label
	list[ x ].label = p;

	// parse the resource file name
	if ( (p = strtok( NULL, "," )) == NULL )
	{
	    cerr << "buildDeviceList(), missing resource file" << endl;
	    return ERROR;
	}
	SKIPWS( p );

	// check for absolute or relative path
	if ( (*p == '/')
	     || ((p[0] == '.') && (p[1] == '/'))
	     || ((p[0] == '.') && (p[1] == '.') && (p[2] == '/')) )
	{
	    list[ x ].resFile = new char[ strlen( p ) + 1 ];
	    strcpy( list[ x ].resFile, p );
	}

	// no path, use default
	else
	{
	    list[ x ].resFile = new char[ strlen( p )
					  + strlen( RES_BASE ) + 1 ];
	    strcpy( list[ x ].resFile, RES_BASE );
	    strcat( list[ x ].resFile, p );
	}

	// get the next label
	if ( (p = strtok( NULL, "," )) == NULL ) break;
	SKIPWS( p );
    }

    // NULLify rest of list
    for ( ++x; x < NUM_DEVICES; ++x )
    {
	list[ x ].index = -1;
	list[ x ].label = NULL;
	list[ x ].resFile = NULL;
    }

    return OK;
}


//
// 
//
static int
loadDeviceResources( Widget w, deviceTag* list )
{
    XrmDatabase rdb = XtScreenDatabase( XtScreen( w ) );

    for ( int x = 0; x < NUM_DEVICES; ++x )
    {
	if ( list[ x ].label == NULL ) break;

	// attempt to load rdb for device
	const char* s = list[ x ].resFile;
	if ( XrmCombineFileDatabase( s, &rdb, True /**False*/ ) == 0 )
	{
	    list[ x ].index = -1;
	    list[ x ].resFile = NULL;
	    cerr << "resFile NOT found: " << s << endl;
	}
    }

    return OK;
}


//
// 
//
static int
buildDeviceMenu( Widget menu, deviceTag* list, int& index )
{
    int x, c;
    Arg args[ 10 ];

    // compute total length
    for ( c = 0, x = 0; x < NUM_DEVICES; ++x )
    {
	if ( list[ x ].label == NULL ) break;
	c += (strlen( list[ x ].label ) + 2);
    }
    ++c;

    // build menu
    char* s = new char[ c ];
    s[ 0 ] = '\0';
    for ( x = 0; x < NUM_DEVICES; ++x )
    {
	if ( list[ x ].label == NULL ) break;	// end of list
	strcat( s, list[ x ].label );
	strcat( s, "\n" );
    }

    // find a valid default entry
    if ( index == -1 ) index = 0;
    if ( list[ index ].index == -1 )
    {
	for ( x = 0; x < NUM_DEVICES; ++x )
	{
	    if ( list[ x ].index != -1 )
	    {
		index = x;
		break;
	    }
	}
	if ( x == NUM_DEVICES ) return ERROR;
    }

    // set the menu string and default label
    int i = 0;
    XtSetArg( args[i], XtNlabel, list[ index ].label );  i++;
    XtSetArg( args[i], XtNmenu, s );  i++;
    XtSetValues( menu, args, i);

    return OK;
}


//
// 
//
static void
setActiveMenu( Widget menu, deviceTag* list )
{
    // search list for (unNULLified) entries
    long active = 0x00000000;
    for ( int x = 0; x < NUM_DEVICES; ++x )
    {
	if ( list[ x ].index != -1 )		// valid entry
	{
	    active |= (1 << x);			// set bit in mask
	}
    }

    // find menu widget [created automatically by PullDown]
    Widget pw;
    Arg args[ 2 ];
    XtSetArg( args[ 0 ], XtNpopup, &pw );
    XtGetValues( menu, args, 1 );

    // set the 'active button list'
    XtSetArg( args[ 0 ], XtNactive, active );
    XtSetValues( pw, args, 1 );
}


static int
buildButtonBoard( Widget ButtonBoard, deviceTag* list )
{
    int count;
    int last;

    for ( int x = 0; x < NUM_DEVICES; ++x )
    {
	// end of list
	if ( list[ x ].label == NULL ) break;

	// handle sparse lists
	if ( list[ x ].index == -1 )
	{
	    devices[ x ].w = NULL;
	    devices[ x ].a = NULL;
	    continue;
	}
	/// cerr << "device: '" << list[ x ].label << "'" << endl;

	// clear the temp array
	for ( int xx = 0; xx < NUM_BUTTONS; ++xx )
	{
	    buttons[ xx ].w = NULL;
	    buttons[ xx ].a = NULL;
	}
	count = 0;
	last = -1;

	// create individual buttons
	const char* label = strrchr( list[ x ].resFile, '/' );
	if ( label == NULL ) label = list[ x ].resFile;
	else ++label;

	// build each defined button
	for ( xx = 0; xx < NUM_BUTTONS; ++xx )
	{
	    switch ( createButton( label,
				   ButtonBoard, xx, buttonCB ) )
	    {
	    case TRUE:
		++count;
		break;

	    case -1:
		last = xx;
		xx = NUM_BUTTONS;
		break;

	    case FALSE:
		break;
	    }
	}
	if ( last == -1 ) last = NUM_BUTTONS;

	// record count
	if ( (devices[ x ].c = count) == 0 )
	{
	    devices[ x ].w = NULL;
	    devices[ x ].a = NULL;
	    list[ x ].index = -1;
	    continue;
	}

	// build dense array of widgets
	int yy;
	devices[ x ].w = new Widget[ count ];
	for ( yy = 0, xx = 0; xx < last; ++xx )
	{
	    if ( buttons[ xx ].w )
	    {
		devices[ x ].w[ yy++ ] = buttons[ xx ].w;
	    }
	}

	if ( yy != count )
	{
	    cerr << "buildButtonBoard(), w != count" << endl;
	    cerr << "w: " << yy << "count: " << count << endl;
	    exit( 1 );
	}

	// build dense array of objects
	devices[ x ].a = new Action*[ count ];
	for ( yy = 0, xx = 0; xx < last; ++xx )
	{
	    if ( buttons[ xx ].a )
	    {
		// setup callback
		XtAddCallback( buttons[ xx ].w, XtNactivate,
			       buttonCB, (XtPointer)yy );
		// pack the object
		devices[ x ].a[ yy++ ] = buttons[ xx ].a;
	    }
	}

	if ( yy != count )
	{
	    cerr << "buildButtonBoard(), a != count" << endl;
	    cerr << "a: " << yy << "count: " << count << endl;
	    exit( 1 );
	}

	//
	char name[ 80 ];
	sprintf( name, "%s_size", label );

	// create an instance of it
	Widget w;
	w = XtVaCreateWidget( name, xfwfLabelWidgetClass, ButtonBoard, NULL );

	// get its label
	String message;
	Arg args[ 1 ];
	XtSetArg( args[ 0 ], XtNlabel, &message );
	XtGetValues( w, args, 1 );

	// delete label widget
	XtDestroyWidget( w );
	sscanf( message, "%dx%d", &devices[ x ].x, &devices[ x ].y );
	XtFree( message );
	/// cerr << "x: " << devices[x].x << ", y: " << devices[x].y << endl;
    }

    return OK;
}


//
// create push buttons
//
static int
createButton( const char* deviceName, Widget parent,
	       int number, XtCallbackProc callBack )
{
    // create a name for the message
    char name[ 256 ];
    sprintf( name, MESSAGE_RES_TMPL, deviceName, number );

#if defined( USE_XRMGETRESOURCE )

    would this be better???

#else  // USE_XRMGETRESOURCE

    // create an instance of it
    Widget w;
    w = XtVaCreateWidget( name, xfwfLabelWidgetClass, parent, NULL );

    // get its label
    String message;
    Arg args[ 1 ];
    XtSetArg( args[ 0 ], XtNlabel, &message );
    XtGetValues( w, args, 1 );

    // delete label widget
    XtDestroyWidget( w );

#endif // USE_XRMGETRESOURCE

    // test it
    if ( message )
    {
	Action* action = addMessage( message );
	XtFree( message );

	// found last message
	if ( action == NULL ) return (-1);

	// record buttons action
	buttons[ number ].a = action;

	// create the actual button
	sprintf( name, BUTTON_RES_TMPL, deviceName, number );
	w = XtVaCreateWidget( name, xfwfButtonWidgetClass, parent, NULL );

	// record widget
	buttons[ number ].w = w;
	return TRUE;
    }

    else
    {
	buttons[ number ].w = NULL;
	buttons[ number ].a = NULL;
	return FALSE;
    }
}


//
// options menu callback
//
static void
optionsCB( Widget w, XtPointer clientData, XtPointer callData )
{
    XfwfTextMenuData& data = *(XfwfTextMenuData*)callData;

    switch ( data.n )
    {
    case 0:
	xVerbal = (xVerbal ? 0 : 1);
	break;
    case 1:
	exit(0);
    default:
	break;
    }
}


//
// device menu callback
//
static void
deviceCB( Widget w, XtPointer clientData, XtPointer callData )
{
    XfwfTextMenuData& data = *(XfwfTextMenuData*)callData;

    // check for invalid device
    if ( deviceList[ data.n ].index == -1 )
    {
	return;
    }

    // unmap the current device buttons
    XtUnmanageChildren( devices[ xDevice ].w, devices[ xDevice ].c );

    // get the new device type
    xDevice = deviceList[ data.n ].index;

    // change size
    Arg args[ 2 ];
    int i = 0;
    XtSetArg( args[i], XtNwidth, devices[ xDevice ].x );  i++;
    XtSetArg( args[i], XtNheight, devices[ xDevice ].y );  i++;
    XtSetValues( topLevel, args, i );

    // map the new device buttons
    XtManageChildren( devices[ xDevice ].w, devices[ xDevice ].c );
}


//
// button pressed callback
//
static void
buttonCB( Widget w, XtPointer clientData, XtPointer callData )
{
    devices[ xDevice ].a[ (int)clientData ]->action();
}


//
// create an action object for the button
//
static Action*
addMessage( char* msg )
{
    Action*	object;
    const char*	myPath;
    const char*	itsPath;

    // make a working copy
    char* args = new char[ strlen( msg ) + 1 ];
    strcpy( args, msg );

    // break string up into address & message
    Tokens& tokens = *new Tokens( (char*)args, 2 );
    int argc = tokens.count();

    // check for end of list
    if ( strcmp( tokens[ 0 ], "MSG_END" ) == 0 )
    {
	return NULL;
    }

    // create a System action
    else if ( strcmp( tokens[ 0 ], SYSTEM_ADDRESS ) == 0 )
    {
	object = new System( tokens[ 1 ] );
    }

    // create an Message action
    else
    {
	myPath = defaults.getString( "RCD_DG_TMP" );
	itsPath = defaults.getString( tokens[ 0 ] );

	if ( (!myPath) || (!itsPath) )
	{
	    delete &tokens;
	    cerr << "couldn't get socket paths\n";
	    return (Action*)ERROR;
	}

	object = new Message( tokens[ 1 ], myPath, itsPath );
    }

    // return memory
    delete &tokens;
    delete args;

    return object;
}
