//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

// $Id: keymap.hh,v 1.2 1996/03/02 08:01:47 smp Exp smp $

//
// header for keymap.cc
//

#ifndef _KEYMAP_H
#define _KEYMAP_H


#define KD_NAME(X)	(keyData[(X)].name)
#define KD_LENGTH(X)	(keyData[(X)].length)
#define KD_DATA(X)	((u_char*)keyData[(X)].data)
#define KD_LSIZE	(sizeof(int))
#define KD_DSIZE	(sizeof(u_char))

#define X10_DATA(X,Y)	(((char*)keyData[(X)].data)[(Y)])
#define X10_DTYPE	u_char
#define X10_DSIZE	(sizeof(u_char))

#define IR_DATA(X,Y)	((((short*)keyData[(X)].data)[(Y)])
#define IR_DTYPE	short
#define IR_DSIZE	(sizeof(short))


typedef struct {
    char*	name;
    int		length;
    void*	data;
} keydata_t;


class KeyMap {

 public:

    const IR_KEY_NUM = 512;
    const X10_KEY_NUM = 512;

    KeyMap()	{ keyCount = 0; };

    int	readMap( char* );
    int	writeMap( char* );

    int	getKey( char*, IR_DTYPE* );			// IR
    int	getKey( char*, X10_DTYPE&, X10_DTYPE& );	// X-10
    int	putKey( char*, IR_DTYPE* );			// IR
    int	putKey( char*, X10_DTYPE, X10_DTYPE );		// X-10

    int getName( const char*&, X10_DTYPE, X10_DTYPE );	// X-10

    //
    // generic, ie CORRECT versions, above is ONLY for legacy reasons
    //

    // COPY keys data to data
    int	getKey( char* name, void* data );			

    // return POINTER to data and data length
    int getKey( char* name, void*& data, int& length );

    // record name + data[ length ]
    int	putKey( char* name, void* data, int length );

    // 
    int getName( const char*& name, u_char* data );


 private:

    int		keyCount;
    keydata_t*	keyData;

};


#endif  // _KEYMAP_H
