//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: namedstr.cc,v 1.3 1996/03/02 07:50:09 smp Exp smp $";

//
// namedstr.cc
//


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "misc.hh"
#include "namedstr.hh"


static int	strCmp( const void* p1, const void* p2 );

int
NamedStr::readStrings( const char* file )
{
    int		x;
    char*	p1;
    char*	p2;
    char	str[ 255 ];
    FILE*	fp;

    // open file
    if ( (fp = fopen( file, "r" )) == NULL ) return ERROR;

    // count the number of entries
    while ( fgets( str, 255, fp ) )
    {
	// look for comments
	for ( p1 = (char *)str; isspace( *p1 ); ++p1 )  ;     // remove WS
	if ( (*p1 == '\0') || (*p1 == '#') ) continue;	      // skip comments
	++_strCount;
    }
    rewind( fp );

    // read the entries
    _strData = new STRDATA[ _strCount ];
    for ( x = 0; x < _strCount;  )
    {
	fgets( str, 255, fp );

	// look for comments
	for ( p1 = (char *)str; isspace( *p1 ); ++p1 )  ;     // remove WS
	if ( (*p1 == '\0') || (*p1 == '#') ) continue;	      // skip comments
	if ( p1 = strrchr( str, '\n' ) ) *p1 = '\0';	      // remove newline

	// break into label & value
	if ( !(p2 = strchr( str, ':' )) )
	{
	    do {
		delete _strData[ x ].label;
		delete _strData[ x ].value;
	    } while ( x-- );
	    delete _strData;
	    fclose( fp );
	    return ERROR;
	}

	*p2++ = '\0';					// terminate label
	for (  ; isspace( *p2 ); ++p2 )  ;		// remove WS

	// record label & string
	_strData[ x ].label = new char[ strlen( str ) + 1 ];
	strcpy( _strData[ x ].label, str );
	_strData[ x ].value = new char[ strlen( p2 ) + 1 ];
	strcpy( _strData[ x ].value, p2 );
	++x;
    }

    fclose( fp );

    // sort the data
    qsort( (void *)_strData, _strCount, sizeof( STRDATA ), strCmp );

    return OK;
}


const char*
NamedStr::getString( const char* s )
{
    STRDATA	target;
    STRDATA*	p;

    target.label = (char*)s;
    p = (STRDATA *)bsearch( &target, (void *)_strData, _strCount,
			   sizeof( STRDATA ), strCmp );

    return p ? (p->value) : (const char*)NULL;
}


static int
strCmp( const void* p1, const void* p2 )
{
    return strcmp( (*(STRDATA*)p1).label, (*(STRDATA*)p2).label );
}
