//
// Copyright 1996 Steve Passe
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHOR (Steve Passe)
// is not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHOR makes no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHOR
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// AUTHOR:     Steve Passe <smp@csn.net>
//

static char rcsid[] = "$Id: procfork.cc,v 1.1 1996/02/05 06:29:58 smp Exp smp $";

#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>

#include "procfork.hh"

//
// 
//
ProcFork::ProcFork( void )
{
    int pid;
    int PPipe[ 2 ], CPipe[ 2 ];

    // open reader/writer communication pipes
    if ( (pipe( PPipe ) < 0) || (pipe( CPipe ) < 0) )
    {
	_create_error = 1;
	return;
    }

    // fork() the child process
    if ( (pid = fork()) < 0 )
    {
	_create_error = 2;
	return;
    }

    else if ( pid == 0 )			// we are the child
    {
	// flag ourself as the child
	_amParent = 0;
	_amChild = 1;

	// close the ends we won't be using
	close( PPipe[ 1 ] );
	close( CPipe[ 0 ] );

	// handles for our I/O
	_input = PPipe[ 0 ];
	_output = CPipe[ 1 ];

	// record parent's PID
	_parentPID = getppid();
	_childPID = -1;
    }

    else					// we are the parent
    {
	// flag ourself as the parent
	_amParent = 1;
	_amChild = 0;

	// close the ends we won't be using
	close( PPipe[ 0 ] );
	close( CPipe[ 1 ] );

	// handles for our I/O
	_input = CPipe[ 0 ];
	_output = PPipe[ 1 ];

	// record child's PID
	_parentPID = -1;
	_childPID = pid;
    }

    _create_error = 0;
}
