//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: udgsocket.cc,v 1.5 1996/03/31 02:22:50 smp Exp smp $";


//
// udgsocket.cc
//

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

#include "misc.hh"
#include "udgsocket.hh"


UnixDGSocket::UnixDGSocket( char* myPath, char* destPath )
{
    // create a socket
    if ( (_socketFD = socket( AF_UNIX, SOCK_DGRAM, 0 )) < 0 )
    {
	_socketFD = ERROR;
	perror( "socket open()" );
	return;
    }

    // prepare our address structure
    unlink( myPath );

    bzero( (char *)&_myAddr, sizeof( _myAddr ) );
    _myAddr.addr.sun_family = AF_UNIX;
    strcpy( _myAddr.addr.sun_path, myPath );

    // a bit of magic, will go away later
    if ( strncmp( UDG_TMPADDR, _myAddr.addr.sun_path, 14 ) == 0 )
    {
	mktemp( _myAddr.addr.sun_path );
    }

    _myAddr.len = sizeof( _myAddr.addr.sun_family )
      + strlen( _myAddr.addr.sun_path ) + 1;

    // bind the socket to it
    if ( bind( _socketFD, (struct sockaddr *)&_myAddr.addr, _myAddr.len ) < 0 )
    {
	close( _socketFD );
	_socketFD = ERROR;
	perror( "socket bind()" );
	return;
    }

    // prepare the destination address structure for use by sendto()
    bzero( (char *)&_destAddr, sizeof( _destAddr ) );

    // clients do this
    if ( destPath )
    {
	_destAddr.addr.sun_family = AF_UNIX;
	strcpy( _destAddr.addr.sun_path, destPath );
	_destAddr.len = sizeof( _destAddr.addr.sun_family )
	  + strlen( _destAddr.addr.sun_path ) + 1;
    }

    //  servers do this
    else
    {
	_destAddr.len = 0;
    }
}


UnixDGSocket::~UnixDGSocket( void )
{
    close( _socketFD );
    unlink( _myAddr.addr.sun_path );
}


int
UnixDGSocket::send( const char* s, int len, UNIXSOCKADDR* da )
{
    // caller is providing a destination, usually a server
    if ( da )
    {
#if 0
	/// FIXME: why doesnt this seem to work (or be necessary)???
	da->len = sizeof( da->addr );
	/// da->len = SUN_LEN( &da->addr );
#endif
    	return sendto( _socketFD, s, len, 0,
		      (struct sockaddr*)&(da->addr), da->len );
    }

    // caller wants to use its member data destination address, ie., a client
    else
    {
    	return sendto( _socketFD, s, len, 0,
		      (struct sockaddr*)&(_destAddr.addr), _destAddr.len );
    }
}


int
UnixDGSocket::recv( char* r, UNIXSOCKADDR* ra )
{
    int	cnt;

    // server mode, save a return address
    if ( ra )
    {
	// for server mode recv() needs to know the size
	/// good only for UDG, (because *ra is UDG)
	ra->len = sizeof( ra->addr );
	cnt = recvfrom( _socketFD, r, MAX_DG_MSG, 0,
		       (struct sockaddr*)&(ra->addr), &(ra->len) );
    }

    // client mode, ignore return address
    else
    {
	cnt = recvfrom( _socketFD, r, MAX_DG_MSG, 0,
		       (struct sockaddr*)0, (int*)0 );
    }
    if ( cnt < 0 )
    {
	perror( "recvUDG()" );
	return ERROR;
    }

    return cnt;
}


//
// 
//
int
UnixDGSocket::makeAddress( UNIXSOCKADDR& addr, const char* path )
{
    strcpy( addr.addr.sun_path, path );
    addr.addr.sun_len = SUN_LEN( &addr.addr );
    addr.addr.sun_family = AF_UNIX;
    addr.len = SUN_LEN( &addr.addr );
}

