//
// Copyright 1993,1994,1995 Steve Passe and Paul Crook
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for any purpose and without fee is hereby granted, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of AUTHORS (Steve Passe and Paul Crook)
// are not be used in advertising or publicity pertaining to distribution of
// the software without specific, written prior permission.  AUTHORS make no
// representations about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
// AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
// BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
// OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
// CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// 1995 Revision
// AUTHORS:     Steve Passe <smp@csn.net>
//              Paul Crook
//

static char rcsid[] = "$Id: voicelib.cc,v 1.2 1996/03/02 07:50:09 smp Exp smp $";


//
// voicelib.cc, misc voice functions
//

#include <termios.h>
#include <stdio.h>
#include <string.h>
#include <sys/file.h>

#include "misc.hh"
#include "voicelib.hh"


FILE *
voiceFopen( int flag )
{
    int			fd;
    FILE		*fp;
    struct termios	newTerm;

    // open NONBLOCKing to allow setup of clocal if necessary
    if ( (fd = open( VOICE, O_RDWR|O_NONBLOCK )) == -1 ) return (FILE *)ERROR;

    if ( flag )
    {
	if ( tcgetattr( fd, &newTerm ) == ERROR ) return (FILE *)ERROR;

	newTerm.c_cc[VMIN] = 1;
	newTerm.c_cc[VTIME] = 0;
	newTerm.c_iflag &= ~( BRKINT|
			     IGNPAR|PARMRK|INPCK|ISTRIP|
			     INLCR|IGNCR|ICRNL|IXON );

	newTerm.c_oflag &= ~(OPOST);

	newTerm.c_cflag &= ~( CSIZE|CSTOPB|HUPCL|PARENB );
	newTerm.c_cflag |= ( CLOCAL|CREAD|CS8 );

	newTerm.c_lflag &= ~( ECHO|ECHOE|ECHOK|ECHONL|
			     ISIG|
			     ICANON|IEXTEN|
			     NOFLSH|
			     TOSTOP);

	if ( cfsetospeed( &newTerm, B9600 ) == ERROR ) return (FILE *)ERROR;
	if ( cfsetispeed( &newTerm, B9600 ) == ERROR ) return (FILE *)ERROR;
	if ( tcsetattr(fd, TCSANOW, &newTerm) == ERROR ) return (FILE *)ERROR;
    }

    if ( (fp = fdopen( fd, "r+" )) == NULL ) return (FILE *)ERROR;
    setbuf( fp, NULL );				// no buffer of output

    return fp;
}


static char *numword[] = {
    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
    "ten", "eleven", "twelve", "thirteen", "fourteen",
    "fifteen", "sixteen", "seventeen", "eighteen", "nineteen",
    "twenty", "thirty", "forty", "fifty",
    "sixty", "seventy", "eighty", "ninety"
};

int
ssayn( char *s, int n )
{
    if ( n < 20 )
    {
	strcpy( s, numword[ n ] );
	return OK;
    }
    else if ( n < 100 )
    {
	strcpy( s, numword[ (n/10) + 18 ] );
	if ( n %= 10 ) strcat( s, numword[ n ] );
	return OK;
    }
    else
    {
	return ERROR;
    }
}
